//**************************************************************************
//                                   Petr Smolik
//**************************************************************************
#include <stdio.h>
#include "usbd11.h"
#include "i2c.h"

//**************************************************************************
// Writes a command unsigned char to the D11 command register and then
// reads or writes byCount unsigned chars to/from the D11 data register.
// Data is obtained from or written to pbyData.
// A write is performed when byRead is zero, otherwise a read is performed
unsigned char D11_CmdDataI2C(unsigned char byCmd, unsigned char * pbyData,
                             unsigned char byCount, unsigned char byRead) {
  I2C_Write(D11_REG_CMD, &byCmd, 1, I2C_WRMOD_NORMAL);        // Wr Cmd
  if(byCount) {
    if(byRead) I2C_Read(D11_REG_DATA_READ, pbyData, byCount);      // Rd Data
    else I2C_Write(D11_REG_DATA_WRITE, pbyData, byCount,I2C_WRMOD_NORMAL);// Wr Data
  }
  return byCount;
}

//**************************************************************************
void D11_CfgInit(void) {
  // POR Routine (from freaking web site app. note)
  unsigned char Data[2];
  
  // disable the hub (D11 is an H11 without the HUB pins)
  Data[0] = 0x00;
  D11_CmdDataI2C(D11_CMD_POR_INIT, Data, 1, WR_DATA);
  // set device address to 0 and enable function
  Data[0] = 0x80;
  D11_CmdDataI2C(D11_CMD_SET_ADDRESS, Data, 1, WR_DATA);
  // disable hub endpoints, enable device function endpoints
  Data[0] = 0x02;
  D11_CmdDataI2C(D11_CMD_SET_EPEN, Data, 1, WR_DATA);
  // configure USB for connection
  Data[0] = 0x97;
  Data[1] = 0x03;
  D11_CmdDataI2C(D11_CMD_SET_MODE, Data, 2, WR_DATA);
}

//**************************************************************************
// configure USB for disconnect
/*void D11_CfgClr(void) {
  unsigned char Data[2] = { 0x87, 0x0B };

  // soft - disconnect
  D11_CmdDataI2C(D11_CMD_SET_MODE, Data, 2, WR_DATA);
  // set device address to 0 and disable function
  Data[0] = 0x00;
  D11_CmdDataI2C(D11_CMD_SET_ADDRESS, Data, 1, WR_DATA);
  // disable hub and device endpoints (D11 is an H11 without the hub pins)
  Data[0] = 0x00;
  D11_CmdDataI2C(D11_CMD_SET_EPEN, Data, 1, WR_DATA);
  D11_GetIrq();
  D11_GetLtStat(D11_EP0IN_IDX);    // Get Last Transaction Status
  D11_GetLtStat(D11_EP0OUT_IDX);
  D11_GetLtStat(D11_EP1IN_IDX);
  D11_GetLtStat(D11_EP1OUT_IDX);
  D11_GetLtStat(D11_EP2IN_IDX);
  D11_GetLtStat(D11_EP2OUT_IDX);
  D11_GetLtStat(D11_EP3IN_IDX);
  D11_GetLtStat(D11_EP3OUT_IDX);
} */

//**************************************************************************
// get last transaction status
unsigned char D11_GetLtStat(unsigned char byEpIdx) {
  unsigned char byLtStat;

  D11_CmdDataI2C(D11_CMD_GET_LTSTAT + byEpIdx, &byLtStat, 1, RD_DATA);
  return byLtStat;
}

//**************************************************************************
// get endpoint status
unsigned char D11_GetEpStat(unsigned char byEpIdx) {
  unsigned char byEpStat;

  D11_CmdDataI2C(D11_CMD_GET_EPSTAT + byEpIdx, &byEpStat, 1, RD_DATA);
  return byEpStat;
}

//**************************************************************************
// set endpoint status to stall condition
void D11_SetEpStall(unsigned char byEpIdx) {
  unsigned char byEpStall = 0x01;

  D11_CmdDataI2C(D11_CMD_SET_EPSTAT + byEpIdx, &byEpStall, 1, WR_DATA);
}

//**************************************************************************
// set endpoint status to stall condition
void D11_ClrEpStall(unsigned char byEpIdx) {
  unsigned char byEpStall = 0x00;

  D11_CmdDataI2C(D11_CMD_SET_EPSTAT + byEpIdx, &byEpStall, 1, WR_DATA);
}

//**************************************************************************
void D11_AckSetup() {
  unsigned char code Cmds[5] = {
    D11_CMD_EP_SELECT+2,
    D11_CMD_ACK_SETUP,
    D11_CMD_CLEAR_BUF,
    D11_CMD_EP_SELECT+3,
    D11_CMD_ACK_SETUP
  };

  I2C_Write(D11_REG_CMD, Cmds, 5, I2C_WRMOD_FLASH);
}

//**************************************************************************
// read USB interrupt register
unsigned D11_GetIrq(void) {
  unsigned char wIrqData[2];

  D11_CmdDataI2C(D11_CMD_GET_IRQ, wIrqData, 2, RD_DATA);
  return wIrqData[0]+wIrqData[1]*256;
}


//**************************************************************************
unsigned char D11_ClrEp(unsigned char byEpIdx) {
  unsigned char BufHeader[2];		// unsigned char 0  = reserved, unsigned char 1 = unsigned char count
  unsigned char byFull = 0;

  D11_CmdDataI2C(D11_CMD_EP_SELECT + byEpIdx, &byFull, 1, RD_DATA);  // ep
  if(byFull) {
    D11_CmdDataI2C(D11_CMD_READ_BUF, BufHeader, 2, RD_DATA); // buf header
    D11_CmdDataI2C(D11_CMD_CLEAR_BUF, 0, 0, 0);	 	     // clear buf
  }
  return BufHeader[1];
}


//**************************************************************************
unsigned char D11_RdEp(unsigned char * pbyData, unsigned char byEpIdx) {
  unsigned char BufHeader[2];	       	// unsigned char 0  = reserved, unsigned char 1 = unsigned char count
  unsigned char byFull = 0;

  BufHeader[1]=0;
  D11_CmdDataI2C(D11_CMD_EP_SELECT + byEpIdx, &byFull, 1, RD_DATA);	// ep
  if(byFull) {
    D11_CmdDataI2C(D11_CMD_READ_BUF, BufHeader, 2, RD_DATA);	// buf header
    if(BufHeader[1]) {
      // buffer contains data, read data from buffer
      D11_CmdDataI2C(D11_CMD_READ_BUF, pbyData, BufHeader[1], RD_DATA);
    }
    D11_CmdDataI2C(D11_CMD_CLEAR_BUF, 0, 0, 0); 		// clear buf
  }
  return BufHeader[1];
}

//**************************************************************************
void D11_WrEp(unsigned char byEpIdx, unsigned char * pbyData, unsigned char byCount,
              unsigned char flash) {
  unsigned char BufHeader[2];

  BufHeader[0] = 0x00;			  // reserved always write a zero
  BufHeader[1] = byCount;	    	  // valid unsigned char count for data buffer
  D11_CmdDataI2C(D11_CMD_EP_SELECT + byEpIdx, 0, 0, 0);		  // endpoint
  D11_CmdDataI2C(D11_CMD_WRITE_BUF, BufHeader, 2, WR_DATA);	  // buf header
  if(byCount) {
    if (!flash) {
      D11_CmdDataI2C(D11_CMD_WRITE_BUF, pbyData, byCount, WR_DATA); // buf data
    } else {
      D11_CmdDataI2C(D11_CMD_WRITE_BUF, 0, 0, WR_DATA); // buf data
      I2C_Write(D11_REG_DATA_WRITE, (unsigned char code *)pbyData, byCount,I2C_WRMOD_FLASH);// Wr Data
    }
  }
  D11_CmdDataI2C(D11_CMD_VALID_BUF, 0, 0, 0); 		         	  // validate
}

//**************************************************************************
void D11_TxNull(unsigned char byEpIdx) {
  unsigned char byCmd;
  unsigned char BufHeader[2] = { 0x00, 0x00 };

  byCmd = D11_CMD_EP_SELECT + byEpIdx;
  I2C_Write(D11_REG_CMD, &byCmd, 1,I2C_WRMOD_NORMAL);          // Select Ep
  D11_CmdDataI2C(D11_CMD_WRITE_BUF, BufHeader, 2, WR_DATA);    // buf header
  byCmd = D11_CMD_VALID_BUF;
  I2C_Write(D11_REG_CMD, &byCmd, 1,I2C_WRMOD_NORMAL);          // Valid Buf
}
