/***************************************************************************/
/*                                                                         */
/*                            Petr Smolik (c) 2001                         */
/*                                                                         */
/***************************************************************************/
#include <stdio.h>
#include <intrins.h>
#include <89c51rd2.H>
#include "i2c.h"

#define I2CWAIT		_nop_(),_nop_(),_nop_(),_nop_(),_nop_(),\
			_nop_(),_nop_(),_nop_(),_nop_(),_nop_(),\
			_nop_(),_nop_(),_nop_(),_nop_(),_nop_()
     		         // 6cycle wait time at 14.7456MHz

sbit SDAin  = 0x90;         // P1.0
sbit SDAout = 0x91;        // P1.1
sbit SCLout = 0xB4;        // P3.4

/***************************************************************************/
bit Si2c_in(unsigned char *value,bit ack ) {
  unsigned char i = 8, b=0;

  SDAout = 1;
  do {
    I2CWAIT;
    SCLout = 1;
    I2CWAIT;
    b <<= 1;		// shift 0 in
    if( SDAin )
      b++;		// set to 1
    SCLout = 0;
  } while( --i );
  SDAout = ack;
  I2CWAIT;
  SCLout = 1;
  I2CWAIT;
  {
    bit ack = SDAin;
    SCLout = 0;
    *value=b;
    return ack;		// 1 = NACK, 0 = ACK
  }
}

/***************************************************************************/
bit Si2c_out(unsigned char b) {
  unsigned char i = 9;

  do {
    SCLout = 0;
    SDAout = b & 0x80;	// SDAout = boolean
    b = (b << 1) + 1;
    I2CWAIT;
    SCLout = 1;
    I2CWAIT;
  } while( --i );
  {
    bit ack = SDAin;
    SCLout = 0;
    return ack;		// 1 = NACK, 0 = ACK
  }
}

/***************************************************************************/
bit Si2c_stop(void){
  SDAout = 0;
  I2CWAIT;
  SCLout = 1;
  I2CWAIT;
  SDAout = 1;
  _nop_();		// time to go high
  return ~SDAin;		// 1 = Error, SDA not high
}

/***************************************************************************/
void Si2c_start(void){
  SDAout = 1;
  SCLout = 1;
  I2CWAIT;
  SDAout = 0;
  I2CWAIT;
  SCLout = 0;
}

/***************************************************************************/
void Si2c_Init(void){
//nothing
}
/***************************************************************************/
bit I2C_Write(unsigned char adr,unsigned char *Data,unsigned char Count) {
  Si2c_start();
  if( Si2c_out(adr)) {
    Si2c_stop();
    return 0;
  }
  while (Count!=0) {
    if( Si2c_out(*Data)) {
      Si2c_stop();
      return 0;
    }
    Data++;
    Count--;
  }
  return ~Si2c_stop();
}

/***************************************************************************/
bit I2C_Read(unsigned char adr,unsigned char *Data,unsigned char Count) {
  Si2c_start();
  if( Si2c_out(adr)) {
    Si2c_stop();
    return 0;
  }
  while (Count!=0) {
    bit ack = Count-1;
    ack=~ack;
    Si2c_in(Data,ack);
    Data++;
    Count--;
  }
  return ~Si2c_stop();
}
