/**************************************************************************/
//                                   Petr Smolik
/**************************************************************************/
#include <reg1210.h>
#include <stdio.h>
#include <string.h>
#include <intrins.h>
#include "usb_spec.h"
#include "usbdefs.h"
#include "pdiusb.h"
#include "ulan.h"

int simple_ser_init(void);

sbit LED_RX = 0xB3;        // P3.3
sbit LED_TX = 0xB3;        // P3.3

sbit USB_IRQ = 0xB2;        // P3.2

#ifdef BOARD_BART
#define WITHOUT_ULAN
sbit LEDr = 0x90;        // P1.0
sbit LEDg = 0x91;        // P1.1
#endif /*BOARD_BART*/


extern bit uLF_RS,uLF_NB,uLF_NA;
ul_fd_t ul_fd;
xdata ul_msginfo msginfo;

#define TIMERH10ms       0xDC               //hodnoty predelicky pro casovac na 10ms
#define TIMERL10ms       0x00

unsigned char            *pSendBuffer0;
unsigned int             BytesToSend0,BytesToSend2,BytesToRec2;
unsigned char            DeviceAddress;
unsigned char            DeviceConfigured;
unsigned char            uLanConfigured;
unsigned char idata      timer_str;
unsigned char idata      timer_rx_off;
unsigned char idata      timer_tx_off;
unsigned char idata      timer_configured;

unsigned char xdata      uLanBuffRx[PDI_EP2_PACKET_SIZE];
unsigned char xdata      uLanBuffTx[PDI_EP2_PACKET_SIZE];

#define VENDOR_START_ULAN               0
#define VENDOR_STOP_ULAN                1
#define VENDOR_IS_RUNNING_ULAN          2
#define VENDOR_ARC_ON_ULAN              3
#define VENDOR_ARC_OFF_ULAN             4

/****************************************************************************/
//definice prusenich (casovac0)
#if 0
void timer0(void) interrupt 1 {
  IE0_=0;                                       //errata 89c51rd2  1)
  TH0=TIMERH10ms;TL0=TIMERL10ms;
  if (timer_rx_off!=0) timer_rx_off--;
  else LED_RX=1;
  if (timer_tx_off!=0) timer_tx_off--;
  else LED_TX=1;
  if (timer_str!=0) {
    timer_str=5;
    ul_str();
    /*if (_testbit_(ES)) {
      if (uLF_NA) {
        if ((uLF_NB) && (uLF_RS)) TI=1;
        uLF_NB=1;
      }
      uLF_NA=1;
      ES=1;
    }*/
  } else timer_str--;
  if (timer_configured!=0) timer_configured--;
  else {
    timer_configured=250;
    if (!DeviceConfigured) timer_configured=20;
    if ((!DeviceConfigured) || (!uLanConfigured)) {
      LED_RX=LED_TX=0;
      timer_rx_off=timer_tx_off=5;
    }
  }
}
#endif

/**************************************************************************/
void WriteBufferToEndPoint0() {
  if (BytesToSend0 == 0) pdiWriteEndpoint( PDI_EP0_IN, 0, 0);
  else {
    if ( BytesToSend0 >= PDI_EP0_PACKET_SIZE) {
      pdiWriteEndpoint( PDI_EP0_IN, PDI_EP0_PACKET_SIZE, pSendBuffer0);  //Data from flash
      BytesToSend0 -= PDI_EP0_PACKET_SIZE;
      pSendBuffer0 += PDI_EP0_PACKET_SIZE;
    } else {
      pdiWriteEndpoint( PDI_EP0_IN, BytesToSend0, pSendBuffer0);
      pSendBuffer0 += BytesToSend0;
      BytesToSend0 = 0;
    }
  }
}

/**************************************************************************/
void WriteBufferToEndPoint2() {
  if (BytesToSend2 != -1) {
    if (BytesToSend2 > PDI_EP2_PACKET_SIZE) {
      ul_read(ul_fd, uLanBuffTx, PDI_EP2_PACKET_SIZE);
      pdiWriteEndpoint( PDI_EP2_IN, PDI_EP2_PACKET_SIZE, uLanBuffTx); // zapis dat do EP
      BytesToSend2 -= PDI_EP2_PACKET_SIZE ;
    } else {
      ul_read(ul_fd, uLanBuffTx, BytesToSend2);
      ul_i_close(ul_fd);                           // uzavreni zpravy
      pdiWriteEndpoint( PDI_EP2_IN, BytesToSend2, uLanBuffTx); // zapis dat do EP
      BytesToSend2 = -1;
    }
  }
}

/**************************************************************************/
void GetDescriptor( PUSB_DEVICE_REQUEST SetupPkt) {
  switch((SetupPkt->wValue & 0xFF00) >> 8) {
    case USB_DESCRIPTOR_TYPE_DEVICE:{
      pSendBuffer0 = (unsigned char code*)&DeviceDescriptor;
      BytesToSend0 = DeviceDescriptor.bLength;
      if (BytesToSend0 > SetupPkt->wLength) BytesToSend0 = SetupPkt->wLength;
      WriteBufferToEndPoint0();
    };break;
    case USB_DESCRIPTOR_TYPE_CONFIGURATION:{
      pSendBuffer0 = (unsigned char code*)&ConfigurationDescriptor;
      BytesToSend0 = sizeof(ConfigurationDescriptor);
      if (BytesToSend0 > SetupPkt->wLength) BytesToSend0 = SetupPkt->wLength;
      WriteBufferToEndPoint0();
    };break;
    case USB_DESCRIPTOR_TYPE_STRING:{
      switch (SetupPkt->wValue & 0xFF){
        case 0 :{
          pSendBuffer0 = (unsigned char code*)&LANGID_Descriptor;
          BytesToSend0 = sizeof(LANGID_Descriptor);
        };break;
        case 1 :{
          pSendBuffer0 = (unsigned char code*)&Manufacturer_Descriptor;
          BytesToSend0 = sizeof(Manufacturer_Descriptor);
        };break;
        case 2 :{
          pSendBuffer0 = (unsigned char code*)&Product_Descriptor;
          BytesToSend0 = sizeof(Product_Descriptor);
        };break;
        case 3 :{
          pSendBuffer0 = (unsigned char code*)&Configuration_Descriptor;
          BytesToSend0 = sizeof(Configuration_Descriptor);
        };break;
        case 4 :{
          pSendBuffer0 = (unsigned char code*)&Interface_Descriptor;
          BytesToSend0 = sizeof(Interface_Descriptor);
        };break;
        case 5 :{
          pSendBuffer0 = (unsigned char code*)&EP1O_Descriptor;
          BytesToSend0 = sizeof(EP1O_Descriptor);
        };break;
        case 6 :{
          pSendBuffer0 = (unsigned char code*)&EP2I_Descriptor;
          BytesToSend0 = sizeof(EP2I_Descriptor);
        };break;
        default :{
          pSendBuffer0 = NULL;
          BytesToSend0 = 0;
        };break;
      }
      if (BytesToSend0 > SetupPkt->wLength) BytesToSend0 = SetupPkt->wLength;
      WriteBufferToEndPoint0();
    };break;
    default:{
      pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
    };break;
  }
}

/**************************************************************************/
void Process_EP0_OUT_Interrupt(void)
{
  USB_DEVICE_REQUEST SetupPkt;
  unsigned char byLtStat,Buffer[2];

  pdiReadEndpoint( PDI_EP0_OUT, 255, (unsigned char*)&SetupPkt);
  pdiAckSetupControl();                   // special ACK required for setup
  SetupPkt.wValue = SWAPWORD(SetupPkt.wValue);
  SetupPkt.wIndex = SWAPWORD(SetupPkt.wIndex);
  SetupPkt.wLength = SWAPWORD(SetupPkt.wLength);
  switch ( SetupPkt.bmRequestType & 0x7F) {
    case (USB_STANDARD_REQUEST + USB_RECIPIENT_DEVICE):{
      switch (SetupPkt.bRequest) {
        case USB_REQUEST_GET_STATUS:{
          Buffer[0] = 0x01;Buffer[1] = 0x00;
          pdiWriteEndpoint( PDI_EP0_IN, 2, Buffer);
        };break;
        case USB_REQUEST_CLEAR_FEATURE:
        case USB_REQUEST_SET_FEATURE:{
          pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
        };break;
        case USB_REQUEST_SET_ADDRESS:{
          pdiWriteEndpoint( PDI_EP0_IN, 0, 0);
          pdiSetAddressEnable( (unsigned char) SetupPkt.wValue, 1);
	  printf("\nA%04X",(int)SetupPkt.wValue);
        };break;
        case USB_REQUEST_GET_DESCRIPTOR:{
          GetDescriptor( &SetupPkt);
        };break;
        case USB_REQUEST_GET_CONFIGURATION:{
          pdiWriteEndpoint( PDI_EP0_IN, 1, &DeviceConfigured);
        };break;
        case USB_REQUEST_SET_CONFIGURATION:{
          DeviceConfigured = SetupPkt.wValue & 0xFF;
          pdiWriteEndpoint( PDI_EP0_IN, 0, 0);
        };break;
        case USB_REQUEST_SET_DESCRIPTOR:
        default:{
          pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
        };break;
      }
    };break;
    case (USB_STANDARD_REQUEST + USB_RECIPIENT_INTERFACE):{
      switch ( SetupPkt.bRequest) {
        case USB_REQUEST_GET_STATUS:{
          Buffer[0] = 0x00;Buffer[1] = 0x00;
          pdiWriteEndpoint( PDI_EP0_IN, 2, Buffer);
        };break;
        case USB_REQUEST_SET_INTERFACE:{
           if ( SetupPkt.wIndex == 0 && SetupPkt.wValue == 0)
             pdiWriteEndpoint( PDI_EP0_IN, 0, 0);
           else
             pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
        };break;
        case USB_REQUEST_GET_INTERFACE:{
          if (SetupPkt.wIndex == 0) { /* Interface Zero */
            Buffer[0] = 0; /* Alternative Setting */
            pdiWriteEndpoint( PDI_EP0_IN, 1, Buffer);
          } else {
            pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
          }
        };break;
        //case CLEAR_FEATURE:
        //case SET_FEATURE:
        default:{
          pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
        };break;
      }
    };break;
    case ( USB_STANDARD_REQUEST + USB_RECIPIENT_ENDPOINT):{
      switch ( SetupPkt.bRequest) {
        case USB_REQUEST_CLEAR_FEATURE:{
          if ( SetupPkt.wValue == 0) {
            switch ( SetupPkt.wIndex & 0xFF) {
              case 0x01:pdiSetEpStatus(PDI_EP1_OUT,0);break;
              case 0x02:pdiSetEpStatus(PDI_EP2_OUT,0);break;
              case 0x81:pdiSetEpStatus(PDI_EP1_IN,0);break;
              case 0x82:pdiSetEpStatus(PDI_EP2_IN,0);break;
              default:{               /* Invalid Endpoint - RequestError */
                pdiSetEpStatus(PDI_EP0_IN, PDI_SET_EP_STALLED);
              };break;
            }
            pdiWriteEndpoint( PDI_EP0_IN, 0, 0);
          } else {
            pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
          }
        };break;
        case USB_REQUEST_SET_FEATURE:{
          if ( SetupPkt.wValue == 0) {
            switch ( SetupPkt.wIndex & 0xFF) {
              case 0x01:pdiSetEpStatus(PDI_EP1_OUT,PDI_SET_EP_STALLED);break;
              case 0x02:pdiSetEpStatus(PDI_EP2_OUT,PDI_SET_EP_STALLED);break;
              case 0x81:pdiSetEpStatus(PDI_EP1_IN,PDI_SET_EP_STALLED);break;
              case 0x82:pdiSetEpStatus(PDI_EP2_IN,PDI_SET_EP_STALLED);break;
              default:{               /* Invalid Endpoint - RequestError */
                pdiSetEpStatus(PDI_EP0_IN,PDI_SET_EP_STALLED);
              };break;
            }
            pdiWriteEndpoint( PDI_EP0_IN, 0, 0);
          } else {
            pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
          }
        };break;
/*
        case USB_REQUEST_GET_STATUS:{
          switch (SetupPkt.wIndex & 0xFF) {
              case 0x01:pdiGetEpStat(D11_EP1OUT_IDX);break;
              case 0x02:pdiGetEpStat(D11_EP2OUT_IDX);break;
              case 0x81:pdiGetEpStat(D11_EP1IN_IDX);break;
              case 0x82:pdiGetEpStat(D11_EP2IN_IDX);break;
            default:{               
              pdiSetEpStatus(D11_EP0IN_IDX);
            };break;
                  }
          if (Buffer[0] & 0x08) Buffer[0] = 0x01;
          else                  Buffer[0] = 0x00;
          Buffer[1] = 0x00;
          pdiWriteEndpoint( PDI_EP0_IN, 2, Buffer);
        };break;
*/
        default:{
          pdiSetEpStatus(PDI_EP0_IN, PDI_SET_EP_STALLED);
        };break;
      }
    };break;
    case (USB_VENDOR_REQUEST + USB_RECIPIENT_DEVICE):
    case (USB_VENDOR_REQUEST + USB_RECIPIENT_ENDPOINT):{
      switch ( SetupPkt.bRequest) {
        case VENDOR_START_ULAN:{
          Buffer[0] = 0x00;                           //not config.
          switch (SetupPkt.wValue) {
            case 9600:
              uLanConfigured=1;
              Buffer[0] = 0x01;
              ul_drv_set_bdiv(10);   // Prenosova rychlost
              ul_drv_set_adr(SetupPkt.wIndex & 0x3F);  // Nastaveni adresy
              break;
            case 19200:
              uLanConfigured=1;
              Buffer[0] = 0x01;
              ul_drv_set_bdiv(5);   // Prenosova rychlost
              ul_drv_set_adr(SetupPkt.wIndex & 0x3F);  // Nastaveni adresy
              break;
            default:{
              uLanConfigured=0;
              /*ES=0;*/                                   //disable int. from serial
            }
          }
          pdiWriteEndpoint( PDI_EP0_IN, 1, Buffer);
        };break;
        case VENDOR_STOP_ULAN:{
          uLanConfigured=0;
          /*ES=0;*/                                     //disable int. from serial
        };break;
        case VENDOR_IS_RUNNING_ULAN:{
          Buffer[0] = uLanConfigured;
          pdiWriteEndpoint( PDI_EP0_IN, 1, Buffer);
        }
        case VENDOR_ARC_ON_ULAN:{                     //zapnuti prijimani vsech zprav na sbernici
          /*ACC=1;
          uL_FNCC(0x07);*/                            //zapnuti
        }
        case VENDOR_ARC_OFF_ULAN:{                    //vypnuti prijimani vsech zprav na sbernici
          /*ACC=0;
          uL_FNCC(0x07);*/                              //vypnuti
        }
        default:{
          pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
        };break;
      }
    };break;
    default:{
      pdiSetEpStatus( PDI_EP0_IN, PDI_SET_EP_STALLED);
    };break;
  }
}

/**************************************************************************/
void D11GetIRQ() {
  unsigned wIrq;
  unsigned char byLtStat,size;

  wIrq=pdiGetInterrupt();
  if(wIrq&~1)
    printf("I%04X",wIrq);
  if (wIrq & PDI_INT_BUSRESET) {
    putchar('r');
    pdiSetAddressEnable( 0, 1);    // enable emb.function
    /*pdiInit();*/
  }
  if (wIrq & PDI_INT_EP0_IN) {
    putchar('i');
    byLtStat=pdiGetLastTransStatus(PDI_EP0_IN);
    if (byLtStat & PDI_LTSTAT_RXTX_OK)
      WriteBufferToEndPoint0();
  }
  if (wIrq & PDI_INT_EP0_OUT) {
    putchar('o');
    byLtStat=pdiGetLastTransStatus( PDI_EP0_OUT);
    if (byLtStat & PDI_LTSTAT_RXTX_OK) {  //Check "Data Receive Success" for 1
      if (byLtStat & PDI_LTSTAT_SETUP) { //Check "Setup Packet received" > ACK
        Process_EP0_OUT_Interrupt();
      } else
        pdiReadEndpoint( PDI_EP0_OUT, 0, 0);
    }
  }
  //EP1 prijem dat z PC (Zpravy k odeslani)
  if (wIrq & PDI_INT_EP1_IN) {
    putchar('1');
    byLtStat=pdiGetLastTransStatus( PDI_EP1_IN);
    if (byLtStat & PDI_LTSTAT_RXTX_OK) {
      pdiSetEpStatus(PDI_EP1_IN, PDI_SET_EP_STALLED);
    }
  }
  if (wIrq & PDI_INT_EP1_OUT) {
    putchar('1');
    byLtStat=pdiGetLastTransStatus(PDI_EP1_OUT);
    if (byLtStat & PDI_LTSTAT_RXTX_OK) {
      size=pdiReadEndpoint(PDI_EP1_OUT, 255, (unsigned char *)&uLanBuffRx[0]);
      if (BytesToRec2==-1) {
        BytesToRec2 =uLanBuffRx[6]+uLanBuffRx[7]*0x100;
        msginfo.dadr=uLanBuffRx[0]&0x7F;  // dadr
        msginfo.sadr=uLanBuffRx[1]&0x7F;  // sadr
        msginfo.cmd =uLanBuffRx[2];       // cmd
        msginfo.flg =uLanBuffRx[3];       // status zpravy
        msginfo.stamp=uLanBuffRx[4];      // stamp
	ul_newmsg(ul_fd, &msginfo);
      } else {
        if (BytesToRec2>0) {
	  ul_write(ul_fd, &uLanBuffRx, size);
          BytesToRec2-=size;
        }
      }	
      if (BytesToRec2<=0) {
	ul_o_close(ul_fd);
        pdiWriteEndpoint( PDI_EP1_IN, 0, 0);
        timer_tx_off=5;LED_TX=0;
        BytesToRec2=-1;
      }
    }
  }
  //EP2 vysilani dat do PC (Prijate zpravy)
  if (wIrq & PDI_INT_EP2_IN) {
    putchar('2');
    byLtStat=pdiGetLastTransStatus( PDI_EP2_IN);
    if (byLtStat & PDI_LTSTAT_RXTX_OK) {
     WriteBufferToEndPoint2();
    }
  }
  if (wIrq & PDI_INT_EP2_OUT) {
    putchar('2');
    byLtStat=pdiGetLastTransStatus( PDI_EP2_OUT);
    if (byLtStat & PDI_LTSTAT_RXTX_OK) {
      pdiSetEpStatus(PDI_EP2_OUT, PDI_SET_EP_STALLED);
    }
  }
}

void main(void) {
  bit init_fl=0;
  P3=0xff;P1=0xff;
  TMOD=0x21;                                    //citac0-16bitovy,1-8bitovy(reload)
  TH0=TIMERH10ms;TL0=TIMERL10ms;                //nastav casovac0
  TCON=0x10;                                    //nul. priz. casov0,1;spust casov0
  /*IE=0x02;*/                                      //povol preruseni casov0
 #if !defined(BOARD_BART)||defined(WITHOUT_ULAN)
  simple_ser_init();
 #endif /*serial*/
 #ifndef WITHOUT_ULAN
  //Default configuration
  ul_drv_set_adr(5);
  ul_drv_set_bdiv(5);
  ul_drv_init();uLanConfigured=1;               //uLan inicializace (musi byt!)
  ul_fd=ul_open(0,0);
 #endif /*WITHOUT_ULAN*/
  //Unconfigured
  //ES=0;uLanConfigured=0;
  LED_TX=0;
  printf("Calling pdiInit\n");
  //pdiInit(); init_fl=1;
  /*printf("pdiGetChipID %4X\n",pdiGetChipID());*/
  EA=1;
  //while(1) {while(BytesToSend2++) LED_TX=0;while(BytesToSend2++) LED_TX=1;};
  BytesToSend2=BytesToRec2=-1;                   //zacatek prijmu a vysilani
  while(1) {
    if(RI){
      switch(getchar()){
        case 'i':
          printf("pdiGetChipID %4X\n",pdiGetChipID());
	  break;
        case 'f':
          printf("pdiGetFrameNumber %4X\n",pdiGetFrameNumber());
	  break;
	  
        case '0':
	  pdiSetMode(0, 3);
	  init_fl=0;
	  break;
        case '1':
          pdiInit();
	  init_fl=1;
	  break;
        case 'c':
	  while(!RI);
	  pdiSetMode(PDI_MODE_REMOTE_WAKEUP | PDI_MODE_NO_LAZY_CLOCK | PDI_MODE_CLOCK_RUNNING |
               PDI_MODE_MUSTBEONE, getchar()-'0');
	  init_fl=0;
	  break;
      }
    }
    if(!init_fl) continue;
    
    /*if(!USB_IRQ)*/
      D11GetIRQ();
    if ((uLanConfigured) && (BytesToSend2==-1)) {
      if(ul_inepoll(ul_fd)&&(ul_acceptmsg(ul_fd, &msginfo)>=0)){
        //pokus o otevreni vstupni zpravy
        if (!(msginfo.flg & UL_BFL_PROC)) {  //rosviceni diody prijmu znaku
          timer_rx_off=5;LED_RX=0;
        }
        BytesToSend2=msginfo.len;
        uLanBuffTx[0]=msginfo.dadr;      uLanBuffTx[1]=msginfo.sadr;
        uLanBuffTx[2]=msginfo.cmd;       uLanBuffTx[3]=msginfo.flg;
        uLanBuffTx[4]=msginfo.stamp;     uLanBuffTx[5]=0;
        uLanBuffTx[6]=BytesToSend2%0x100;uLanBuffTx[7]=BytesToSend2/0x100;
        pdiWriteEndpoint( PDI_EP2_IN, 8, uLanBuffTx); // zapis dat do EP
        if (BytesToSend2==0) {
          ul_o_close(ul_fd);                    // uzavreni zpravy
	  BytesToSend2=-1;
	}
      }
    }
  }
}
