/*************************************************/
/***   Module : USB product definitions        ***/
/***   Author : Roman Bartosinski 17.07.2002   ***/
/***   Modify : 08.08.2002,27.08.2002          ***/
/***            05.09.2002                     ***/
/*************************************************/

#ifndef USB_DEFINITIONS_HEADER_FILE
 #define USB_DEFINITIONS_HEADER_FILE

 #include "usb_spec.h"
 #include "pdiusb.h" // for endpoint packet sizes

 #define USB_VENDOR_ID   0xDEAD   // my own VID
// #define USB_PRODUCT_ID  0x6F56  // utermd12 28.5.2002
// #define USB_PRODUCT_ID  0x5912  // usbcom 22.8.2002
 #define USB_PRODUCT_ID  0x2262  // firmware upload 8.8.2002

 #define USB_RELEASE_VER 0x0100

// *** Class codes for device description
  #define USB_CLASS_CODE            0xFF
  #define USB_SUBCLASS_CODE         0xFF
  #define USB_PROTOCOL_CODE         0xFF


//*****************************************************//
//*** Static data structures for device descriptors ***//
//*****************************************************//
  #define NUM_ENDPOINTS	2

  #define CONFIG_DESCRIPTOR_LENGTH sizeof( USB_CONFIGURATION_DESCRIPTOR) \
                                 + sizeof( USB_INTERFACE_DESCRIPTOR) \
                                 + (NUM_ENDPOINTS * sizeof( USB_ENDPOINT_DESCRIPTOR)) 

#define SWAPWORD(x)   ( (((x) << 8) & 0xFF00) | (((x) >> 8) & 0x00FF) )

USB_DEVICE_DESCRIPTOR code DeviceDescriptor = {
    sizeof(USB_DEVICE_DESCRIPTOR), /* bLength */
    USB_DESCRIPTOR_TYPE_DEVICE,        /* bDescriptorType */
    SWAPWORD(0x0110),               // usb version = 01.10 (binary coded decimal)
    0,                             /* bDeviceClass */
    0,                             /* bDeviceSubclass */
    0,                             /* bDeviceProtocol */
    PDI_EP0_PACKET_SIZE,               /* bMaxPacketSize 8 Bytes */
//    SWAPWORD(0x045E),              // vendor id code (VID)  /* Intel 82930 USB Bulk IO Test Board */
    SWAPWORD(0x4343),              // vendor id code (VID)   /* Petr Smolik */
//    SWAPWORD(0x930A),              // product id code (PID)
    SWAPWORD(0x0001),              // product id code (PID)  /* usb2ulan convertor */
    SWAPWORD(0x0001),              // device release number = 01.00 (bcd)
    1,                             /* iManufacturer String Index */
    2,                             /* iProduct String Index */
    0,                             /* iSerialNumber String Index */
    1                              /* bNumberConfigurations */
};

typedef struct {
        USB_CONFIGURATION_DESCRIPTOR ConfigDescriptor;
        USB_INTERFACE_DESCRIPTOR InterfaceDescriptor;
        USB_ENDPOINT_DESCRIPTOR EndpointDescriptor0;
        USB_ENDPOINT_DESCRIPTOR EndpointDescriptor1;
} USB_CONFIG_DATA, *PUSB_CONFIG_DATA;

USB_CONFIG_DATA code ConfigurationDescriptor = {
    {                              /* configuration descriptor */
    sizeof(USB_CONFIGURATION_DESCRIPTOR), /* bLength */
    USB_DESCRIPTOR_TYPE_CONFIGURATION, /* bDescriptorType */
    SWAPWORD(sizeof(USB_CONFIG_DATA)),/* wTotalLength */
    1,                             /* bNumInterfaces */
    1,                             /* bConfigurationValue */
    3,                             /* iConfiguration String Index */
    0x80,                          /* bmAttributes Bus Powered, No Remote Wakeup */
    0x32                           /* bMaxPower, 100mA */
    },
    {                              /* interface descriptor */
    sizeof(USB_INTERFACE_DESCRIPTOR), /* bLength */
    USB_DESCRIPTOR_TYPE_INTERFACE,     /* bDescriptorType */
    0,                             /* bInterface Number */
    0,                             /* bAlternateSetting */
    2,                             /* bNumEndpoints */
    0xFF,                          /* bInterfaceClass (Vendor specific) */
    0xFF,                          /* bInterfaceSubClass */
    0xFF,                          /* bInterfaceProtocol */
    4                              /* iInterface String Index */
    },
    {                              /* endpoint descriptor */
    sizeof(USB_ENDPOINT_DESCRIPTOR),  /* bLength */
    USB_DESCRIPTOR_TYPE_ENDPOINT,      /* bDescriptorType */
    0x01,                          /* bEndpoint Address EP1 OUT */
    0x02,                          /* bmAttributes - BULK */
    SWAPWORD(PDI_EP2_RX_SIZE),              /* wMaxPacketSize */
    0x00                           /* bInterval */
    },
    {                              /* endpoint descriptor */
    sizeof(USB_ENDPOINT_DESCRIPTOR),  /* bLength */
    USB_DESCRIPTOR_TYPE_ENDPOINT,      /* bDescriptorType */
    0x82,                          /* bEndpoint Address EP2 IN */
    0x02,                          /* bmAttributes - BULK */
    SWAPWORD(PDI_EP2_TX_SIZE),              /* wMaxPacketSize */
    0x00                           /* bInterval */
    }
};

USB_STRING_DESCRIPTOR code LANGID_Descriptor = { /* LANGID String Descriptor Zero */
    4,          /* bLenght */
    USB_DESCRIPTOR_TYPE_STRING,             /* bDescriptorType */
    SWAPWORD(0x0409)                    /* LANGID US English */
};

char code Manufacturer_Descriptor[] = { /* ManufacturerString 1 */
    48,                     /* bLenght */
    USB_DESCRIPTOR_TYPE_STRING,                        /* bDescriptorType */    
      'S',0,
      'm',0,
      'o',0,
      'l',0,
      'i',0,
      'k',0,
      ',',0,
      'B',0,
      'a',0,
      'r',0,
      't',0,
      'o',0,
      's',0,
      'i',0,
      'n',0,
      's',0,
      'k',0,
      'i',0,
      ',',0,
      'P',0,
      'i',0,
      's',0,
      'a',0
};

char code Product_Descriptor[] = { /* ProductString */
    38,                /* bLenght */
    USB_DESCRIPTOR_TYPE_STRING,                   /* bDescriptorType */
      'U',0,
      'S',0,
      'B',0,
      '2',0,
      'u',0,
      'L',0,
      'a',0,
      'n',0,
      ' ',0,
      'c',0,
      'o',0,
      'n',0,
      'v',0,
      'e',0,
      'r',0,
      't',0,
      'o',0,
      'r',0
};

char code Configuration_Descriptor[] = { /* ConfigurationString */
    16,                /* bLenght */
    USB_DESCRIPTOR_TYPE_STRING,                   /* bDescriptorType */
      'd',0,
      'e',0,
      'f',0,
      'a',0,
      'u',0,
      'l',0,
      't',0
};

char code Interface_Descriptor[] = { /* InterfaceString */
    16,                /* bLenght */
    USB_DESCRIPTOR_TYPE_STRING,                   /* bDescriptorType */
      'd',0,
      'e',0,
      'f',0,
      'a',0,
      'u',0,
      'l',0,
      't',0
};

char code EP1O_Descriptor[] = { /* EP1 OUT descriptor */
    136,             /* bLenght */
    USB_DESCRIPTOR_TYPE_STRING,                     /* bDescriptorType */
      'E',0,
      'P',0,
      '1',0,
      'O',0,
      'U',0,
      'T',0,
      '-',0,
      's',0,
      'e',0,
      'n',0,
      'd',0,
      ' ',0,
      'm',0,
      'e',0,
      's',0,
      's',0,
      'a',0,
      'g',0,
      'e',0,
      '-',0,
      'd',0,
      'a',0,
      'd',0,
      'r',0,
      ',',0,
      's',0,
      'a',0,
      'd',0,
      'r',0,
      ',',0,
      'c',0,
      'm',0,
      'd',0,
      ',',0,
      'f',0,
      'l',0,
      'g',0,
      ',',0,
      's',0,
      't',0,
      'a',0,
      'm',0,
      'p',0,
      ',',0,
      'f',0,
      'r',0,
      'e',0,
      'e',0,
      ',',0,
      'l',0,
      'e',0,
      'n',0,
      '-',0,
      'l',0,
      'o',0,
      ',',0,
      'l',0,
      'e',0,
      'n',0,
      '-',0,
      'h',0,
      'i',0,
      ',',0,
      'd',0,
      'a',0,
      't',0,
      'a',0
};

char code EP2I_Descriptor[] = { /* EP2 IN descriptor */
    140,             /* bLenght */
    USB_DESCRIPTOR_TYPE_STRING,                     /* bDescriptorType */
      'E',0,
      'P',0,
      '2',0,
      'I',0,
      'N',0,
      '-',0,
      'r',0,
      'e',0,
      'c',0,
      'e',0,
      'i',0,
      'v',0,
      'e',0,
      ' ',0,
      'm',0,
      'e',0,
      's',0,
      's',0,
      'a',0,
      'g',0,
      'e',0,
      '-',0,
      'd',0,
      'a',0,
      'd',0,
      'r',0,
      ',',0,
      's',0,
      'a',0,
      'd',0,
      'r',0,
      ',',0,
      'c',0,
      'm',0,
      'd',0,
      ',',0,
      'f',0,
      'l',0,
      'g',0,
      ',',0,
      's',0,
      't',0,
      'a',0,
      'm',0,
      'p',0,
      ',',0,
      'f',0,
      'r',0,
      'e',0,
      'e',0,
      ',',0,
      'l',0,
      'e',0,
      'n',0,
      '-',0,
      'l',0,
      'o',0,
      ',',0,
      'l',0,
      'e',0,
      'n',0,
      '-',0,
      'h',0,
      'i',0,
      ',',0,
      'd',0,
      'a',0,
      't',0,
      'a',0
};

/*

  code const USB_DEVICE_DESCRIPTOR DeviceDescription = {
    sizeof( USB_DEVICE_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_DEVICE,
    SWAP( 0x0100),
    USB_CLASS_CODE,
    USB_SUBCLASS_CODE,
    USB_PROTOCOL_CODE,
    EP0_PACKET_SIZE,
    SWAP( USB_VENDOR_ID),
    SWAP( USB_PRODUCT_ID),  
    SWAP( USB_RELEASE_VER),
    1,
    2,
    3,
    1
  };
  code const USB_CONFIGURATION_DESCRIPTOR ConfigDescription = {
    sizeof( USB_CONFIGURATION_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_CONFIGURATION,
    SWAP( CONFIG_DESCRIPTOR_LENGTH),
    1,
    1,
    0,
    0x60, // ??? in spec is, taha bit 7 must be set to one -> 0xe0
    0     // 0 mA odber z hostu
  };

  code const USB_INTERFACE_DESCRIPTOR InterfaceDescription = {      // is sended with ConfigDescription
    sizeof( USB_INTERFACE_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_INTERFACE,
    0,    // index of this interface for SetInterface request
    0,
    NUM_ENDPOINTS,
    USB_CLASS_CODE,
    USB_SUBCLASS_CODE,
    USB_PROTOCOL_CODE,
    0
  };

  code const USB_ENDPOINT_DESCRIPTOR Ep2TxDescription = {           // is sended with ConfigDescription
    sizeof( USB_ENDPOINT_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_ENDPOINT,
    0x82,
    USB_ENDPOINT_TYPE_BULK,
    SWAP( EP2_PACKET_SIZE),
    0
  };
  code const USB_ENDPOINT_DESCRIPTOR Ep2RxDescription = {           // is sended with ConfigDescription
    sizeof( USB_ENDPOINT_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_ENDPOINT,
    0x02,
    USB_ENDPOINT_TYPE_BULK,
    SWAP( EP2_PACKET_SIZE),
    0
  };
  code const USB_ENDPOINT_DESCRIPTOR Ep1TxDescription = {           // is sended with ConfigDescription
    sizeof( USB_ENDPOINT_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_ENDPOINT,
    0x12,
    USB_ENDPOINT_TYPE_BULK,
    SWAP( EP1_PACKET_SIZE),
    0
  };
  code const USB_ENDPOINT_DESCRIPTOR Ep1RxDescription = {           // is sended with ConfigDescription
    sizeof( USB_ENDPOINT_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_ENDPOINT,
    0x01,
    USB_ENDPOINT_TYPE_BULK,
    SWAP( EP1_PACKET_SIZE),
    0
  };
  

  code const char Str0Desc[] = {                   // descibe language of strings
    4, 0x03,  // 2+2*N , N is count of supported languages 
    0x05,0x04 // czech language 0x0405 , english 0x0409
  };
  code const char Str1Desc[] = {                   // 1 = manufacturer
    28, 0x03,                                               
    'R',0,
    '.',0,
    'B',0,
    'a',0,
    'r',0,
    't',0,
    'o',0,
    's',0,
    'i',0,
    'n',0,
    's',0,
    'k',0,
    'i',0
  };
  code const char Str2Desc[] = {                   // 2 = product RTM
    34, 0x03,                                               
    'P',0,
    'r',0,
    'o',0,
    'g',0,
    'r',0,
    'a',0,
    'm',0,
    ' ',0,
    'U',0,
    'p',0,
    'l',0,
    'o',0,
    'a',0,
    'd',0,
    'e',0,
    'r',0
  };
  code const char Str3Desc[] = {                   // 3 = version
    28, 0x03,                                               
    '2',0,
    '.',0,
    '0',0,
    '-',0,
    '2',0,
    '4',0,
    '.',0,
    '9',0,
    '.',0,
    '2',0,
    '0',0,
    '0',0,
    '2',0
  };
#define CNT_STRINGS 4

  code const PUSB_STRING_DESCRIPTOR StringDescriptors[] = {         // all strings in pointers array
    Str0Desc,
    Str1Desc,
    Str2Desc,
    Str3Desc,
  };

//  extern code const (*standardDeviceFunctions[])(void);
//  extern code const (*dfuClassFunctions[])(void);

*/
#endif
