/**************************************************************************/
//                                   Petr Smolik
/**************************************************************************/
//#include <AT89X52.H>
#include <reg1210.h>
#include <stdio.h>
#include <string.h>
#include <intrins.h>
#include "usbdesc.h"
#include "usbd11.h"
#include "ulan.h"

int simple_ser_init(void);

#if 0
sbit LED_RX = 0x94;        // P1.4
sbit LED_TX = 0x95;        // P1.5
sbit USB_IRQ = 0xB2;       // P3.2
#else
sbit LED_RX = 0xB3;        // P3.3
sbit LED_TX = 0xB3;        // P3.3
sbit USB_IRQ = 0xB2;       // P3.2
#endif

extern bit uLF_RS,uLF_NB,uLF_NA;
ul_fd_t ul_fd;
xdata ul_msginfo msginfo;

#define TIMERH10ms       0xDC               //hodnoty predelicky pro casovac na 10ms
#define TIMERL10ms       0x00

unsigned char            *pSendBuffer0,*pSendBuffer2;
unsigned int             BytesToSend0,BytesToSend2,BytesToRec2;
unsigned char            DeviceAddress;
unsigned char            DeviceConfigured;
unsigned char            uLanConfigured;
unsigned char idata      timer_str;
unsigned char idata      timer_rx_off;
unsigned char idata      timer_tx_off;
unsigned char idata      timer_configured;

unsigned char xdata      uLanBuffRx[MAX_PACKET_SIZE];
unsigned char xdata      uLanBuffTx[MAX_PACKET_SIZE];

/****************************************************************************/
//definice prusenich (casovac0)
#if 0
void timer0(void) interrupt 1 {
  TH0=TIMERH10ms;TL0=TIMERL10ms;
  if (timer_rx_off!=0) timer_rx_off--;
  else LED_RX=1;
  if (timer_tx_off!=0) timer_tx_off--;
  else LED_TX=1;
  if (timer_str!=0) {
    timer_str=5;
    ul_str();
    /*if (_testbit_(ES)) {
      if (uLF_NA) {
        if ((uLF_NB) && (uLF_RS)) TI=1;
        uLF_NB=1;
      }
      uLF_NA=1;
      ES=1;
    }*/
  } else timer_str--;
  if (timer_configured!=0) timer_configured--;
  else {
    timer_configured=250;
    if (!uLanConfigured) timer_configured=20;
    if ((!DeviceConfigured) || (!uLanConfigured)) {
      LED_RX=LED_TX=0;
      timer_rx_off=timer_tx_off=5;
    }
  }
}
#endif

/**************************************************************************/
void WriteBufferToEndPoint0() {
  if (BytesToSend0 == 0) D11_TxNull(D11_EP0IN_IDX);
  else {
    if (BytesToSend0 >= MAX_PACKET_SIZE) {
      D11_WrEp(D11_EP0IN_IDX,pSendBuffer0,MAX_PACKET_SIZE,1);  //Data from flash
      BytesToSend0 -= MAX_PACKET_SIZE;
      pSendBuffer0 += MAX_PACKET_SIZE;
    } else {
      D11_WrEp(D11_EP0IN_IDX,pSendBuffer0,BytesToSend0,1);
      pSendBuffer0 += BytesToSend0;
      BytesToSend0 = 0;
    }
  }
}

/**************************************************************************/
void WriteBufferToEndPoint2() {
  if (BytesToSend2 != -1) {
    if (BytesToSend2 > MAX_PACKET_SIZE) {
      ul_read(ul_fd,uLanBuffTx,MAX_PACKET_SIZE); // read data from uLan
      D11_WrEp(D11_EP2IN_IDX,uLanBuffTx,MAX_PACKET_SIZE,0); // zapis dat do EP
      BytesToSend2 -= MAX_PACKET_SIZE;
    } else {
      ul_read(ul_fd,uLanBuffTx,BytesToSend2);	// read data from uLan
      ul_i_close(ul_fd);			// close received message
      D11_WrEp(D11_EP2IN_IDX,uLanBuffTx,BytesToSend2,0); // zapis dat do EP
      BytesToSend2 = -1;
     }
   }
 }

/**************************************************************************/
void GetDescriptor(PUSB_SETUP_REQUEST SetupPkt) {
  switch((SetupPkt->wValue & 0xFF00) >> 8) {
    case TYPE_DEVICE_DESCRIPTOR:{
      pSendBuffer0 = (unsigned char code*)&DeviceDescriptor;
      BytesToSend0 = DeviceDescriptor.bLength;
      if (BytesToSend0 > SetupPkt->wLength) BytesToSend0 = SetupPkt->wLength;
      WriteBufferToEndPoint0();
    };break;
    case TYPE_CONFIGURATION_DESCRIPTOR:{
      pSendBuffer0 = (unsigned char code*)&ConfigurationDescriptor;
      BytesToSend0 = sizeof(ConfigurationDescriptor);
      if (BytesToSend0 > SetupPkt->wLength) BytesToSend0 = SetupPkt->wLength;
      WriteBufferToEndPoint0();
    };break;
    case TYPE_STRING_DESCRIPTOR:{
      switch (SetupPkt->wValue & 0xFF){
        case 0 :{
          pSendBuffer0 = (unsigned char code*)&LANGID_Descriptor;
          BytesToSend0 = sizeof(LANGID_Descriptor);
        };break;
        case 1 :{
          pSendBuffer0 = (unsigned char code*)&Manufacturer_Descriptor;
          BytesToSend0 = sizeof(Manufacturer_Descriptor);
        };break;
        case 2 :{
          pSendBuffer0 = (unsigned char code*)&Product_Descriptor;
          BytesToSend0 = sizeof(Product_Descriptor);
        };break;
        case 3 :{
          pSendBuffer0 = (unsigned char code*)&Configuration_Descriptor;
          BytesToSend0 = sizeof(Configuration_Descriptor);
        };break;
        case 4 :{
          pSendBuffer0 = (unsigned char code*)&Interface_Descriptor;
          BytesToSend0 = sizeof(Interface_Descriptor);
        };break;
        case 5 :{
          pSendBuffer0 = (unsigned char code*)&EP1O_Descriptor;
          BytesToSend0 = sizeof(EP1O_Descriptor);
        };break;
        case 6 :{
          pSendBuffer0 = (unsigned char code*)&EP2I_Descriptor;
          BytesToSend0 = sizeof(EP2I_Descriptor);
        };break;
        default :{
          pSendBuffer0 = NULL;
          BytesToSend0 = 0;
        };break;
      }
      if (BytesToSend0 > SetupPkt->wLength) BytesToSend0 = SetupPkt->wLength;
      WriteBufferToEndPoint0();
    };break;
    default:{
      D11_SetEpStall(D11_EP0IN_IDX);
    };break;
  }
}

/**************************************************************************/
void Process_EP0_OUT_Interrupt(void)
{
  USB_SETUP_REQUEST SetupPkt;
  unsigned char byLtStat,Buffer[2];
  
      D11_RdEp((unsigned char*)&SetupPkt, D11_EP0OUT_IDX);
      D11_AckSetup(); 			// special ACK required for setup
      SetupPkt.wValue = SWAPWORD(SetupPkt.wValue);
      SetupPkt.wIndex = SWAPWORD(SetupPkt.wIndex);
      SetupPkt.wLength = SWAPWORD(SetupPkt.wLength);
      switch (SetupPkt.bmRequestType & 0x7F) {
        case STANDARD_DEVICE_REQUEST:{
          switch (SetupPkt.bRequest) {
            case GET_STATUS:{
              Buffer[0] = 0x01;Buffer[1] = 0x00;
              D11_WrEp(D11_EP0IN_IDX,Buffer,2,0);
            };break;
            case CLEAR_FEATURE:
            case SET_FEATURE:{
              D11_SetEpStall(D11_EP0IN_IDX);
            };break;
            case SET_ADDRESS:{
              DeviceAddress = SetupPkt.wValue | 0x80;
              D11_TxNull(D11_EP0IN_IDX);
              D11_CmdDataI2C(D11_CMD_SET_ADDRESS, &DeviceAddress, 1, WR_DATA);
            };break;
            case GET_DESCRIPTOR:{
              GetDescriptor(&SetupPkt);
            };break;
            case GET_CONFIGURATION:{
              D11_WrEp(D11_EP0IN_IDX,&DeviceConfigured,1,0);
            };break;
            case SET_CONFIGURATION:{
              DeviceConfigured = SetupPkt.wValue & 0xFF;
              D11_TxNull(D11_EP0IN_IDX);
            };break;
            case SET_DESCRIPTOR:
            default:{
              D11_SetEpStall(D11_EP0IN_IDX);
            };break;
          }
        };break;
        case STANDARD_INTERFACE_REQUEST:{
          switch (SetupPkt.bRequest) {
            case GET_STATUS:{
              Buffer[0] = 0x00;Buffer[1] = 0x00;
              D11_WrEp(D11_EP0IN_IDX,Buffer,2,0);
            };break;
            case SET_INTERFACE:{
               if (SetupPkt.wIndex == 0 && SetupPkt.wValue == 0)
                 D11_TxNull(D11_EP0IN_IDX);
               else
                 D11_SetEpStall(D11_EP0IN_IDX);
            };break;
            case GET_INTERFACE:{
              if (SetupPkt.wIndex == 0) { /* Interface Zero */
                Buffer[0] = 0; /* Alternative Setting */
                D11_WrEp(D11_EP0IN_IDX,Buffer,1,0);
              } else {
                D11_SetEpStall(D11_EP0IN_IDX);
              }
            };break;
            //case CLEAR_FEATURE:
            //case SET_FEATURE:
            default:{
              D11_SetEpStall(D11_EP0IN_IDX);
            };break;
          }
        };break;
        case STANDARD_ENDPOINT_REQUEST:{
          switch (SetupPkt.bRequest) {
            case CLEAR_FEATURE:{
              if (SetupPkt.wValue == ENDPOINT_HALT) {
                switch (SetupPkt.wIndex & 0xFF) {
                  case 0x01:D11_ClrEpStall(D11_EP1OUT_IDX);break;
                  case 0x02:D11_ClrEpStall(D11_EP2OUT_IDX);break;
                  case 0x03:D11_ClrEpStall(D11_EP3OUT_IDX);break;
                  case 0x81:D11_ClrEpStall(D11_EP1IN_IDX);break;
                  case 0x82:D11_ClrEpStall(D11_EP2IN_IDX);break;
                  case 0x83:D11_ClrEpStall(D11_EP3IN_IDX);break;
                  default:{               /* Invalid Endpoint - RequestError */
                    D11_SetEpStall(D11_EP0IN_IDX);
                  };break;
                }
                D11_TxNull(D11_EP0IN_IDX);
              } else {
                D11_SetEpStall(D11_EP0IN_IDX);
              }
            };break;
            case SET_FEATURE:{
              if (SetupPkt.wValue == ENDPOINT_HALT) {
                switch (SetupPkt.wIndex & 0xFF) {
                  case 0x01:D11_SetEpStall(D11_EP1OUT_IDX);break;
                  case 0x02:D11_SetEpStall(D11_EP2OUT_IDX);break;
                  case 0x03:D11_SetEpStall(D11_EP3OUT_IDX);break;
                  case 0x81:D11_SetEpStall(D11_EP1IN_IDX);break;
                  case 0x82:D11_SetEpStall(D11_EP2IN_IDX);break;
                  case 0x83:D11_SetEpStall(D11_EP3IN_IDX);break;
                  default:{               /* Invalid Endpoint - RequestError */
                    D11_SetEpStall(D11_EP0IN_IDX);
                  };break;
                }
                D11_TxNull(D11_EP0IN_IDX);
              } else {
                D11_SetEpStall(D11_EP0IN_IDX);
              }
            };break;
            case GET_STATUS:{
              /* Get Status Request to Endpoint should return */
              /* Halt Status in D0 for Interrupt and Bulk */
              switch (SetupPkt.wIndex & 0xFF) {
                  case 0x01:D11_GetEpStat(D11_EP1OUT_IDX);break;
                  case 0x02:D11_GetEpStat(D11_EP2OUT_IDX);break;
                  case 0x03:D11_GetEpStat(D11_EP3OUT_IDX);break;
                  case 0x81:D11_GetEpStat(D11_EP1IN_IDX);break;
                  case 0x82:D11_GetEpStat(D11_EP2IN_IDX);break;
                  case 0x83:D11_GetEpStat(D11_EP3IN_IDX);break;
                default:{               /* Invalid Endpoint - RequestError */
                  D11_SetEpStall(D11_EP0IN_IDX);
                };break;
	              }
              if (Buffer[0] & 0x08) Buffer[0] = 0x01;
              else                  Buffer[0] = 0x00;
              Buffer[1] = 0x00;
              D11_WrEp(D11_EP0IN_IDX,Buffer,2,0);
            };break;
            default:{
              D11_SetEpStall(D11_EP0IN_IDX);
            };break;
          }
        };break;
        case VENDOR_DEVICE_REQUEST:
        case VENDOR_ENDPOINT_REQUEST:{
          switch (SetupPkt.bRequest) {
            case VENDOR_START_ULAN:{
              Buffer[0] = 0x00;                           //not config.
              switch (SetupPkt.wValue) {
                case 9600:{
                  uLanConfigured=1;
                  Buffer[0] = 0x01;
                  //uLanInit(SetupPkt.wIndex & 0x1F,6);     //address, baud rate
                };break;
                case 19200:{
                  uLanConfigured=1;
                  Buffer[0] = 0x01;
                  //uLanInit(SetupPkt.wIndex & 0x1F,3);     //address, baud rate
                };break;
                default:{
                  uLanConfigured=0;
                  /*ES=0;*/                                 //disable int. from serial
                }
              }
              D11_WrEp(D11_EP0IN_IDX,Buffer,1,0);
            };break;
            case VENDOR_STOP_ULAN:{
              uLanConfigured=0;
              //ES=0;                                       //disable int. from serial
            };break;
            case VENDOR_IS_RUNNING_ULAN:{
              Buffer[0] = uLanConfigured;
              D11_WrEp(D11_EP0IN_IDX,Buffer,1,0);
              break;
            }
            case VENDOR_ARC_ON_ULAN:{                       // enable promiscuous mode
              ACC=1;
              //uL_FNCC(0x07);                              //enable
            };break;
            case VENDOR_ARC_OFF_ULAN:{                      // disable promiscuous mode
              ACC=0;
              //uL_FNCC(0x07);                              //disable
            };break;
            default:{
              D11_SetEpStall(D11_EP0IN_IDX);
            };break;
          }
        };break;
        default:{
          D11_SetEpStall(D11_EP0IN_IDX);
        };break;
      }
}

/**************************************************************************/
void D11GetIRQ() {
  unsigned wIrq;
  unsigned char byLtStat;
  unsigned char size;

  wIrq=D11_GetIrq();
  if (wIrq & D11_INT_BUSRESET) {
    D11_CfgInit();
  }
  if (wIrq & D11_INT_EP0IN) {
    byLtStat=D11_GetLtStat(D11_EP0IN_IDX);
    if (byLtStat & D11_LTSTAT_RXTXOK)
      WriteBufferToEndPoint0();
  }
  if (wIrq & D11_INT_EP0OUT) {
    byLtStat=D11_GetLtStat(D11_EP0OUT_IDX);
    if (byLtStat & D11_LTSTAT_RXTXOK) {	 //Check "Data Receive Success" for 1
      if (byLtStat & D11_LTSTAT_SETUP) { //Check "Setup Packet received" > ACK
        Process_EP0_OUT_Interrupt();
      } else
        D11_ClrEp(D11_EP0OUT_IDX);
    }
  }
  //EP1 prijem dat z PC (Zpravy k odeslani)
  if (wIrq & D11_INT_EP1IN) {
    byLtStat=D11_GetLtStat(D11_EP1IN_IDX);
    if (byLtStat & D11_LTSTAT_RXTXOK) {
      D11_SetEpStall(D11_EP1IN_IDX);
    }
  }
  if (wIrq & D11_INT_EP1OUT) {
    byLtStat=D11_GetLtStat(D11_EP1OUT_IDX);
    if (byLtStat & D11_LTSTAT_RXTXOK) {
      size=D11_RdEp((unsigned char *)&uLanBuffRx[0], D11_EP1OUT_IDX);
      if (BytesToRec2==-1) {
        BytesToRec2=uLanBuffRx[6]+uLanBuffRx[7]*0x100;
        msginfo.dadr =uLanBuffRx[0]&0x7F;
        msginfo.sadr =uLanBuffRx[1]&0x7F;
        msginfo.cmd  =uLanBuffRx[2];
        msginfo.flg  =uLanBuffRx[3];
        msginfo.stamp=uLanBuffRx[4];
	if(ul_newmsg(ul_fd,&msginfo)<0){
	  /* Error */ ;
        }
      } else {
        if (BytesToRec2>0) {
          if(ul_write(ul_fd,uLanBuffRx,size)!=size){
	    /* Error */ ;
          } 
          BytesToRec2-=size;
	}
      }	
      if (BytesToRec2<=0) {
        ul_freemsg(ul_fd_t ul_fd);
        D11_TxNull(D11_EP1IN_IDX);
        timer_tx_off=5;LED_TX=0;
        BytesToRec2=-1;
      }
    }
  }

  //EP2 vysilani dat do PC (Prijate zpravy)
  if (wIrq & D11_INT_EP2IN) {
    byLtStat=D11_GetLtStat(D11_EP2IN_IDX);
    if (byLtStat & D11_LTSTAT_RXTXOK) {
      WriteBufferToEndPoint2();
    }
  }
  if (wIrq & D11_INT_EP2OUT) {
    byLtStat=D11_GetLtStat(D11_EP2OUT_IDX);
    if (byLtStat & D11_LTSTAT_RXTXOK) {
      D11_SetEpStall(D11_EP2OUT_IDX);
    }
  }
  if (wIrq & D11_INT_EP3IN) {
    byLtStat=D11_GetLtStat(D11_EP3IN_IDX);
    if (byLtStat & D11_LTSTAT_RXTXOK) {
      D11_SetEpStall(D11_EP3IN_IDX);
    }
  }
  if (wIrq & D11_INT_EP3OUT) {
    byLtStat=D11_GetLtStat(D11_EP3IN_IDX);
    if (byLtStat & D11_LTSTAT_RXTXOK) {
      D11_SetEpStall(D11_EP3OUT_IDX);
    }
  }
}

int msc_adc_init(void);
long msc_adc_test(void);

void main(void) {

  P3=0xff;P1=0xff;
  /*P3_4=0;*/					// RX/TX dir
  TMOD=0x21;                                    //citac0-16bitovy,1-8bitovy(relod)
  TH0=TIMERH10ms;TL0=TIMERL10ms;                //nastav casovac0
  TCON=0x10;                                    //nul. priz. casov0,1;spust casov0
  IE=0;
  /*IE=0x02;*/                                      //povol preruseni casov0
  D11_CfgInit();
  //Default configuration
  ul_drv_set_adr(5);
  ul_drv_set_bdiv(5);
  ul_drv_init();uLanConfigured=1;               //uLan inicializace (musi byt!)
  ul_fd=ul_open(0,0);
  msc_adc_init();
  EA=1;
  //for(;;) msc_adc_test();
  BytesToSend2=BytesToRec2=-1;                   //zacatek prijmu a vysilani
  while(1) {
    D11GetIRQ();
    if ((uLanConfigured) && (BytesToSend2==-1)) {
      if(ul_acceptmsg(ul_fd,&msginfo)>=0){
        if (!(msginfo.flg && UL_BFL_PROC)) {
          timer_rx_off=5;LED_RX=0;
        }
        uLanBuffTx[0]=msginfo.dadr;     uLanBuffTx[1]=msginfo.sadr;
        uLanBuffTx[2]=msginfo.cmd;      uLanBuffTx[3]=msginfo.flg;
        uLanBuffTx[4]=msginfo.stamp;    uLanBuffTx[5]=0;
        uLanBuffTx[6]=msginfo.len%0x100;uLanBuffTx[7]=msginfo.len/0x100;
        BytesToSend2=msginfo.len;
        D11_WrEp(D11_EP2IN_IDX,uLanBuffTx,8,0); // zapis dat do EP
        if (BytesToSend2==0) {
          ul_i_close(ul_fd);                    // uzavreni zpravy
	  BytesToSend2=-1;
        }
      }
    }
  }
}
