//**************************************************************************
//                                   Petr Smolik
//**************************************************************************
#if !defined(__USBDESC_H)
        #define __USBDESC_H

#define STANDARD_DEVICE_REQUEST         0x00
#define STANDARD_INTERFACE_REQUEST      0x01
#define STANDARD_ENDPOINT_REQUEST       0x02
#define VENDOR_DEVICE_REQUEST           0x40
#define VENDOR_ENDPOINT_REQUEST         0x42

#define GET_STATUS                      0
#define CLEAR_FEATURE                   1
#define SET_FEATURE                     3
#define SET_ADDRESS                     5
#define GET_DESCRIPTOR                  6
#define SET_DESCRIPTOR                  7
#define GET_CONFIGURATION               8
#define SET_CONFIGURATION               9
#define GET_INTERFACE                   10
#define SET_INTERFACE                   11
#define SYNCH_FRAME                     12
#define VENDOR_INIT_ULAN                0

#define ENDPOINT_HALT                   0

#define TYPE_DEVICE_DESCRIPTOR          1
#define TYPE_CONFIGURATION_DESCRIPTOR   2
#define TYPE_STRING_DESCRIPTOR          3
#define TYPE_INTERFACE_DESCRIPTOR       4
#define TYPE_ENDPOINT_DESCRIPTOR        5
#define TYPE_HID_DESCRIPTOR             0x21

#define USB_ENDPOINT_TYPE_CONTROL       0x00
#define USB_ENDPOINT_TYPE_ISOCHRONOUS   0x01
#define USB_ENDPOINT_TYPE_BULK          0x02
#define USB_ENDPOINT_TYPE_INTERRUPT     0x03

typedef struct {
        unsigned char bmRequestType;
        unsigned char bRequest;
        unsigned wValue;
        unsigned wIndex;
        unsigned wLength;
} USB_SETUP_REQUEST, *PUSB_SETUP_REQUEST;

typedef struct {
                unsigned char bLength;
                unsigned char bDescriptorType;
                unsigned bcdUSB;
                unsigned char bDeviceClass;
                unsigned char bDeviceSubClass;
                unsigned char bDeviceProtocol;
                unsigned char bMaxPacketSize0;
                unsigned idVendor;
                unsigned idProduct;
                unsigned bcdDevice;
                unsigned char iManufacturer;
                unsigned char iProduct;
                unsigned char iSerialNumber;
                unsigned char bNumConfigurations;
} USB_DEVICE_DESCRIPTOR, *PUSB_DEVICE_DESCRIPTOR;

typedef struct {
                unsigned char bLength;
                unsigned char bDescriptorType;
                unsigned char bEndpointAddress;
                unsigned char bmAttributes;
                unsigned wMaxPacketSize;
                unsigned char bInterval;
} USB_ENDPOINT_DESCRIPTOR, *PUSB_ENDPOINT_DESCRIPTOR;

typedef struct {
                unsigned char bLength;
                unsigned char bDescriptorType;
                unsigned wTotalLength;
                unsigned char bNumInterfaces;
                unsigned char bConfigurationValue;
                unsigned char iConfiguration;
                unsigned char bmAttributes;
                unsigned char MaxPower;
} USB_CONFIGURATION_DESCRIPTOR, *PUSB_CONFIGURATION_DESCRIPTOR;

typedef struct {
                unsigned char bLength;
                unsigned char bDescriptorType;
                unsigned char bInterfaceNumber;
                unsigned char bAlternateSetting;
                unsigned char bNumEndpoints;
                unsigned char bInterfaceClass;
                unsigned char bInterfaceSubClass;
                unsigned char bInterfaceProtocol;
                unsigned char iInterface;
} USB_INTERFACE_DESCRIPTOR, *PUSB_INTERFACE_DESCRIPTOR;

typedef struct {
                unsigned char bLength;
                unsigned char bDescriptorType;
                unsigned wHIDClassSpecComp;
                unsigned char bCountry;
                unsigned char bNumDescriptors;
                unsigned char b1stDescType;
                unsigned w1stDescLength;
} USB_HID_DESCRIPTOR, *PUSB_HID_DESCRIPTOR;

typedef struct {
        USB_CONFIGURATION_DESCRIPTOR ConfigDescriptor;
        USB_INTERFACE_DESCRIPTOR InterfaceDescriptor;
        USB_ENDPOINT_DESCRIPTOR EndpointDescriptor0;
        USB_ENDPOINT_DESCRIPTOR EndpointDescriptor1;
} USB_CONFIG_DATA, *PUSB_CONFIG_DATA;

typedef struct {
        unsigned char bLenght;
        unsigned char bDescriptorType;
        char     bString[40];
} STRING_DESCRIPTOR, *PSTRING_DESCRIPTOR;

typedef struct {
        unsigned char bLenght;
        unsigned char bDescriptorType;
        char     bString[140];
} STRING_DESCRIPTOR_LONG, *PSTRING_DESCRIPTOR_LONG;

typedef struct {
        unsigned char bLenght;
        unsigned char bDescriptorType;
        unsigned wLANGID0;
} LANGID_DESCRIPTOR, *PLANGID_DESCRIPTOR;

#define SWAPWORD(x)   ( (((x) << 8) & 0xFF00) | (((x) >> 8) & 0x00FF) )

USB_DEVICE_DESCRIPTOR code DeviceDescriptor = {
    sizeof(USB_DEVICE_DESCRIPTOR), /* bLength */
    TYPE_DEVICE_DESCRIPTOR,        /* bDescriptorType */
    SWAPWORD(0x0110),               // usb version = 01.10 (binary coded decimal)
    0,                             /* bDeviceClass */
    0,                             /* bDeviceSubclass */
    0,                             /* bDeviceProtocol */
    8,                             /* bMaxPacketSize 8 Bytes */
    SWAPWORD(0x045E),              // vendor id code (VID)  /* Intel 82930 USB Bulk IO Test Board */
//    SWAPWORD(0x4343),              // device release number = 01.00 (bcd)
    SWAPWORD(0x930A),              // product id code (PID)
//    SWAPWORD(0x0001),              // product id code (PID)
    SWAPWORD(0x0001),              // device release number = 01.00 (bcd)
    1,                             /* iManufacturer String Index */
    2,                             /* iProduct String Index */
    0,                             /* iSerialNumber String Index */
    1                              /* bNumberConfigurations */
};

USB_CONFIG_DATA code ConfigurationDescriptor = {
    {                              /* configuration descriptor */
    sizeof(USB_CONFIGURATION_DESCRIPTOR), /* bLength */
    TYPE_CONFIGURATION_DESCRIPTOR, /* bDescriptorType */
    SWAPWORD(sizeof(USB_CONFIG_DATA)),/* wTotalLength */
    1,                             /* bNumInterfaces */
    1,                             /* bConfigurationValue */
    3,                             /* iConfiguration String Index */
    0x80,                          /* bmAttributes Bus Powered, No Remote Wakeup */
    0x32                           /* bMaxPower, 100mA */
    },
    {                              /* interface descriptor */
    sizeof(USB_INTERFACE_DESCRIPTOR), /* bLength */
    TYPE_INTERFACE_DESCRIPTOR,     /* bDescriptorType */
    0,                             /* bInterface Number */
    0,                             /* bAlternateSetting */
    2,                             /* bNumEndpoints */
    0xFF,                          /* bInterfaceClass (Vendor specific) */
    0xFF,                          /* bInterfaceSubClass */
    0xFF,                          /* bInterfaceProtocol */
    4                              /* iInterface String Index */
    },
    {                              /* endpoint descriptor */
    sizeof(USB_ENDPOINT_DESCRIPTOR),  /* bLength */
    TYPE_ENDPOINT_DESCRIPTOR,      /* bDescriptorType */
    0x01,                          /* bEndpoint Address EP1 OUT */
    0x02,                          /* bmAttributes - BULK */
    SWAPWORD(0x0008),              /* wMaxPacketSize */
    0x00                           /* bInterval */
    },
    {                              /* endpoint descriptor */
    sizeof(USB_ENDPOINT_DESCRIPTOR),  /* bLength */
    TYPE_ENDPOINT_DESCRIPTOR,      /* bDescriptorType */
    0x82,                          /* bEndpoint Address EP2 IN */
    0x02,                          /* bmAttributes - BULK */
    SWAPWORD(0x0008),              /* wMaxPacketSize */
    0x00                           /* bInterval */
    }
};

LANGID_DESCRIPTOR code LANGID_Descriptor = { /* LANGID String Descriptor Zero */
    sizeof(LANGID_DESCRIPTOR),          /* bLenght */
    TYPE_STRING_DESCRIPTOR,             /* bDescriptorType */
    SWAPWORD(0x0409)                    /* LANGID US English */
};

STRING_DESCRIPTOR code Manufacturer_Descriptor = { /* ManufacturerString 1 */
    sizeof(STRING_DESCRIPTOR),                     /* bLenght */
    TYPE_STRING_DESCRIPTOR,                        /* bDescriptorType */
      'P',0,
      'e',0,
      't',0,
      'r',0,
      ' ',0,
      'S',0,
      'm',0,
      'o',0,
      'l',0,
      'i',0,
      'k',0
};

STRING_DESCRIPTOR code Product_Descriptor = { /* ProductString */
    sizeof(STRING_DESCRIPTOR),                /* bLenght */
    TYPE_STRING_DESCRIPTOR,                   /* bDescriptorType */
    {
      'U',0,
      'S',0,
      'B',0,
      '2',0,
      'u',0,
      'L',0,
      'a',0,
      'n',0,
      ' ',0,
      'c',0,
      'o',0,
      'n',0,
      'v',0,
      'e',0,
      'r',0,
      't',0,
      'o',0,
      'r',0
    }
};

STRING_DESCRIPTOR code Configuration_Descriptor = { /* ConfigurationString */
    sizeof(STRING_DESCRIPTOR),                /* bLenght */
    TYPE_STRING_DESCRIPTOR,                   /* bDescriptorType */
    {
      'd',0,
      'e',0,
      'f',0,
      'a',0,
      'u',0,
      'l',0,
      't',0
    }
};

STRING_DESCRIPTOR code Interface_Descriptor = { /* InterfaceString */
    sizeof(STRING_DESCRIPTOR),                /* bLenght */
    TYPE_STRING_DESCRIPTOR,                   /* bDescriptorType */
    {
      'd',0,
      'e',0,
      'f',0,
      'a',0,
      'u',0,
      'l',0,
      't',0
    }
};

STRING_DESCRIPTOR_LONG code EP1O_Descriptor = { /* EP1 OUT descriptor */
    sizeof(STRING_DESCRIPTOR_LONG),             /* bLenght */
    TYPE_STRING_DESCRIPTOR,                     /* bDescriptorType */
    {
      'E',0,
      'P',0,
      '1',0,
      'O',0,
      'U',0,
      'T',0,
      '-',0,
      's',0,
      'e',0,
      'n',0,
      'd',0,
      ' ',0,
      'm',0,
      'e',0,
      's',0,
      's',0,
      'a',0,
      'g',0,
      'e',0,
      '-',0,
      'd',0,
      'a',0,
      'd',0,
      'r',0,
      ',',0,
      's',0,
      'a',0,
      'd',0,
      'r',0,
      ',',0,
      'c',0,
      'm',0,
      'd',0,
      ',',0,
      'f',0,
      'l',0,
      'g',0,
      ',',0,
      's',0,
      't',0,
      'a',0,
      'm',0,
      'p',0,
      ',',0,
      'f',0,
      'r',0,
      'e',0,
      'e',0,
      ',',0,
      'l',0,
      'e',0,
      'n',0,
      '-',0,
      'l',0,
      'o',0,
      ',',0,
      'l',0,
      'e',0,
      'n',0,
      '-',0,
      'h',0,
      'i',0,
      ',',0,
      'd',0,
      'a',0,
      't',0,
      'a',0
    }
};

STRING_DESCRIPTOR_LONG code EP2I_Descriptor = { /* EP2 IN descriptor */
    sizeof(STRING_DESCRIPTOR_LONG),             /* bLenght */
    TYPE_STRING_DESCRIPTOR,                     /* bDescriptorType */
    {
      'E',0,
      'P',0,
      '2',0,
      'I',0,
      'N',0,
      '-',0,
      'r',0,
      'e',0,
      'c',0,
      'e',0,
      'i',0,
      'v',0,
      'e',0,
      ' ',0,
      'm',0,
      'e',0,
      's',0,
      's',0,
      'a',0,
      'g',0,
      'e',0,
      '-',0,
      'd',0,
      'a',0,
      'd',0,
      'r',0,
      ',',0,
      's',0,
      'a',0,
      'd',0,
      'r',0,
      ',',0,
      'c',0,
      'm',0,
      'd',0,
      ',',0,
      'f',0,
      'l',0,
      'g',0,
      ',',0,
      's',0,
      't',0,
      'a',0,
      'm',0,
      'p',0,
      ',',0,
      'f',0,
      'r',0,
      'e',0,
      'e',0,
      ',',0,
      'l',0,
      'e',0,
      'n',0,
      '-',0,
      'l',0,
      'o',0,
      ',',0,
      'l',0,
      'e',0,
      'n',0,
      '-',0,
      'h',0,
      'i',0,
      ',',0,
      'd',0,
      'a',0,
      't',0,
      'a',0
    }
};

#endif
