/*****************************************************/
/***   Module : command shell functions            ***/
/***   Author : Roman Bartosinski (C) 28.04.2002   ***/
/***   Modify : 08.08.2002                         ***/
/*****************************************************/

#include "hal.h"
#include "commands.h"
#include "stdio_d.h"

/* next are needed for some commands */
#include "serial.h"
#include "pdiusb.h"
#include "usb.h"

#ifdef COMMANDS_PLUGIN

  xdata unsigned char cmdCntParam, cmdIndex;
  xdata unsigned int cmdParams[MAXCMDPARAM];
  xdata volatile unsigned int cLen;
  xdata volatile char cLine[CBUFLEN];

#define HELPTABSPACE 10

#define CMDDEBUG 4
#define CMDECHO  3

/* commands & its descriptions */
  code const COMMAND cmd_help     = { "help"   , "list commands", cmd_help_fnc};
  code const COMMAND cmd_ver      = { "ver"    , "print ver strings", cmd_ver_fnc};

  code const COMMAND cmd_echo     = { "echo"   , "(val) get/set ECHO flag", cmd_switched_flag_fnc}; //cmd_echo_flag_fnc};
#ifdef DEBUG
  code const COMMAND cmd_debug    = { "debug"  , "(val) get/set DEBUG flag", cmd_switched_flag_fnc}; //cmd_debug_flag_fnc};
#endif
  code const COMMAND cmd_swreset  = { "swreset", "software reset", cmd_sw_reset_fnc};

  code const COMMAND cmd_d12id    = { "d12id",   "print PDIUSBD12 chip ID", cmd_d12_chip_id_fnc};
  code const COMMAND cmd_d12gfrnum= { "d12fnum"   ,"print current USB frame number", cmd_d12_get_frame_number};
  code const COMMAND cmd_d12gsusp = { "d12susp"   ,"print current USB suspend state", cmd_d12_get_suspend};

  code const COMMAND cmd_get      = { "get",     "get value from req.address", cmd_get_byte_fnc};
  code const COMMAND cmd_set      = { "set",     "set value at req.address", cmd_set_byte_fnc};
  code const COMMAND cmd_upload   = { "upload",  "upload data to memory by terminal", cmd_upload_fnc};
  code const COMMAND cmd_download = { "download","download data from memory by terminal", cmd_download_fnc};
  code const COMMAND cmd_dump     = { "dump",    "dump piece of memory", cmd_dump_fnc};
  
  code const COMMAND cmd_usbon    = { "usbon",   "Connect USB and start communicate with host", USBConnect};
  code const COMMAND cmd_usboff   = { "usboff",  "Disconnect USB from host", USBDisconnect};


/* array of all known commands */
  code const PCOMMAND cmdArray[] = {
    &cmd_help, 
    &cmd_ver,    
    &cmd_swreset,
    &cmd_echo,
#ifdef DEBUG
    &cmd_debug,
#endif
    &cmd_get,
    &cmd_set,
    &cmd_upload,
    &cmd_download,
    &cmd_dump,
    &cmd_d12id,
    &cmd_d12gfrnum,
    &cmd_d12gsusp,
    &cmd_usbon,
    &cmd_usboff,
    NULL
  };

/*******************************************************************/

// *** funkce pro praci s prikazi
  void clear_command_line( void) critical {
    cLen = 0;
    pflags.Command = 0;
    cLine[0] = 0;
    pflags.CommandExec = 0;
  }

  unsigned char get_line_parameters(xdata char *Line) { // parameters are in parentheses and separated by comma
    unsigned char chr;
    xdata unsigned char minus = 0;
    xdata char *pNow;
    xdata unsigned char i = 0, type = 0;
    xdata unsigned int Value = 0;

    pNow = (xdata char *) Line;

    while(( *pNow != '(') && ( *pNow != ' ')) { // search beginning of parameters
      if (!(*pNow)) return 0;
      pNow++;
    }
    pNow++;
    while( 1) {
      chr = *pNow; 
      pNow++;
      if (( chr == ',') || ( chr == ')') || ( chr == ' ') || ( chr == 0)) {      // next parameter
        if ( type) {
          if ( minus)
            Value = (unsigned int)(-((signed int)Value));
          cmdParams[i] = Value; 
#ifdef DEBUG
          if ( pflags.Debug) printf1( " cmd parameter : 0x%X\n", Value);
#endif
          i++; minus =0;
        }
        type = 0; 
        Value = 0;
        if (( chr == ',') || ( chr == ' '))
          continue;
        else
          break;
      }
      if (i == MAXCMDPARAM) break;
      switch( type) {      
      case 0:
        if (( chr >= '1') && ( chr <= '9')) {                   // decimal
          type = 1; Value = chr - '0';
        }
        if ((( chr | 0x20) >= 'a') && (( chr | 0x20) <= 'f')) { // hexadecimal
          type = 2; Value = ( chr | 0x20) - 'a' + 10;
        }
        if ( chr == '-')                                        // negative value
          minus = 1;
        if ( chr == 39)                                         // char
          type = 3;
        if ( chr == '"')                                        // string
          type = 4;
        if ( chr == '0') {
          if (( *pNow | 0x20) == 'x') {          // hexadecimal
            type = 2; 
          } else {                                              // decimal
            type = 1;
          }
        }             
        if (( chr == 'T') || ( chr == 't')) {                   // boolean with value = 1
          Value = 1; type = 15;
        }
        if (( chr == 'F') || ( chr == 'f')) {                   // boolean with value = 0
          Value = 0; type = 15;
        }
        break;
      case 1:                                         // decimal number
        if (( chr >= '0') && ( chr <= '9')) {
          Value = (( Value << 3) + ( Value << 1)) + ( chr - '0');
        }
        break;
      case 2:                                         // hexadecimal number
        chr |= 0x20;
        if (( chr >= '0') && ( chr <= '9')) {
          Value = ( Value << 4);
          Value += ( chr - '0'); }
        if (( chr >= 'a') && ( chr <= 'f')) {
          Value = ( Value << 4);
          Value += ( chr - 'a' + 10); }
        break;
      case 3:                                         // char
        Value = chr; type = 15; // -> only first char
        break;        
      case 4:                                         // string
        Value = ((unsigned int) pNow) - 1; type = 5; 
        break;        
      case 5:                                         // -> made end of string from ending quote
        if ( chr == '"') {        
          pNow--; 
          *pNow = 0; 
        }
        break;
      }
      pNow++;
    }
    return i;
  }

  void execute_command( void) {
    code unsigned char *str1;
    xdata unsigned char *str2;

    if ( pflags.Command && !pflags.CommandExec) {
      pflags.CommandExec = 1;
      cmdIndex = 0;
      while((unsigned int) cmdArray[cmdIndex]) {
        str1 = cmdArray[cmdIndex]->sCmd;
        str2 = cLine;
        while ( *str1 && (*str1 == *str2)) {
          str1++; str2++;
        }
        if ( *str1 == 0) { // cely retezec
          cmdCntParam = get_line_parameters(cLine);
#ifdef DEBUG
      if ( pflags.Debug) 
        printf2( "cmdIdx=%d,CntPar=%d\n", cmdIndex, cmdCntParam);
#endif
          cmdArray[cmdIndex]->fnc();
          clear_command_line();
          return;
        } else {
          cmdIndex++;
        }
      }
      printf1( "Bad cmd!(help) - %s\n", cLine);
      clear_command_line();
    }
  }

/*** commands functions ***/

  void help_fnc_params( void) {
    printf( " Parameters are (t,a[,v|c])\n");
    printf( "    t\n'R'-RAM mem\n'S'-SFR mem\n'E'-ext.mem\n'C'-code mem\n'D'-data mem\n");
    printf( "    a = address\n    v = byte value\n    c = count of readed/writed byte\n");
  }

  void cmd_help_fnc( void) {
    char i = 0, j;
    printf1("%s\n    'cmd' [(param1,param2,param3)]\n",FWID);
    while ((unsigned int) cmdArray[i]) {
      printf1("%s", cmdArray[i]->sCmd);
      j = chstrlen(cmdArray[i]->sCmd);
      while ( j < HELPTABSPACE) {
        putchar( ' '); j++;
      }
      printf1("- %s\n", cmdArray[i]->sDesc);
      i++; //str += sizeof( command);
    }
  }

  void cmd_ver_fnc( void) {
    printf3( "%s\n >RS232: %s\n >USB  : %s\n", FWID, COMID, USBID);
  }


  void cmd_switched_flag_fnc( void) {
    char flag;
    if ( cmdCntParam) {
      if ( cmdParams[0]) flag = 1;
      else flag = 0;
#ifdef DEBUG
      if ( cmdIndex == CMDDEBUG) pflags.Debug = flag;
      else 
#endif
        if ( cmdIndex == CMDECHO) pflags.Echo = flag;
    } else {
#ifdef DEBUG
      if ( cmdIndex == CMDDEBUG) flag = pflags.Debug;
      else 
#endif
        if ( cmdIndex == CMDECHO) flag = pflags.Echo;

      printf2( "%s=%d\n", cmdArray[cmdIndex]->sCmd, flag);
    }
  }
  
  void cmd_sw_reset_fnc( void) _naked {
    _asm
      mov  dptr,#_cmdCntParam
      movx a,@dptr
      jz   0999$
      mov  dptr,#_cmdParams
      movx a,@dptr
      mov  dptr,#0xfffe        // ExtMark ( for both cases
      cjne a,#0x45,0888$
      mov  a,#0x69             // RUN_EXTERNAL_PROGRAM sign 'iE'
      movx @dptr,a
      inc  dptr
      mov  a,#0x45
      movx @dptr,a
      sjmp 0999$
    0888$:
      clr  a
      movx @dptr,a
      inc  dptr
      movx @dptr,a
    0999$:
      ljmp 0x0000
    _endasm;
  }



// return byte from memory selected by cmdParams[0]
  unsigned char get_one_byte( unsigned int adr) {
    switch( (unsigned char) cmdParams[0]) {
      case 'E': 
      case 'e': return *((xdata unsigned char *) adr);
      case 'C': 
      case 'c': return *((code unsigned char *) adr); 
      case 'R': 
      case 'r': return *((idata unsigned char *) adr);
//        case 'S': codesfr[0] = 0xE5; codesfr[1] = (char) adr; codesfr[2] = 0x22; // it means : mov A,adr;ret;
      default: return 0;
    }
  }


// *** data transfer functions  
  void cmd_get_byte_fnc( void) {
    register unsigned char out;

    if ( cmdCntParam > 1) {
      out = get_one_byte( cmdParams[1]);
#ifdef DEBUG
      if ( pflags.Debug) {
        printf1( "Value on 0x%X is ", cmdParams[1]);
      }
#endif
      printf1("0x%X\n",out);
    } else
      help_fnc_params();
  }

  void cmd_set_byte_fnc( void) {
    register unsigned char type;
    register unsigned int adr;
    register unsigned char byte;

    if ( cmdCntParam > 2) {
      type = (unsigned char) cmdParams[0]; 
      adr = cmdParams[1];
      byte = (unsigned char) cmdParams[2];
#ifdef DEBUG
      if ( pflags.Debug)
        printf2( "Set value on 0x%X to 0x%X",adr,byte);
#endif   
      switch( type) {
        case 'E': *((xdata unsigned char *) adr) = byte; break;
        case 'R': *((idata unsigned char *) adr) = byte; break;
//        case 'D':        // not implement yet - to eeprom data memory
//        case 'S': break;
      }
    } else
      help_fnc_params();
  }

  void cmd_upload_fnc( void) {
    register unsigned char byte;
    unsigned int adr, cnt, sum;
    if ( cmdCntParam > 2) {
      adr = cmdParams[1]; cnt = cmdParams[2];
      sum = 0;
      printf( "W\n");
      sflags.Terminal = 1;
      while ( cnt) {
        if ( srCnt) {
          byte = ser_getc();
          sum += byte;
          if (((unsigned char) cmdParams[0]) == 'R') {     // into RAM
            *((idata unsigned char *) adr) = byte;
          } else {                // otherwise External
            *((xdata unsigned char *) adr) = byte;
          }
          adr++; cnt--;
        }
      }
      sflags.Terminal = 0;
      printf1( "S%X\n",sum);
//      WriteStringLn("OK");
    } else
      help_fnc_params();
  }

  void cmd_download_fnc( void) {
    register unsigned int adr;
    register unsigned int cnt;

    if ( cmdCntParam > 2) {
//      type = (unsigned char) cmdParams[0]; 
      adr = cmdParams[1]; cnt = cmdParams[2];
      sflags.Terminal = 1;
      while ( cnt) {
        if ((SER_SBLEN - ssCnt) != 0) {
          ser_putc( get_one_byte( adr));
          adr++; cnt--;
        }
      }
      sflags.Terminal = 0;
      printf("\n");
    } else
      help_fnc_params();
  }

  void cmd_dump_fnc( void) {
    register unsigned char byte, lcnt;
//    register unsigned int adr;
    register unsigned int cnt;
#ifdef DEBUG
    unsigned char dbgflg = pflags.Debug;
    pflags.Debug = 0;
#endif
    if ( cmdCntParam > 2) {
//      adr = cmdParams[1]; 
      cnt = cmdParams[2];
      lcnt = 0;
      while ( cnt) {
        if ( !lcnt) printf1("\n%X | ",cmdParams[1]);

        byte = get_one_byte( cmdParams[1]++);
        printf1("%X ", byte);
        cnt--; lcnt++;
        if ( lcnt == 16) {
          cmdParams[1] -= 16;
          printf("| ");
          while ( lcnt--) {
            byte = get_one_byte( cmdParams[1]++);
            if ( byte > 0x1f) {
              printf1("%c",byte);
            } else {
              printf(" ");
            }            
          }
          lcnt = 0;
        }
      }
      printf("\n");
    } else
      help_fnc_params();

#ifdef DEBUG
    pflags.Debug = dbgflg;
#endif
  }
  void cmd_setbaud_fnc( void) {
    printf( "Set New BaudRate\n");
    if ( cmdCntParam) {
      SerialSetBaudRate( cmdParams[0]);
    }
  }


// *** D12 commands
  void cmd_d12_chip_id_fnc( void) {
    printf1( "D12 id=0x%X\n", pdiGetChipID());
  }
  void cmd_d12_get_frame_number( void) {
    printf1( "D12 fNum=%d\n", pdiGetFrameNumber());
  }
  void cmd_d12_get_suspend( void) {
    printf1( "D12 suspend=%d\n", bD12Suspend);
  }


#endif // #ifdef COMMANDS_PLUGIN
// end of file ...
