/*****************************************************/
/***   Module : command shell - header file        ***/
/***   Author : Roman Bartosinski (C) 28.04.2002   ***/
/***   Modify : 08.08.2002                         ***/
/*****************************************************/

#ifndef _COMMAND_LINE_MODULE_
  #define _COMMAND_LINE_MODULE_

  #ifdef COMMANDS_PLUGIN 

/* global variables for command line & parameters */
    #define CBUFLEN     128    /* size of command line buffer */
    #define MAXCMDPARAM 3      /* max.count of remembered parameters from command line */

    extern xdata volatile char cLine[CBUFLEN];
    extern xdata volatile unsigned int cLen;
    extern xdata unsigned char cmdCntParam, cmdIndex;
    extern xdata unsigned int cmdParams[MAXCMDPARAM];


/*** functions for work with commands ***/
    void clear_command_line(void) critical;
    unsigned char get_line_parameters( xdata const char *Line);
    void execute_command( void);

/* structure for command definitions */
    typedef struct _tag_command {
      code const unsigned char *sCmd;
      code const char *sDesc;
      void ( *fnc)(void);
    } COMMAND, *PCOMMAND;

/*** commands functions ***/
  /* main */
    void cmd_help_fnc( void);
    void cmd_ver_fnc( void);
    void cmd_terminal_fnc( void);
    void cmd_sw_reset_fnc( void) _naked;
    void cmd_switched_flag_fnc( void);
  /* serial data transfer */
    void cmd_get_byte_fnc( void);
    void cmd_set_byte_fnc( void);
    void cmd_upload_fnc( void);
    void cmd_download_fnc( void);
    void cmd_dump_fnc( void);
    void cmd_setbaud_fnc( void);
  /* direct PDIUSBD12 */
    void cmd_d12_chip_id_fnc( void);
    void cmd_d12_get_frame_number( void);
  /* usb */
    void cmd_usb_send_fnc( void);

  #endif /* COMMANDS_PLUGIN */

#endif
