/********************************************************/
/***   Module : Hardware abstract layer & main vars   ***/
/***   Author : Roman Bartosinski (C) 28.04.2002      ***/
/***   Modify : 22.12.2002 - transform to 'base'      ***/
/********************************************************/

#ifndef _HARDWARE_ABSTRACTION_LAYER
 #define _HARDWARE_ABSTRACTION_LAYER

 #include <8052.h>

 #ifndef ULONG
   #define ULONG unsigned long
 #endif
 #ifndef USHORT
   #define USHORT unsigned short
 #endif
 #ifndef UCHAR
   #define UCHAR unsigned char
 #endif

 #define NULL 0x0000

/* program prelozen s moznosti debug vypisu */
 #define DEBUG
/* preklad s moznosti COM prikazoveho radku */
 #define COMMANDS_PLUGIN
/* preklad se zapnutim USB po startu ( znovuzapnuti - vypne a zapne) */
// #define START_WITH_USB


//#define CPU_SYS_KHZ 11059
#define CPU_SYS_KHZ 18432

/* hardware dependent bits */
// sbit at 0x90 LEDr;        // P1.0
// sbit at 0x91 LEDg;        // P1.1
// sbit at 0x93 bD12Suspend; // P1.3

 sbit at 0xB3 LEDr;        // P3.3
 sbit at 0xB3 LEDg;        // P3.3
 sbit at 0xB3 bD12Suspend; // P3.3

/* PDIUSBD11 connection */

#define D11_REG_CMD         0x36
#define D11_REG_DATA_WRITE  0x34
#define D11_REG_DATA_READ   0x35

 sbit at 0x90 SDAin;         // P1.0
 sbit at 0x91 SDAout;        // P1.1
 sbit at 0xB4 SCLout;        // P3.4
 sbit at 0xB2 pdiIRQ;        // P3.2

/* FOR VECTORS - Begin */
 xdata at 0xFFFE unsigned int ExtMark; /* if there is saved 0x69,0x45 ('i','E') - program will be continue directly to external */
 #define RUN_EXTERNAL_PROGRAM 0x4569

 #define PRE_MUL_CHAR #8
 #define VAL_REG_ADDR(bank,idx) bank*8+idx
 #define INTERRUPT_PRE( bank) \
    _asm             \
      ar0 = bank*8+0x00 \
      ar1 = bank*8+0x01 \
      ar2 = bank*8+0x02 \
      ar3 = bank*8+0x03 \
      ar4 = bank*8+0x04 \
      ar5 = bank*8+0x05 \
      ar6 = bank*8+0x06 \
      ar7 = bank*8+0x07 \
      push acc       \
      push b         \
      push dpl       \
      push dph       \
      push psw       \
      mov psw, PRE_MUL_CHAR * bank \
    _endasm
 #define INTERRUPT_POST() \
    _asm      \
      pop psw \
      pop dph \
      pop dpl \
      pop b   \
      pop acc \
      reti    \
    _endasm

#define PRE_VAL_CHAR  #
#define VAL_LJMP_CODE #0x02  // ljmp code
/* pozor adresa skoku musi byt ulozena Hi,Lo */
 #define REGISTER_INT( addr, fnc) \
    _asm                          \
      mov dptr,addr               \
      mov a, VAL_LJMP_CODE        \
      movx @dptr,a                \
      inc dpl                     \
      mov a,fnc >> 8              \
      movx @dptr,a                \
      inc dpl                     \
      mov a,fnc                   \
      movx @dptr,a                \
    _endasm

#define RST_INT_ADDR #0x8400
#define EX0_INT_ADDR #0x8403
#define T0_INT_ADDR  #0x840B
#define EX1_INT_ADDR #0x8413
#define T1_INT_ADDR  #0x841B
#define SER_INT_ADDR #0x8423
#define T2_INT_ADDR  #0x842B

/* FOR VECTORS - End */

// set/clear timer interrupt flag
  #define tcli _asm clr ET0 _endasm;
  #define tsti _asm setb ET0 _endasm;
// set/clear serial interrupt flag
  #define scli _asm clr ES _endasm;
  #define ssti _asm setb ES _endasm;
// set/clear usb (ext0) interrupt flag
  #define ucli _asm clr EX0 _endasm;
  #define usti /*_asm setb EX0 _endasm*/;
// help instruction for inputb,outputb ( SDCC ;-) )
  #define outportb(p,v) p=v
  #define inportb(p) p

#if 0 // keil
  #define SWAP(x) ((((x) & 0xFF) << 8) | (((x) >> 8) & 0xFF))
#else // sdcc
  #define SWAP(x) (x)
#endif

  #define MSB(x)  (unsigned char)(((x) >> 8) & 0xFF)
  #define LSB(x)  (unsigned char)((x) & 0xFF)


/* program flags - Debug, Input/Output, ... */
  typedef struct appl_flags {
    unsigned char Echo:1;        /* received char is send as echo */
    unsigned char Debug:1;       /* send debug info */
    unsigned char Command:1;     /* command line */
    unsigned char CommandExec:1; /* command is executed */
  } appl_flags_t;

  extern appl_flags_t pflags;

/* main string declarations */
  extern code const char FWID[];
  extern code const char COMID[];
  extern code const char USBID[];

#endif
