/*********************************************************/
/***   Module : PDIUSB D11,H11,H11A,D12 - implement.   ***/
/***   Author : Roman Bartosinski (C) 03.10.2002       ***/
/***   Description : Integrate common functions for    ***/
/***                PDIUSBD11,PDIUSBD12,PDIUSBH11(old) ***/
/***                PDIUSBH11A in Single/Multiple mode ***/
/***                 to one common file.               ***/
/***   Modify : 10.10.2002 - add H11                   ***/
/***            13.10.2002 - add spec.fnc for 'using'  ***/
/*********************************************************/

#include "hal.h"
#include "pdiusb.h"

  #if defined(PDI_CMD_RWD_INTERNAL) && !defined(PDIUSBD12)
   #include "i2c.h"
  #endif
// for internal data buffer
// all data communication with pdiusb is through this buffer (exclude write/read buffer, GetInterrupt and GetLastTransStatus)
unsigned char pdiData[2];


/*********************************************************/
// Function for read and write from/into chip

 #if defined(PDI_CMD_RWD_INTERNAL)
  #if defined(PDIUSBD12) // parallel interface

   void pdiSendCommand( unsigned char byCmd) {
     *((volatile unsigned char xdata*) PDIUSB_COMMAND_ADDR) = byCmd;
   }
   unsigned char pdiReadData( unsigned char byCount, unsigned char *pbyData) {
     unsigned char out = byCount;
     while (byCount--) {
       *pbyData++ = *((volatile unsigned char xdata*) PDIUSB_READ_DATA_ADDR);
     }
     return out;
   }
   void pdiWriteData( unsigned char byCount, unsigned char *pbyData) {
     while (byCount--) {
       *((volatile unsigned char xdata*) PDIUSB_WRITE_DATA_ADDR) = *pbyData++;
     }
   }

#if defined(PDI_USE_USING)
   unsigned short pdiIntCmdReadData( unsigned char byCmd, unsigned char byShort) _PDI_USING {
     unsigned char i[2];
     *((volatile unsigned char xdata*) PDIUSB_COMMAND_ADDR) = byCmd;
     i[0] = *((volatile unsigned char xdata*) PDIUSB_READ_DATA_ADDR);
     if ( !byShort) {
      i[1] = 0;
      return i[0];
     }
     i[1] = *((volatile unsigned char xdata*) PDIUSB_READ_DATA_ADDR);
     return (((unsigned short) i[1]) << 8) + i[0];
   }
#endif

  #else // serial interface iic
   void pdiSendCommand( unsigned char byCmd) {
     I2C_Write( D11_REG_CMD, &byCmd, 1);
   }
   unsigned char pdiReadData( unsigned char byCount, unsigned char *pbyData) {
     I2C_Read( D11_REG_DATA_READ, pbyData, byCount);
     return byCount;
   }
   void pdiWriteData( unsigned char byCount, unsigned char *pbyData) {
     I2C_Write( D11_REG_DATA_WRITE, pbyData, byCount);
   }
#if defined(PDI_USE_USING)
   unsigned short pdiIntCmdReadData( unsigned char byCmd, unsigned char byShort) _PDI_USING {
   }
#endif

  #endif
 #endif

/*********************************************************/
/*********************************************************/
// internal function (mainly for SDCC)
   unsigned char pdiReadInt( unsigned char byCount) {
     return pdiReadData( byCount, pdiData);
   }
   void pdiWriteInt( unsigned char byCount) {
     pdiWriteData( byCount, pdiData);
   }

/*********************************************************/
/*********************************************************/
// PDIUSB common commands

#if defined(PDIUSBH11) || defined(PDIUSBH11A)

 /*********************************************************/
 // pdiSetHUBAddressEnable
 // enable HUB function and set address (byAddress is 0-0x7F, byEnable is 0 or 1)
   void pdiSetHUBAddressEnable( unsigned char byAddress, unsigned char byEnable) {
     pdiData[0] = (byAddress & 0x7F) | (byEnable << 7);
     pdiSendCommand( PDI_CMD_HUB_ENB_ADDR);
     pdiWriteInt( 1);
   }

#endif
#if !defined(PDIUSBH11A_MULTPLE) // D11,D12,H11,H11A_S(emb.fnc)

 /*********************************************************/
 // pdiSetAddressEnable
 // Enable function and set address (byAddress is 0-0x7F, byEnable is 0 or 1)
   void pdiSetAddressEnable( unsigned char byAddr_Enb) {
//     pdiData[0] = (byAddress & 0x7F) | (byEnable << 7);
     pdiData[0] = byAddr_Enb;
     pdiSendCommand( PDI_CMD_FNC_ENB_ADDR);
     pdiWriteInt( 1);
   }

#else

 /*********************************************************/
 // pdiSetEmbFncAddressEnable
 // Enable Embedded function and set address (byAddress is 0-0x7F, byEnable is 0 or 1)
 // byFnc - function index (zero based) 0-3
   void pdiSetEmbFncAddressEnable( unsigned char byFnc, unsigned char byAddress, unsigned char byEnable) {
     pdiData = (byAddress & 0x7F) | (byEnable << 7);
     pdiSendCommand( PDI_CMD_FNC1_ENB_ADDR + byFnc);
     pdiWriteInt( 1);
   }

#endif



 /*********************************************************/
 // pdiSetEndpointEnable
 // enable/disable endpoints (PDI_EPEN_xxx)
   void pdiSetEndpointEnable( unsigned char byEnable) {
     pdiData[0] = byEnable;
     pdiSendCommand( PDI_CMD_EPEN);
     pdiWriteInt( 1);
   }


#if !defined(PDIUSBH11)  // H11 has not it

 /*********************************************************/
 // pdiSetMode
 // set chip mode (PDI_MODE_xxx) and clock division factor (PDI_CLOCK_xxx)
  void pdiSetMode( unsigned int wMode_Clock) {
     pdiData[0] = (unsigned char) (wMode_Clock & 0xff);
     pdiData[1] = (unsigned char) (wMode_Clock >> 8);
     pdiSendCommand( PDI_CMD_SET_MODE);
     pdiWriteInt( 2);
   }

#endif

 #if defined(PDIUSBD12)

 /*********************************************************/
 // pdiSetDMA
 // set DMA mode (PDI_DMA_xxx)
  void pdiSetDMA( unsigned char byDma) {
    pdiData[0] = byDma;
    pdiSendCommand( PDI_CMD_SET_DMA);
    pdiWriteInt( 1);
  }

 /*********************************************************/
 // pdiGetDMA
 // get current DMA mode
  unsigned char pdiGetDMA( void) {
    pdiSendCommand( PDI_CMD_GET_DMA);
    pdiReadInt( 1);
    return pdiData[0];
  }

 #endif


 #if defined(PDIUSBH11)

 /*********************************************************/

 // pdiGetInterrupt - H11 return only one byte
 // get interrupt register (PDI_INT_xxx)
  unsigned char pdiGetInterrupt( void) _PDI_USING {
#if defined(PDI_USE_USING)
    return pdiIntCmdReadData( PDI_CMD_GET_INT_REG, 0);
#else
    pdiSendCommand( PDI_CMD_GET_INT_REG);
    pdiReadInt( 1);
    return pdiData[0];
#endif
  }

 #else

 /*********************************************************/
 // pdiGetInterrupt
 // get interrupt register (PDI_INT_xxx)
  unsigned short pdiGetInterrupt( void) _PDI_USING {
#if defined(PDI_USE_USING)
    return pdiIntCmdReadData( PDI_CMD_GET_INT_REG, 1);
#else
    pdiSendCommand( PDI_CMD_GET_INT_REG);
    pdiReadInt( 2);
    return pdiData[0] + (((unsigned short)pdiData[1])<<8);
#endif
  }

 #endif

 /*********************************************************/
 // pdiSelectEp
 // set internal buffer pointer to selected endpoint (zero based) (PDI_SELEP_xxx)
  unsigned char pdiSelectEp( unsigned char byEpIdx) {
    pdiSendCommand( PDI_CMD_SELECT_EP + byEpIdx);
    pdiReadInt( 1);
    return pdiData[0];
  }

 /*********************************************************/
 // pdiGetLastTransStatus
 // get Last transaction status (PDI_LTSTAT_xxx and PDI_ERR_xxx)
  unsigned char pdiGetLastTransStatus( unsigned char byEpIdx) _PDI_USING {
#if defined(PDI_USE_USING)
    return pdiIntCmdReadData( PDI_CMD_GET_LAST_STAT + byEpIdx, 0);
#else
    pdiSendCommand( PDI_CMD_GET_LAST_STAT + byEpIdx);
    pdiReadInt( 1);
    return pdiData[0];
#endif
  }


 #if defined(PDIUSBD11) || defined(PDIUSBH11) || defined(PDIUSBH11A)

 /*********************************************************/
 // pdiGetEpStatus
 // get Endpoint Status (PDI_EPSTAT_xxx)
  unsigned char pdiGetEpStatus( unsigned char byEpIdx) {
    pdiSendCommand( PDI_CMD_GET_EP_STAT + byEpIdx);
    pdiReadInt( 1);
    return pdiData[0];
  }

 #endif

 /*********************************************************/
 // pdiReadFromEpBuffer - raw reading
 // read data from selected internal chip buffer
 // if byLength < length of buffer data, so we read only byLength data)
  unsigned char pdiReadFromEpBuffer( unsigned char byLength, unsigned char *pToBuff) {
    pdiSendCommand( PDI_CMD_READ_BUFFER);
    pdiReadInt( 2);
    if ( pdiData[1]) { // there is some data
      if ( byLength < pdiData[1]) // we need less data then is received
        pdiData[1] = byLength;
      pdiReadData( pdiData[1], pToBuff);
    }
    return pdiData[1];
  }

 /*********************************************************/
 // pdiWriteToEpBuffer - raw writing
 // write data to selected internal chip buffer
  void pdiWriteToEpBuffer( unsigned char byLength, unsigned char *pFromBuff) {
    pdiSendCommand( PDI_CMD_WRITE_BUFFER);
    pdiData[0] = 0; pdiData[1] = byLength;
    pdiWriteInt( 2);
    if ( byLength) {
      pdiWriteData( byLength, pFromBuff);
    }
  }

 /*********************************************************/
 // pdiSetEpStatus
 // set endpoint stall flag
  void pdiSetEpStatus( unsigned char byEpIdx, unsigned char byStatus) {
    pdiData[0] = byStatus;
    pdiSendCommand( PDI_CMD_SET_EP_STAT + byEpIdx);
    pdiWriteInt( 1);
  }

 /*********************************************************/
 // pdiAcknowledgeSetup
 // chip disable fncs Validate and Clear after SETUP packet,
 // this cmd re-enable these fncs
  void pdiAcknowledgeSetup( void) {
    pdiSendCommand( PDI_CMD_ACK_SETUP);
  }

 /*********************************************************/
 // pdiClearBuffer
 // set endpoint flag 'empty' and next data can be receive
  void pdiClearBuffer( void) {
    pdiSendCommand( PDI_CMD_CLEAR_BUFFER);
  }

 /*********************************************************/
 // pdiValidateBuffer
 // set endpoint flag 'full' and data can be send
  void pdiValidateBuffer( void) {
    pdiSendCommand( PDI_CMD_VALID_BUFFER);
  }

 /*********************************************************/
 // pdiSendResume
 // send an upstream resume signal for 10ms
  void pdiSendResume( void) {
    pdiSendCommand( PDI_CMD_SEND_RESUME);
  }

 /*********************************************************/
 // pdiGetFrameNumber
 // return frame number of last successfully received SOF
  unsigned short pdiGetFrameNumber( void) {
    pdiSendCommand( PDI_CMD_GET_FRAME);
    pdiReadInt( 2);
    return pdiData[0] + (((unsigned short)pdiData[1])<<8);
  }
  
 /*********************************************************/
 // pdiGetChipID  - this function is undocumented
 // read chip ID (not documented function) ( LSB is maybe type of chip in hex (0x12,0x11))
  unsigned short pdiGetChipID( void) {
    pdiSendCommand( PDI_CMD_GET_CHIP_ID);
    pdiReadInt( 2);
    return pdiData[0] + (((unsigned short)pdiData[1])<<8);
  }    


 /*********************************************************/
 /*********************************************************/
 // HUB command
#if defined(PDIUSBH11) || defined(PDIUSBH11A)
 /*********************************************************/
 // pdiClearPortFeature
 // clear feature 'byFeature' in downstream port 2-5 'byEpIdx' (zero based)
  void pdiClearPortFeature( unsigned char byEpIdx, unsigned char byFeature) {
    pdiData[0] = byFeature;
    pdiSendCommand( PDI_CMD_P2_CLR_FEATURE + byEpIdx);
    pdiWriteInt( 1);
  }

 /*********************************************************/
 // pdiSetPortFeature
 // set feature 'byFeature' in downstream port 2-5 'byEpIdx' (zero based)
  void pdiSetPortFeature( unsigned char byEpIdx, unsigned char byFeature) {
    pdiData[0] = byFeature;
    pdiSendCommand( PDI_CMD_P2_SET_FEATURE + byEpIdx);
    pdiWriteInt( 1);
  }
  
 /*********************************************************/
 // pdiGetPortFeature
 // get port status (port status byte and port status change byte)
  unsigned short pdiGetPortFeature( unsigned char byEpIdx) {
    pdiSendCommand( PDI_CMD_P2_GET_STATUS + byEpIdx);
    pdiReadInt( 2);
    return pdiData[0] + (((unsigned short)pdiData[1])<<8);
  }
  
 /*********************************************************/
 // pdiSetStatusChangeBits
 // set local power change bits status
  void pdiSetStatusChangeBits( unsigned char byBits) {
    pdiData[0] = byBits;
    pdiSendCommand( PDI_CMD_SET_CHNG_BITS);
    pdiWriteInt( 1);
  }

#endif



/*********************************************************/
/*********************************************************/
// PDIUSB other commands

 // complex function for select endpoint, write data and validate data in endpoint buffer
 void pdiWriteEndpoint( unsigned char byEpIdx, unsigned char byLength, unsigned char *pbyData) {
   pdiSelectEp( byEpIdx);
   pdiWriteToEpBuffer( byLength, pbyData);
   pdiValidateBuffer();
 }

 // complex function for select endpoint, read data and clear endpoint buffer
 // byMaxLength means how many bytes will be maximally read.
 unsigned char pdiReadEndpoint( unsigned char byEpIdx, unsigned char byMaxLength, unsigned char *pbyData) {
   pdiData[1] = 0;
   pdiData[0] = pdiSelectEp( byEpIdx);
   if ( pdiData[0] & PDI_SELEP_FULL) {
    pdiData[1] = pdiReadFromEpBuffer( byMaxLength, pbyData);
    pdiClearBuffer();
   }
   return pdiData[1];
 }

 // complex universal function for select command and read/write data to/from PDIUSB
 unsigned char pdiCmdData( unsigned char byCmd, unsigned char *pbyData,
                           unsigned char byCount, unsigned char byRead) {
   pdiSendCommand( byCmd);
   if ( byCount) {
     if( byRead) byCount = pdiReadData( byCount, pbyData);  // Read Data
     else pdiWriteData( byCount, pbyData);                  // Write Data
   }
   return byCount;   
 }

#if !defined(PDIUSBH11A_MULTIPLE)
 // complex function for acknowledge control endpoint
 void pdiAckSetupControl( void) {
   pdiSendCommand( PDI_CMD_SELECT_EP + PDI_EP0_RX);
   pdiSendCommand( PDI_CMD_ACK_SETUP);
   pdiSendCommand( PDI_CMD_CLEAR_BUFFER);
   pdiSendCommand( PDI_CMD_SELECT_EP + PDI_EP0_TX);
   pdiSendCommand( PDI_CMD_ACK_SETUP);
 }
#else
 // complex function for acknowledge control endpoint ( for one emb.fnc. 1,6,7) (zero-based 0,1,2)
 void pdiAckSetupFncControl( unsigned char Fnc) {
   unsigned char FncTab[3] = { PDI_F1_EP0_RX, PDI_F6_EP0_RX, PDI_F7_EP0_RX};
   pdiSendCommand( PDI_CMD_SELECT_EP + FncTab[Fnc]);
   pdiSendCommand( PDI_CMD_ACK_SETUP);
   pdiSendCommand( PDI_CMD_CLEAR_BUFFER);
   pdiSendCommand( PDI_CMD_SELECT_EP + FncTab[Fnc] + 1);
   pdiSendCommand( PDI_CMD_ACK_SETUP);
 }
#endif

#if defined(PDIUSBD12) // parallel interface
 void pdiInit( void) {
   pdiSetAddressEnable( PDI_ENAD_ENABLE);
   pdiSetEndpointEnable( PDI_EPEN_ENABLE);
   pdiSetDMA( PDI_DMA_EP4_INT | PDI_DMA_EP5_INT);
   pdiSetMode( PDI_MODE_NO_LAZY_CLOCK | PDI_MODE_CLOCK_RUNNING |
               PDI_MODE_SOFT_CONNECT | PDI_CLOCK_12M);
 }
#endif

#if defined(PDIUSBH11A_SINGLE) // serial interface
 void pdiInit( void) {
   pdiSetHUBAddressEnable( 0, 0); // disable HUB
   pdiSetAddressEnable( PDI_ENAD_ENABLE);    // enable emb.function
   pdiSetEndpointEnable( PDI_EPEN_FNC_ENB);
   pdiSetMode( PDI_MODE_REMOTE_WAKEUP | PDI_MODE_NO_LAZY_CLOCK | PDI_MODE_CLOCK_RUNNING |
               PDI_MODE_SOFT_CONNECT | PDI_MODE_SINGLE_FNC | PDI_CLOCK_12M);
 }
#endif

#if defined(PDIUSBD11) // serial interface
 void pdiInit( void) {
   pdiSetAddressEnable( 0 );    // enable emb.function
   pdiSetEndpointEnable(PDI_EPEN_ENABLE);
   pdiSetMode( PDI_MODE_REMOTE_WAKEUP | PDI_MODE_NO_LAZY_CLOCK | PDI_MODE_CLOCK_RUNNING |
               PDI_MODE_SOFT_CONNECT | PDI_MODE_MUSTBEONE| PDI_CLOCK_12M);
 }
#endif

 /*********************************************************/
 /*********************************************************/
