/*******************************************************/
/***   Module : Serial communication functions       ***/
/***   Author : Roman Bartosinski (C) 28.04.2002     ***/
/***   Modify : 27.08.2002                           ***/
/*******************************************************/

#include "hal.h"
#include "serial.h"
#include "stdio_d.h"

/*include "usb.h"*/

#ifdef COMMANDS_PLUGIN
  #include "commands.h"
#endif

  serial_flags_t sflags;

  bit ssBusy;

  xdata SerialCnt_t srCnt, ssCnt, srPos, ssPos;
  xdata volatile unsigned char srBuf[SER_RBLEN], ssBuf[SER_SBLEN];
  xdata volatile unsigned long baudrate;
  
  void InitSerial( void) {
    scli;
    srCnt = 0; 
    ssCnt = 0; 
    srPos = 0; 
    ssPos = 0;  /* init buffers */

    SCON = 0x50;
    TMOD &= 0x0f;                   /* use timer 1 */
    TMOD |= 0x20;
    TR1 = 1;    
    sflags.Output=sflags.Input=sflags.Busy=sflags.Terminal=0;
    ssBusy=0;
    srCnt=ssCnt=srPos=ssPos=0;
    
    SerialSetBaudRate( 19200);

    REGISTER_INT( SER_INT_ADDR, #_SerialInterrupt);

    ssti;
  }


  unsigned char SerialSetBaudRate( unsigned short NewBaud) {   
    register char sint;
    unsigned divisor;
    while ( /*sflags.Busy*/ ssBusy) ;
    sint = EA; EA=0;
    PCON |= 0x80;        /* SMOD = 1; */
    /*divisor = (CPU_SYS_KHZ*1000/NewBaud/12/16);*/
    divisor = CPU_SYS_KHZ/(NewBaud/200);
    divisor = divisor*5/(12*8);
    divisor = (divisor+1)/2;
    TL1 = TH1 = -divisor;
    baudrate = (unsigned long) NewBaud;
    if ( sint) EA=1;
    return 1;
  }


// interrupt function : normally - void SerailInterrupt(void) interrupt 4 using 1;
// void SerialInterrupt( void) _naked interrupt using 3{
// void SerialInterrupt( void) interrupt using 3{
  void SerialInterrupt( void)  interrupt {
    register unsigned char chr;

    /*INTERRUPT_PRE(3);*/

    LEDr = 1;
    if (RI) {
      chr = SBUF;
//      if ( fDebug) LEDBar = chr;
      RI = 0;
      if ( sflags.Terminal) {      // vystup prijateho znaku je do bufferu
        if ( srCnt < SER_RBLEN) {                // znak se do bufferu vejde
          SerialCnt_t rNow = srPos+srCnt++;
          if ( rNow >= SER_RBLEN) rNow -= SER_RBLEN;
          srBuf[rNow] = chr;
        }
        if ( chr == 0x1b)   
          sflags.Terminal = 0;
      } else {        
#ifdef COMMANDS_PLUGIN
          if ( chr == '\r' && cLen) {
            pflags.Command = 1;
          } else {
            if ( pflags.Echo && sflags.Output) {
                ser_putc( chr);
            }
            if (( cLen < CBUFLEN) && ( chr > 0x1f)) { // odfiltrovani ridicich znaku
              cLine[cLen] = chr;
              cLen++;
              cLine[cLen] = 0; // aby to bylo ukonceno nulou :)
            }
            if (( cLen) && ( chr == '\b')) {
              cLen--;
              cLine[cLen] = 0;
            }
          }

#else
        if ( pflags.Echos && sflags.Output) {
          ser_putc( chr);
        }
        if ( srCnt < SER_RBLEN) {                // znak se do bufferu vejde
          SerialCnt_t rNow = srPos+srCnt++;
          if ( rNow >= SER_RBLEN) rNow -= SER_RBLEN;
          srBuf[rNow] = chr;
        }
#endif
      }
    }
    if (TI) {
      TI = 0;
      if (( /*sflags.Busy*/ ssBusy = ( ssCnt > 0))) {
        ssCnt--;
        SBUF = ssBuf[ssPos++];
        if ( ssPos >= SER_SBLEN)
          ssPos = 0;
      }
    }
    LEDr=0;

    /*INTERRUPT_POST();*/
  }

  void ser_putc( unsigned char c) reentrant {
    while ( ssCnt >= SER_SBLEN)
      ;
    scli;
    if ( /*sflags.Busy*/ ssBusy) {
      SerialCnt_t sNow = ssPos + ssCnt++;
      if ( sNow >= SER_SBLEN) sNow -= SER_SBLEN;
      ssBuf[sNow] = c;
    } else {
      TI=0;
      SBUF = c;
      /*sflags.Busy = 1;*/
      ssBusy = 1;
    }
    ssti;
  }

  unsigned char ser_getc(void) {
    unsigned char c;
    if ( sflags.Input) {
      while ( !srCnt) ;
      scli;
      srCnt--;
      c = srBuf[srPos++];
      if ( srPos >= SER_RBLEN)
        srPos = 0;
      ssti;
      return (c);
    }
    return 0;
  }
