/*****************************************************/
/***   Module : USB module                         ***/
/***   Author : Roman Bartosinski (C) 28.04.2002   ***/
/***   Modify : 08.08.2002, 16.04.2003             ***/
/*****************************************************/

#include "hal.h"

#include "usbdefs.h"
#include "pdiusb.h"
#include "usb.h"
#include "vendor.h"

#include "stdio_d.h"

  usb_flags_t uflags;

  static xdata volatile unsigned char LastTrans;
  xdata volatile unsigned int LastD12Int;
  /* ep0 variables */
  xdata usb_ep0_descr_t usb_ep0;
  xdata USB_DEVICE_REQUEST dreq;
  xdata unsigned char ep0_buffer[ MAX_CONTROL_XFER_DATA_SIZE];


/*************************************************************
 *** Interrupt handler
 *************************************************************/
  void UsbInterrupt( void) interrupt _PDI_USING
  {
    LEDr = 1;
    LastD12Int = pdiGetInterrupt();
    if ( LastD12Int & PDI_INT_BUSRESET) {                 /* D12 - Bus reset reached */
      uflags.BusReset = 1;
    } else {
      #ifdef PDI_INT_SUSPEND
      if ( LastD12Int & PDI_INT_SUSPEND)                  /* D12 - Suspend flag changed */
        uflags.Suspend = 1;
      #endif
      if ( LastD12Int & PDI_INT_EP0_OUT) {                /* D12 - Ep0RxDone - some data was received in EP0 */
        uflags.Ep0RxInt = 1;
        LastTrans = pdiGetLastTransStatus( PDI_EP0_RX);
      }
      if ( LastD12Int & PDI_INT_EP0_IN) {                 /* D12 - Ep0TxDone - data in EP0 was sended */
        uflags.Ep0TxInt = 1;
        LastTrans = pdiGetLastTransStatus( PDI_EP0_TX);
      }
      if ( LastD12Int & PDI_INT_EP1_OUT) {                /* D12 - Ep1RxDone - some data was received in EP1 */
        LastTrans = pdiGetLastTransStatus( PDI_EP1_RX);
        uflags.Ep1RxInt = 1;
      }
      if ( LastD12Int & PDI_INT_EP1_IN) {                 /* D12 - Ep1TxDone - data in EP1 was sended */
        LastTrans = pdiGetLastTransStatus( PDI_EP1_TX);
        uflags.Ep1TxInt = 1;
      }

      if ( LastD12Int & PDI_INT_EP2_OUT) {                /* D12 - Ep2RxDone - some data was received in EP2 */
        LastTrans = pdiGetLastTransStatus( PDI_EP2_RX);
        uflags.Ep2RxInt = 1;
      }
      if ( LastD12Int & PDI_INT_EP2_IN) {                 /* D12 - Ep2TxDone - data in EP2 was sended */
        LastTrans = pdiGetLastTransStatus( PDI_EP2_TX);
        uflags.Ep2TxInt = 1;
      }
    }
    LEDr = 0;
  }

/* usb initialize */
  void ClearFlags( void) {
    uflags.ControlPhase = uflags.Configured = uflags.RemoteWakeUp = uflags.SetupPacket = 0;
    uflags.BusReset = uflags.Suspend = uflags.Ep0RxInt = uflags.Ep0TxInt = uflags.Ep1RxInt = uflags.Ep1TxInt = uflags.Ep2RxInt = uflags.Ep2TxInt = 0;
  }

  void InitUSB( void) {

    ClearFlags();
    bD12Suspend = 1;
    ucli;
    LastD12Int = 0;
   #if 0
    PX0 = 1;
    REGISTER_INT( EX0_INT_ADDR, #_UsbInterrupt);
   #endif
    usti;
  }

// connencting to USB by SoftConnect
  void USBConnect( void) {
    ucli;
    LastD12Int = 0; ClearFlags();
    usti;
   #if defined(PDIUSBD12)
    pdiSetDMA( PDI_DMA_EP4_INT | PDI_DMA_EP5_INT); // ???
    pdiSetMode( PDI_MODE_NO_LAZY_CLOCK | PDI_MODE_CLOCK_RUNNING | 
                PDI_MODE_SOFT_CONNECT | PDI_CLOCK_SET_TO_ONE | PDI_CLOCK_4M);
   #endif /* PDIUSBD12 */
   #if defined(PDIUSBD11)
    pdiSetEndpointEnable(PDI_EPEN_ENABLE);
    pdiSetMode( PDI_MODE_REMOTE_WAKEUP | PDI_MODE_NO_LAZY_CLOCK | PDI_MODE_CLOCK_RUNNING |
                PDI_MODE_SOFT_CONNECT | PDI_MODE_MUSTBEONE| PDI_CLOCK_12M);
   #endif /* PDIUSBD11 */
  }

  void USBDisconnect( void) {
   #if defined(PDIUSBD12)
    pdiSetMode( PDI_MODE_NO_LAZY_CLOCK | PDI_MODE_CLOCK_RUNNING | PDI_CLOCK_SET_TO_ONE | PDI_CLOCK_4M);
   #endif /* PDIUSBD12 */
   #if defined(PDIUSBD11)
    pdiSetMode( PDI_MODE_NO_LAZY_CLOCK | PDI_MODE_CLOCK_RUNNING | PDI_MODE_MUSTBEONE| PDI_CLOCK_12M);
   #endif /* PDIUSBD11 */
    ucli;
    uflags.Configured = 0;
    usti;
  }

  void USBStallEp0( void) {
    pdiSetEpStatus( PDI_EP0_TX, PDI_SET_EP_STALLED);
    pdiSetEpStatus( PDI_EP0_RX, PDI_SET_EP_STALLED);
  }

  void USBInitUnconfig( void) {
    pdiSetEndpointEnable( 0);
  }

  void USBInitConfig( void) {
    pdiSetEndpointEnable( PDI_EPEN_ENABLE);
  }
  
// ************************************
// ***   Standard device requests   ***
// ************************************

/* Send any data in the data phase of the control transfer */
  void SendControlData( unsigned char *pRomData, unsigned int len) {
#ifdef DEBUG
    if ( pflags.Debug) {
      printf2( "Send ctrl data F=0x%X(L=%d)\n", pRomData, len);
    }
#endif
    ucli;
    usb_ep0.goes = 0;
    if ( usb_ep0.size > len) usb_ep0.size = len;
    usb_ep0.ptr = pRomData;
    if ( usb_ep0.size > PDI_EP0_PACKET_SIZE) {
      pdiWriteEndpoint( PDI_EP0_TX, PDI_EP0_PACKET_SIZE, usb_ep0.ptr);
      usb_ep0.goes += PDI_EP0_PACKET_SIZE;
      uflags.ControlPhase = USB_TRANSMIT;
    } else {
      pdiWriteEndpoint( PDI_EP0_TX, usb_ep0.size, (xdata unsigned char *)pRomData);
      usb_ep0.goes += usb_ep0.size;
      uflags.ControlPhase = USB_IDLE;
    }
    usti;
  }
  // **************************** 
  void USBGetStatus( void) {
    unsigned char c, endp;

    *((unsigned int *)&(ep0_buffer[0])) = 0;
    switch( dreq.bmRequestType & USB_RECIPIENT) {
      case USB_RECIPIENT_DEVICE:
        if ( uflags.RemoteWakeUp == 1)
          ep0_buffer[0] = USB_GETSTATUS_REMOTE_WAKEUP_ENABLED | USB_GETSTATUS_SELF_POWERED;
        else
          ep0_buffer[0] = USB_GETSTATUS_SELF_POWERED;
        break;
      case USB_RECIPIENT_INTERFACE:
        break;
      case USB_RECIPIENT_ENDPOINT:
        endp = ( unsigned char)( dreq.wIndex & PDI_CNT_EP);
        if ( dreq.wIndex & ( unsigned char) USB_ENDPOINT_DIRECTION_MASK)
          c = pdiSelectEp( endp * 2 + 1); // endpoint in
        else
          c = pdiSelectEp( endp * 2);     // endpoint Out
       #ifdef PDI_SELEP_STALL
        ep0_buffer[0] = (( c & PDI_SELEP_STALL) == PDI_SELEP_STALL);
       #endif /*PDI_SELEP_STALL*/
        break;
      default:
        USBStallEp0();
        return;
    }
    pdiWriteEndpoint( PDI_EP0_TX, 2, ep0_buffer);
  }

  void USBClearFeature( void) {
    unsigned char endp;

    switch( dreq.bmRequestType & USB_RECIPIENT) {
      case USB_RECIPIENT_DEVICE:
        if ( dreq.wValue == USB_FEATURE_REMOTE_WAKEUP) {
          ucli;
          uflags.RemoteWakeUp = 0;
          usti;
          pdiWriteEndpoint( PDI_EP0_TX, 0, 0);
          return;
        }
        break;
      case USB_RECIPIENT_ENDPOINT:
        if ( dreq.wValue == USB_FEATURE_ENDPOINT_STALL) {
          endp = ( unsigned char)( dreq.wIndex & PDI_CNT_EP);
          if ( dreq.wIndex & ( unsigned char) USB_ENDPOINT_DIRECTION_MASK)
            pdiSetEpStatus( endp * 2 + 1, 0); // clear TX stall for IN on EPn
          else
            pdiSetEpStatus( endp * 2, 0);     // clear RX stall for OUT on EPn
          pdiWriteEndpoint( PDI_EP0_TX, 0, 0);
          return;
        }
        break;
    }
    USBStallEp0();
  }

  void USBSetFeature( void) {
    unsigned char endp;

    switch( dreq.bmRequestType & USB_RECIPIENT) {
      case USB_RECIPIENT_DEVICE:
        if ( dreq.wValue == USB_FEATURE_REMOTE_WAKEUP) {
          ucli;
          uflags.RemoteWakeUp = 1;
          usti;
          pdiWriteEndpoint( PDI_EP0_TX, 0, 0);
          return;
        }
        break;
      case USB_RECIPIENT_ENDPOINT:
        if ( dreq.wValue == USB_FEATURE_ENDPOINT_STALL) {
          endp = ( unsigned char)( dreq.wIndex & PDI_CNT_EP);
          if ( dreq.wIndex & ( unsigned char) USB_ENDPOINT_DIRECTION_MASK)
            pdiSetEpStatus( endp * 2 + 1, 1); // set TX stall for IN on EPn
          else
            pdiSetEpStatus( endp * 2, 1);     // set RX stall for OUT on EPn
          pdiWriteEndpoint( PDI_EP0_TX, 0, 0);
          return;
        }
        break;
    }
    USBStallEp0();
  }

  void USBSetAddress( void) {  
    pdiSetAddressEnable( dreq.wValue & DEVICE_ADDRESS_MASK | PDI_ENAD_ENABLE);
    pdiWriteEndpoint( PDI_EP0_TX, 0, 0);
  }



  void USBGetDescriptor( void) {
//    code unsigned char *pDesc = NULL;
    unsigned char *pDesc;
    unsigned int Len;

  #ifdef DEBUG
    if ( pflags.Debug) printf1( " - %s desc.\n", (unsigned int) DescType(MSB( dreq.wValue)));
  #endif
/*
#ifdef DEBUG
    if ( pflags.Debug) printf1( " -bmDesc %X\n", MSB( dreq.wValue));
#endif
*/
    ucli;
    switch ( MSB( dreq.wValue)) {
      case USB_DESCRIPTOR_TYPE_DEVICE:
        pDesc = &DeviceDescription;
        Len = sizeof( USB_DEVICE_DESCRIPTOR);
        break;
      case USB_DESCRIPTOR_TYPE_CONFIGURATION:
        pDesc = &ConfigDescription;
        Len = CONFIG_DESCRIPTOR_LENGTH;
        break;
      case USB_DESCRIPTOR_TYPE_INTERFACE:
        pDesc = &InterfaceDescription;
        Len = sizeof( USB_INTERFACE_DESCRIPTOR);
        break;
      case USB_DESCRIPTOR_TYPE_STRING:
        if ( LSB( dreq.wValue) < CNT_STRINGS) {
          pDesc = (unsigned char *) StringDescriptors[ LSB( dreq.wValue)];
          Len = *pDesc;
        } else {
          USBStallEp0();
          return;
        }
        break;
      default:
        USBStallEp0();
        return;
    }
    if ( dreq.wLength < Len) Len = dreq.wLength;
    usti;
    SendControlData( pDesc, Len); // mozna problemy s char/int
 }

  void USBGetConfiguration( void) {
    ep0_buffer[0] = uflags.Configured;
    pdiWriteEndpoint( PDI_EP0_TX, 1, ep0_buffer);
  }

  void USBSetConfiguration( void) {
    unsigned char iCfg = LSB( dreq.wValue);
    if ( iCfg < 2) {       // put device in unconfigured state or set configuration 1 ( no else)
      pdiWriteEndpoint( PDI_EP0_TX, 0, 0);
      USBInitUnconfig();
      if ( iCfg)
        USBInitConfig();
      ucli;
      uflags.Configured = iCfg;
      usti;
    } else
      USBStallEp0();
  }

  void USBGetInterface( void) {
    ep0_buffer[0] = 0;
    pdiWriteEndpoint( PDI_EP0_TX, 1, &ep0_buffer[0]);
  }

  void USBSetInterface( void) {
    if (( dreq.wValue == 0) && ( dreq.wIndex == 0))
      pdiWriteEndpoint( PDI_EP0_TX, 0, 0);
    else
      USBStallEp0();
  }



/*************************************************************
 *** Interrupt responses
 *************************************************************/
  char execute_usb_request( void) {
    char was_req = 0;
/* response to interrupt BusReset */
    if ( uflags.BusReset) {
      ucli; uflags.BusReset = 0; uflags.Configured = 0; usti;
#ifdef DEBUG
      if ( pflags.Debug) printf( "USB bus reset\n");
#endif
      was_req = 1;
    }
/* response to interrupt Suspend */
    if ( uflags.Suspend) {
      ucli; uflags.Suspend = 0; usti;
#ifdef DEBUG
      if ( pflags.Debug) printf1( "USB was %s.\n", (char *)((bD12Suspend)?"STARTED":"STOPPED"));
#endif
      was_req = 1;
    }
/* response to interrupt SetupPacket execute response to standard device request or vendor request */
    if ( uflags.SetupPacket) {
      unsigned char type, req;
      uflags.SetupPacket = 0;
      
      type = dreq.bmRequestType & USB_REQUEST_TYPE_MASK;
      req = dreq.bRequest & USB_REQUEST_MASK;
    #ifdef DEBUG
      if ( pflags.Debug) {
        printf4( "Receive (0x%X) %s req. for %s , data %s host\n", dreq.bmRequestType, 
                 (unsigned int)ReqType( dreq.bmRequestType), (unsigned int)ReqRecipient(dreq.bmRequestType),
                 (unsigned int)((dreq.bmRequestType & USB_DATA_DIR_MASK) ? "TO":"FROM"));
        printf2( " Request (0x%X) %s\n", dreq.bRequest, 
                 (unsigned int) ((!(dreq.bmRequestType&USB_REQUEST_TYPE_MASK))? ReqName( dreq.bRequest):"UNKNOWN"));
        printf3( "  args: wVal=0x%X,wIdx=0x%X,wLen=0x%X\n",dreq.wValue,dreq.wIndex,dreq.wLength);
      }
    #endif

      if ( type == USB_STANDARD_REQUEST) {
        switch( req) {
          case USB_REQUEST_GET_STATUS:        USBGetStatus(); break;
          case USB_REQUEST_CLEAR_FEATURE:     USBClearFeature(); break;
          case USB_REQUEST_SET_FEATURE:       USBSetFeature(); break;
          case USB_REQUEST_SET_ADDRESS:       USBSetAddress(); break;
          case USB_REQUEST_GET_DESCRIPTOR:    USBGetDescriptor(); break;
//          case USB_REQUEST_SET_DESCRIPTOR:    break;
          case USB_REQUEST_GET_CONFIGURATION: USBGetConfiguration(); break;
          case USB_REQUEST_SET_CONFIGURATION: USBSetConfiguration(); break;
          case USB_REQUEST_GET_INTERFACE:     USBGetInterface(); break;
          case USB_REQUEST_SET_INTERFACE:     USBSetInterface(); break;
//          case USB_REQUEST_SYNC_FRAME:        break;
          default:                            USBStallEp0(); break;
        }
      } else
        if ( type == USB_VENDOR_REQUEST) {
          if ( USBVendorRequest(NULL) == -1)
            USBStallEp0();
        } else if ( type == USB_CLASS_REQUEST) {
          if ( USBClassRequest(NULL) == -1)
            USBStallEp0();
        } else
          USBStallEp0();
      was_req = 1;
    }

/* response to interrupt Ep0RxInt */
    if ( uflags.Ep0RxInt) {
      ucli; uflags.Ep0RxInt = 0; usti;
#ifdef DEBUG
      if ( pflags.Debug) {
        printf2( "Ep0Rx:LI=0x%X,LTS=0x%X\n",LastD12Int,LastTrans); 
      }    
#endif
      if ( LastTrans & PDI_LTSTAT_SETUP) {
        usb_ep0.size = 0;
        usb_ep0.goes = 0;
        if ( pdiReadEndpoint( PDI_EP0_RX, sizeof( USB_DEVICE_REQUEST), (unsigned char *) dreq) != sizeof( USB_DEVICE_REQUEST)) {
          USBStallEp0();
          uflags.ControlPhase = USB_IDLE;
          return 1;
        }

        dreq.wValue  = SWAP( dreq.wValue);
        dreq.wIndex  = SWAP( dreq.wIndex);
        dreq.wLength = SWAP( dreq.wLength);

        // acknowledge setup here
        pdiAckSetupControl();
        usb_ep0.size = dreq.wLength;
        usb_ep0.goes = 0;
#ifdef DEBUG
        if ( pflags.Debug) {
          printf2( "DevReqType=0x%X ,DevReqLen=0x%X\n", dreq.bmRequestType, dreq.wLength);
        }
#endif
        if (( dreq.bmRequestType & ( unsigned char) USB_ENDPOINT_DIRECTION_MASK) || ( dreq.wLength == 0)) {
          uflags.SetupPacket  = 1;
          uflags.ControlPhase = USB_IDLE; // get command
        } else {
          if( dreq.wLength > MAX_CONTROL_XFER_DATA_SIZE) {
            uflags.ControlPhase = USB_IDLE;
            USBStallEp0();
          } else {
            uflags.ControlPhase = USB_RECEIVE;	// set command with OUT token
          }
        } // else set command
      } else { // if setup packet    
        register unsigned char i;
        if ( uflags.ControlPhase == USB_RECEIVE) {
          i = pdiReadEndpoint( PDI_EP0_RX, PDI_EP0_PACKET_SIZE, ep0_buffer + usb_ep0.goes); // ??? really ep0_buffer ???
          usb_ep0.goes += i;
          if (( i != PDI_EP0_PACKET_SIZE) || (usb_ep0.goes >= usb_ep0.size)) {
            uflags.SetupPacket = 1;
            uflags.ControlPhase = USB_IDLE;
          }
        } else {
          uflags.ControlPhase = USB_IDLE;
        }
      }
      was_req = 1;
    }
/* response to interrupt Ep0TxInt */
    if ( uflags.Ep0TxInt) {
      short i = usb_ep0.size - usb_ep0.goes;
      ucli; uflags.Ep0TxInt = 0; usti;
#ifdef DEBUG
      if ( pflags.Debug) printf1( "EP0Tx:LTS=0x%X\n", LastTrans);
#endif
      if ( uflags.ControlPhase == USB_TRANSMIT) { 
        if( i >= PDI_EP0_PACKET_SIZE) { // jeste se musi poslat vic nez je max ep0 size
          pdiWriteEndpoint( PDI_EP0_TX, PDI_EP0_PACKET_SIZE, usb_ep0.ptr + usb_ep0.goes);
          usb_ep0.goes += PDI_EP0_PACKET_SIZE;
          uflags.ControlPhase = USB_TRANSMIT;
        } else 
          if( i != 0) { // je potreba poslat jeste neco ( min nez ep0 size )
            pdiWriteEndpoint( PDI_EP0_TX, i, usb_ep0.ptr + usb_ep0.goes);
            usb_ep0.goes += i;
            uflags.ControlPhase = USB_IDLE;
          } else 
            if ( i == 0) { // vsechno bylo poslano
              pdiWriteEndpoint( PDI_EP0_TX, 0, 0); // Send zero packet at the end ???
              uflags.ControlPhase = USB_IDLE;
            }
      }
      was_req = 1;
    }

    return was_req;
  }



/*************************************************************
 *** Debug infos
 *************************************************************/
#ifdef DEBUG
  char *ReqRecipient( char rqt) {
    switch ( rqt & USB_RECIPIENT) {
      case USB_RECIPIENT_DEVICE:    return "DEVICE";
      case USB_RECIPIENT_INTERFACE: return "INTERFACE";
      case USB_RECIPIENT_ENDPOINT:  return "ENDPOINT";
    }
    return "OTHER";
  }
  char *ReqType( char rqt) {
    switch ( rqt & USB_REQUEST_TYPE_MASK) {
      case USB_STANDARD_REQUEST: return "STANDARD";
      case USB_CLASS_REQUEST:    return "CLASS";
      case USB_VENDOR_REQUEST:   return "VENDOR";
    }
    return "RESERVED";
  }
  char *ReqName( char req) {
    switch ( req & USB_REQUEST_MASK) {
      case USB_REQUEST_GET_STATUS:        return "GET STATUS";
      case USB_REQUEST_CLEAR_FEATURE:     return "CLEAR FEATURE";
      case USB_REQUEST_SET_FEATURE:       return "SET FEATURE";
      case USB_REQUEST_SET_ADDRESS:       return "SET ADDRESS";

      case USB_REQUEST_GET_DESCRIPTOR:    return "GET DESCRIPTOR";
      case USB_REQUEST_SET_DESCRIPTOR:    return "SET DESCRIPTOR";
      case USB_REQUEST_GET_CONFIGURATION: return "GET CONFIGURATION";
      case USB_REQUEST_SET_CONFIGURATION: return "SET CONFIGURATION";
      case USB_REQUEST_GET_INTERFACE:     return "GET INTERFACE";
      case USB_REQUEST_SET_INTERFACE:     return "SET INTERFACE";
      case USB_REQUEST_SYNC_FRAME:        return "SYNC FRAME";
    }
    return "UNKNOWN";
  }
  char *DescType( unsigned char desc) {
    switch ( desc) {
      case USB_DESCRIPTOR_TYPE_DEVICE:                    return"DEVICE";
      case USB_DESCRIPTOR_TYPE_CONFIGURATION:             return"CONFIGURATION";
      case USB_DESCRIPTOR_TYPE_STRING:                    return"STRING";
      case USB_DESCRIPTOR_TYPE_INTERFACE:                 return"INTERFACE";
      case USB_DESCRIPTOR_TYPE_ENDPOINT:                  return"ENDPOINT";
      case USB_DESCRIPTOR_TYPE_DEVICE_QUALIFIER:          return"DEVICE_QUALIFIER";
      case USB_DESCRIPTOR_TYPE_OTHER_SPEED_CONFIGURATION: return"OTHER_SPEED_CONFIG";
      case USB_DESCRIPTOR_TYPE_POWER:                     return"POWER";
    }
    return "UNKNOWN";
  }
#endif

