/**************************************************************/
/***   Module : USB module - header file                    ***/
/***   Author : Roman Bartosinski (C) 28.04.2002            ***/
/***   Modify : 08.08.2002                                  ***/
/***   Rewrite: 05.09.2002                                  ***/
/**************************************************************/

#ifndef _USB_MODULE_
  #define _USB_MODULE_

  #include "usb_spec.h"
  #include "pdiusb.h"

/* USB flags */

  typedef struct usb_flags {
    unsigned char BusReset:1;
    unsigned char Suspend :1;
    unsigned char Ep0RxInt:1;
    unsigned char Ep0TxInt:1;
    unsigned char Ep1RxInt:1;
    unsigned char Ep1TxInt:1;
    unsigned char Ep2RxInt:1;
    unsigned char Ep2TxInt:1;
    
    unsigned char ControlPhase:2;
    unsigned char Configured:1;
    unsigned char RemoteWakeUp:1;
    unsigned char SetupPacket:1;
  } usb_flags_t;

  extern usb_flags_t uflags;

// ***   Control transfer state   ***
  #define USB_IDLE     0
  #define USB_TRANSMIT 1
  #define USB_RECEIVE  2

//  #define UsbControlState(a) ((char)(((char) uControlState2 << 1) | (char) uControlState1) == a)
//  #define SetUsbControlState(a) { uControlState2 = ((char)(a&0x02)>>1); uControlState1 = (a & 0x01);}

/* control endpoint */
  #define MAX_CONTROL_XFER_DATA_SIZE 8
  typedef struct usb_ep0_descr {
    unsigned char *ptr;
    unsigned char size;
    unsigned char goes;
  } usb_ep0_descr_t;


/* functions */
  void USBConnect( void);
  void USBDisconnect( void);
  void USBStallEp0( void);
  void USBInitUnconfig( void);
  void USBInitConfig( void);
  void USBControlHandler( void);

/* Standard requests functions */
  void USBGetStatus( void); 
  void USBClearFeature( void);
  void USBSetFeature( void);
  void USBSetAddress( void);
  void USBGetDescriptor( void);
  void USBGetConfiguration( void);
  void USBSetConfiguration( void);
  void USBGetInterface( void);
  void USBSetInterface( void);


/* usb interrupt handler */
  void UsbInterrupt( void) interrupt _PDI_USING;
/* init usb port */
  void InitUSB( void);
/* response to interrupt requests */
  char execute_usb_request( void);

/* DEBUG infos */
  char *ReqRecipient( char rqt);
  char *ReqType( char rqt);
  char *ReqName( char req);
  char *DescType( unsigned char desc);


#endif
