/*****************************************************/
/***   Module : USB definitions from specification ***/
/***   Author : Roman Bartosinski (C) 28.04.2002   ***/
/***   Modify : 08.08.2002, 16.04.2003             ***/
/*****************************************************/

#ifndef _USB_SPECIFICATIONS_AND_DEFINITIONS_MODULE
 #define _USB_SPECIFICATIONS_AND_DEFINITIONS_MODULE

// this section is from __USB100.H__ and __CHAP9.H__
// and define USB constants

// *** USB Device Request *** (spec. 9.3)
  typedef struct _tag_device_request {
    unsigned char bmRequestType;
    unsigned char bRequest;
    unsigned short wValue;
    unsigned short wIndex;
    unsigned short wLength;
  } USB_DEVICE_REQUEST;

// ************************************************************************************** 
// *** definitions for USB tranfer standard packets described in USB specif. - chapter 9
// ************************************************************************************** 
  #define DEVICE_ADDRESS_MASK   0x7F

// bmRequestType D7 - Data transfer direction
  #define USB_DATA_DIR_MASK        (unsigned char)0x80
  #define USB_DATA_DIR_FROM_HOST   (unsigned char)0x00
  #define USB_DATA_DIR_TO_HOST     (unsigned char)0x80
// bmRequestType D4..D0 - Recipient
  #define USB_RECIPIENT            (unsigned char)0x1F
  #define USB_RECIPIENT_DEVICE     (unsigned char)0x00
  #define USB_RECIPIENT_INTERFACE  (unsigned char)0x01
  #define USB_RECIPIENT_ENDPOINT   (unsigned char)0x02
  #define USB_RECIPIENT_OTHER      (unsigned char)0x03
// bmRequestType D6..D5 - Type
  #define USB_REQUEST_TYPE_MASK    (unsigned char)0x60
  #define USB_STANDARD_REQUEST     (unsigned char)0x00
  #define USB_CLASS_REQUEST        (unsigned char)0x20
  #define USB_VENDOR_REQUEST       (unsigned char)0x40
// Standard request codes (spec. 9.4)
  #define USB_REQUEST_MASK         (unsigned char)0x0F
  #define USB_REQUEST_GET_STATUS                  0x00
  #define USB_REQUEST_CLEAR_FEATURE               0x01
  #define USB_REQUEST_SET_FEATURE                 0x03
  #define USB_REQUEST_SET_ADDRESS                 0x05
  #define USB_REQUEST_GET_DESCRIPTOR              0x06
  #define USB_REQUEST_SET_DESCRIPTOR              0x07
  #define USB_REQUEST_GET_CONFIGURATION           0x08
  #define USB_REQUEST_SET_CONFIGURATION           0x09
  #define USB_REQUEST_GET_INTERFACE               0x0A
  #define USB_REQUEST_SET_INTERFACE               0x0B
  #define USB_REQUEST_SYNC_FRAME                  0x0C
// Descriptor types (spec. 9.4)
  #define USB_DESCRIPTOR_TYPE_DEVICE                    0x01
  #define USB_DESCRIPTOR_TYPE_CONFIGURATION             0x02
  #define USB_DESCRIPTOR_TYPE_STRING                    0x03
  #define USB_DESCRIPTOR_TYPE_INTERFACE                 0x04
  #define USB_DESCRIPTOR_TYPE_ENDPOINT                  0x05
  #define USB_DESCRIPTOR_TYPE_DEVICE_QUALIFIER          0x06
  #define USB_DESCRIPTOR_TYPE_OTHER_SPEED_CONFIGURATION 0x07
  #define USB_DESCRIPTOR_TYPE_POWER                     0x08

// values for the bits returned by the USB GET_STATUS command (spec. 9.4.5)
  #define USB_GETSTATUS_SELF_POWERED                0x01
  #define USB_GETSTATUS_REMOTE_WAKEUP_ENABLED       0x02

// values for standard request Clear Feature
  #define USB_FEATURE_ENDPOINT_STALL          0x0000
  #define USB_FEATURE_REMOTE_WAKEUP           0x0001


// ******************************************* 
// *** Standard USB Descriptor Definitions *** (spec. 9.6)
// *******************************************

// *** DEVICE *** (spec. 9.6.1)
  typedef struct _tag_usb_device_descriptor {
    unsigned char bLength;
    unsigned char bDescriptorType;
    unsigned int  bcdUSB;
    unsigned char bDeviceClass;
    unsigned char bDeviceSubClass;
    unsigned char bDeviceProtocol;
    unsigned char bMaxPacketSize0;
    unsigned int  idVendor;
    unsigned int  idProduct;
    unsigned int  bcdDevice;
    unsigned char iManufacturer;
    unsigned char iProduct;
    unsigned char iSerialNumber;
    unsigned char bNumConfigurations;
  } USB_DEVICE_DESCRIPTOR, *PUSB_DEVICE_DESCRIPTOR;

// *** DEVICE_QUALIFIER *** (spec. 9.6.2)
  typedef struct _tag_usb_device_qualifier_descriptor {
    unsigned char bLength;
    unsigned char bDescriptorType;
    unsigned int  bcdUSB;
    unsigned char bDeviceClass;
    unsigned char bDeviceSubClass;
    unsigned char bDeviceProtocol;
    unsigned char bMaxPacketSize0;
    unsigned char bNumConfigurations;
    unsigned char bReserved;
  } USB_DEVICE_QUALIFIER_DESCRIPTOR, *PUSB_DEVICE_QUALIFIER_DESCRIPTOR;

// *** CONFIGURATION *** (spec. 9.6.3)
  typedef struct _tag_usb_configuration_descriptor {
    unsigned char bLength;
    unsigned char bDescriptorType;
    unsigned int  wTotalLength;
    unsigned char bNumInterfaces;
    unsigned char bConfigurationValue;
    unsigned char iConfiguration;
    unsigned char bmAttributes;
    unsigned char MaxPower;
  } USB_CONFIGURATION_DESCRIPTOR, *PUSB_CONFIGURATION_DESCRIPTOR;
// definitions for bits in the bmAttributes field of a configuration descriptor. (spec. 9.6.3)
  #define USB_CONFIG_POWERED_MASK  0xc0
  #define USB_CONFIG_BUS_POWERED   0x80
  #define USB_CONFIG_SELF_POWERED  0x40
  #define USB_CONFIG_REMOTE_WAKEUP 0x20
  #define BUS_POWERED              0x80
  #define SELF_POWERED             0x40
  #define REMOTE_WAKEUP            0x20

// *** OTHER_SPEED_CONFIGURATION *** (spec. 9.6.4)
  typedef struct _tag_usb_other_speed_configuration {
    unsigned char bLength;
    unsigned char bDescriptorType;
    unsigned int  wTotalLength;
    unsigned char bNumInterfaces;
    unsigned char bConfigurationValue;
    unsigned char iConfiguration;
    unsigned char bmAttributes;
    unsigned char bMaxPower;
  } USB_OTHER_SPEED_CONFIGURATION_DESCRIPTOR, *PUSB_OTHER_SPEED_CONFIGURATION_DESCRIPTOR;

// *** INTERFACE *** (spec. 9.6.5)
  typedef struct _tag_usb_interface_descriptor {
    unsigned char bLength;
    unsigned char bDescriptorType;
    unsigned char bInterfaceNumber;
    unsigned char bAlternateSetting;
    unsigned char bNumEndpoints;
    unsigned char bInterfaceClass;
    unsigned char bInterfaceSubClass;
    unsigned char bInterfaceProtocol;
    unsigned char iInterface;
  } USB_INTERFACE_DESCRIPTOR, *PUSB_INTERFACE_DESCRIPTOR;

// *** ENDPOINT *** (spec. 9.6.6)
  typedef struct _tag_usb_endpoint_descriptor {
    unsigned char bLength;
    unsigned char bDescriptorType;
    unsigned char bEndpointAddress;
    unsigned char bmAttributes;
    unsigned int  wMaxPacketSize;
    unsigned char bInterval;
  } USB_ENDPOINT_DESCRIPTOR, *PUSB_ENDPOINT_DESCRIPTOR;

// Endpoint direction bit, stored in address (spec. 9.6.6)
  #define USB_ENDPOINT_DIRECTION_MASK               0x80
  // test direction bit in the bEndpointAddress field of an endpoint descriptor.
  #define USB_ENDPOINT_DIRECTION_OUT(addr)          (!((addr) & USB_ENDPOINT_DIRECTION_MASK))
  #define USB_ENDPOINT_DIRECTION_IN(addr)           ((addr) & USB_ENDPOINT_DIRECTION_MASK)
// Values for bmAttributes field of an endpoint descriptor (spec. 9.6.6)
  #define USB_ENDPOINT_TYPE_MASK                    0x03
  #define USB_ENDPOINT_TYPE_CONTROL                 0x00
  #define USB_ENDPOINT_TYPE_ISOCHRONOUS             0x01
  #define USB_ENDPOINT_TYPE_BULK                    0x02
  #define USB_ENDPOINT_TYPE_INTERRUPT               0x03

// *** STRING *** (spec. 9.6.7)
  typedef struct _tag_usb_string_descriptor {
    unsigned char bLength;
    unsigned char bDescriptorType;
    unsigned char bString[];
  } USB_STRING_DESCRIPTOR, *PUSB_STRING_DESCRIPTOR;
  

// *** USB_IF
  // defined USB device classes
  #define USB_DEVICE_CLASS_RESERVED             0x00
  #define USB_DEVICE_CLASS_AUDIO                0x01
  #define USB_DEVICE_CLASS_COMMUNICATIONS       0x02
  #define USB_DEVICE_CLASS_HUMAN_INTERFACE      0x03
  #define USB_DEVICE_CLASS_MONITOR              0x04
  #define USB_DEVICE_CLASS_PHYSICAL_INTERFACE   0x05
  #define USB_DEVICE_CLASS_POWER                0x06
  #define USB_DEVICE_CLASS_PRINTER              0x07
  #define USB_DEVICE_CLASS_STORAGE              0x08
  #define USB_DEVICE_CLASS_HUB                  0x09
  #define USB_DEVICE_CLASS_APPLICATION_SPECIFIC 0xFE
  #define USB_DEVICE_CLASS_VENDOR_SPECIFIC      0xFF

  // define application specific device class subclasses
  #define USB_APPL_SUBCLASS_FIRMWARE_UPDATE     0x01
  #define USB_APPL_SUBCLASS_IRDA_USB_BRIDGE     0x02



#endif
