/*****************************************************/
/***   Module : USB product definitions            ***/
/***   Author : Roman Bartosinski (C) 17.07.2002   ***/
/***   Modify : 08.08.2002                         ***/
/*****************************************************/

#ifndef USB_DEFINITIONS_HEADER_FILE
 #define USB_DEFINITIONS_HEADER_FILE

 #include "usb_spec.h"
 #include "pdiusb.h" /* for endpoint packet sizes */

 #define USB_VENDOR_ID   0xDEAD  /* Testing vendor ID - my own vendor, not registered in the USB IF */
 #define USB_PRODUCT_ID  0x0001  /* First product ID - skeleton of USB application */
 #define USB_RELEASE_VER 0x0001  /* Release ID */

/* Class codes for device description */
  #define USB_CLASS_CODE            0xFF
  #define USB_SUBCLASS_CODE         0x00
  #define USB_PROTOCOL_CODE         0x00


/*****************************************************/
/*** Static data structures for device descriptors ***/
/*****************************************************/
  #define NUM_ENDPOINTS	2
  #define CONFIG_DESCRIPTOR_LENGTH sizeof( USB_CONFIGURATION_DESCRIPTOR) \
                                   + sizeof( USB_INTERFACE_DESCRIPTOR) \
                                   + ( NUM_ENDPOINTS * sizeof( USB_ENDPOINT_DESCRIPTOR))
                                   

/*** Device descriptor ***/
  code const USB_DEVICE_DESCRIPTOR DeviceDescription = {
    sizeof( USB_DEVICE_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_DEVICE,
    SWAP( 0x0100),
    USB_CLASS_CODE,
    USB_SUBCLASS_CODE,
    USB_PROTOCOL_CODE,
    PDI_EP0_PACKET_SIZE,
    SWAP( USB_VENDOR_ID),
    SWAP( USB_PRODUCT_ID),  
    SWAP( USB_RELEASE_VER),
    1, /* manufacturer string ID */
    2, /* product string ID */
    3, /* serial number string ID */
    1
  };

/*** Configuration descriptor ***/
  code const USB_CONFIGURATION_DESCRIPTOR ConfigDescription = {
    sizeof( USB_CONFIGURATION_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_CONFIGURATION,
    SWAP( CONFIG_DESCRIPTOR_LENGTH),
    1, /* cnt of interfaces */
    1, /* this configuration ID */
    4, /* config.name string ID*/
    0xa0, /* cfg */
    32    /* device power current from host */
  };

  code const USB_INTERFACE_DESCRIPTOR InterfaceDescription = {      /* is sended with ConfigDescription */
    sizeof( USB_INTERFACE_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_INTERFACE,
    0,    /* index of this interface for SetInterface request */
    0,    /* ID alternate interface */
    NUM_ENDPOINTS,
    USB_CLASS_CODE,
    USB_SUBCLASS_CODE,
    USB_PROTOCOL_CODE,
    5
  };

  code const USB_ENDPOINT_DESCRIPTOR Ep2TxDescription = {           /* is sended with ConfigDescription */
    sizeof( USB_ENDPOINT_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_ENDPOINT,
    0x82,
    USB_ENDPOINT_TYPE_BULK,
    SWAP( PDI_EP2_PACKET_SIZE),
    0
  };
  code const USB_ENDPOINT_DESCRIPTOR Ep2RxDescription = {           /* is sended with ConfigDescription */
    sizeof( USB_ENDPOINT_DESCRIPTOR),
    USB_DESCRIPTOR_TYPE_ENDPOINT,
    0x02,
    USB_ENDPOINT_TYPE_BULK,
    SWAP( PDI_EP2_PACKET_SIZE),
    0
  };


  code const char Str0Desc[] = {                   /* descibe language of strings */
    6, 0x03,   /* 2+2*N , N is count of supported languages */
    0x05,0x04, /* czech language 0x0405 , english 0x0409 */
    0x09,0x04
  };
  code const char Str1Desc[] = {                   // 1 = manufacturer
    58, 0x03,                                               
    'R',0,
    'o',0,
    'm',0,
    'a',0,
    'n',0,
    ' ',0,
    'B',0,
    'a',0,
    'r',0,
    't',0,
    'o',0,
    's',0,
    'i',0,
    'n',0,
    's',0,
    'k',0,
    'i',0,
    ',',0,
    'C',0,
    'T',0,
    'U',0,
    ' ',0,
    'P',0,
    'r',0,
    'a',0,
    'g',0,
    'u',0,
    'e',0
  };
  code const char Str2Desc[] = {                   // 2 = product
    58, 0x03,                                               
    'i',0,
    '5',0,
    '1',0,
    ' ',0,
    'U',0,
    'S',0,
    'B',0,
    ' ',0,
    'a',0,
    'p',0,
    'p',0,
    'l',0,
    'i',0,
    'c',0,
    'a',0,
    't',0,
    'i',0,
    'o',0,
    'n',0,
    ' ',0,
    's',0,
    'k',0,
    'e',0,
    'l',0,
    'e',0,
    't',0,
    'o',0,
    'n',0
  };
  code const char Str3Desc[] = {                   // 3 = version
    24, 0x03,                                               
    '2',0,
    '8',0,
    '.',0,
    '0',0,
    '4',0,
    '.',0,
    '0',0,
    '3',0,
    '|',0,
    '0',0,
    '1',0
  };
  code const char Str4Desc[] = {                   // 4 = configuration
    40, 0x03,                                               
    'F',0,
    'i',0,
    'r',0,
    's',0,
    't',0,
    ' ',0,
    'C',0,
    'o',0,
    'n',0,
    'f',0,
    'i',0,
    'g',0,
    'u',0,
    'r',0,
    'a',0,
    't',0,
    'i',0,
    'o',0,
    'n',0
  };
  code const char Str5Desc[] = {                   // 5 = interface
    28, 0x03,                                               
    'I',0,
    'n',0,
    't',0,
    'e',0,
    'r',0,
    'f',0,
    'a',0,
    'c',0,
    'e',0,
    ' ',0,
    '0',0,
    '/',0,
    '0',0    
  };

#define CNT_STRINGS 0x06

  code const PUSB_STRING_DESCRIPTOR StringDescriptors[] = {         // all strings in pointers array
    Str0Desc,
    Str1Desc,
    Str2Desc,
    Str3Desc,
    Str4Desc,
    Str5Desc
  };

#endif
