
;********************************************************************
;*                                                                  *
;* ulan communication - version for generic msc51 core based mcu    *
;*                                                                  *
;* ulan.asm        - low level assembly implementation of ulan      *
;*                   protocol implementation for mcs51              *
;*                                                                  *
;*                                                                  *
;* copyright holders and originators:                               *
;*     (c) 1991-2003 by pavel pisa pisa@cmp.felk.cvut.cz            *
;*     (c) 1992-2003 by pisoft praha                                *
;*     (c) 1994-2003 by pikron ltd. http://www.pikron.com           *
;*                                                                  *
;*   the more ulan related information can be foud at next sites    *
;*            http://cmp.felk.cvut.cz/~pisa/ulan/                   *
;*            http://www.pikron.com/en/chromul_over.html            *
;*                                                                  *
;* the ulan for small embedded devices can be used, modified        *
;* and redistributed if at least conditions of one of the           *
;* following licenses are met                                       *
;*     - gpl - general .globl license                               *
;*     - lgpl - lesser general .globl license                       *
;*     - mpl - mozilla .globl license                               *
;*     - other license conditions assigned by originators           *
;*                                                                  *
;* if significant code contributor disagrees with some of above     *
;* licenses, he or she can delete such line/s, but if he deletes    *
;* all lines, he is not allowed redistribute code in any binary     *
;* or source form.                                                  *
;********************************************************************


for_sdcc = 1     ; the sdcc asx8051 is used
vector_fl = 1     ; vyuzivaji se vektory preruseni v ram
neg_dr_eo = 0     ; negovana hodnota dr_eo
with_tails = 1     ; povolit podporu tailovanych zprav
cx_merged_fl = 0     ; code a xdata zcela sloucena
tgt_t89c51rd2 = 0    ; kod pro t89c51rd2

spn = 1	      ; cislo serioveho portu

.if 1^spn
; standardni seriovy port 8051
pcon=0x087 ;data 
sbuf_u=sbuf ;data 
scon_u=scon ;data 
sm2_u=sm2 
ren_u=ren 
tb8_u=tb8 
rb8_u=rb8 
ti_u=ti 
pcon_u=pcon ;data 	; smod bit
ri_u=ri 
es_u=es 
rxd_u=rxd 
txd_u=txd 
sint_u=sint ;code 
dr_eo=p3.5     ; aktivni v dle neg_dr_eo
;dr_eo   bit   p3.4    ; aktivni v dle neg_dr_eo
;dr_ei   bit   p1.6    ; aktivni v 0 pro rs-486
                      ; aktivni v 1 pro rs-232
.else 
; port 1 mikrokontroleru ti msc1210
.if for_sdcc
.include "reg1210.ins"
.else 
.include "reg1210.inc"
.endif 
sbuf_u=sbuf1 ;data 
scon_u=scon1 ;data 
sm2_u=sm21 
ren_u=ren1 
tb8_u=tb81 
rb8_u=rb81 
ti_u=ti1 
ri_u=ri1 
pcon_u=eicon ;data 	; smod bit
es_u=es1 
rxd_u=rxd1 
txd_u=txd1 
sint_u=sint1 ;code 
dr_eo=p3.5     ; aktivni v dle neg_dr_eo
.endif 


end_ram=0x0f000 ;xdata   ; konec pameti
leng_ib =   4       ; delka vstupniho bufferu v 256 bytu
leng_ob =   4       ; delka vystupniho bufferu
s_speed =   3       ; bd=osc/12/16/s_speed pro 19200 pri 11.0592 mhz
s_frln  =   1024    ; maximalni delka ramce

.if for_sdcc
sint=0x00023 ;code 	      ; sdcc nema hodnotu definovanu
.endif 

.if vector_fl
  .globl vec_set ; code
  v_ul_add =   0x1e
  v_ul_fnc =   0x26
.else 
;  .globl ul_r_bu,ul_r_co ; code
.endif 

.globl ul_idb,ul_ide ; code     ; jmeno modulu

.globl  ul_ib_l,ul_ob_l
.globl  ul_fnc,_ul_fncc,ul_init
.globl  ul_str,ul_err,ulf_err,ulf_ine,ulf_kmp,ulf_trp
.globl  ul_flg,ul_flh
.globl  s_int,ser_stack_b

.globl  ul_o_op,ul_o_new,ul_wr,ul_o_cl,ul_wrb,ul_o_ln,ul_wrc,ul_o_abrt
.globl  ul_i_op,ul_rd,ul_i_cl,ul_rdb,ul_i_ln,ul_i_l,ul_i_ne

; funkce volane z rutin zpracovani prikazu
.globl  ack_cmd,snd_beb,snd_chc,snd_end,s_waitd,nak_cmd
.globl          rec_beg,rec_chr,rec_end,rec_cme
.globl  rec_bi, snd_bi, rec_bx, snd_bx, snd_bc, s_r0fb, s_eqp
; data
.globl  ul_adr,ul_sa,ul_da,ul_cmd,ul_st
.globl  ul_isad,ul_idad,ul_icom,ul_ist,ul_ista
.globl  ulf_rs,ulf_nb,ulf_na

.if tgt_t89c51rd2
fcon=0x0d1 ;data 
saddr=0x0a9 ;data 
.endif 

;ser___c segment code
;ser___x segment xdata
;ser___b segment data bitaddressable
;ser_stack segment data

.if 1^vector_fl
.area CSEG_A (ABS,CODE)
.org sint_u
        ljmp   s_int
        ljmp   ul_fnc

.area CSEG_A (ABS,CODE)
.org timer1
        ljmp   s_int
.endif 

.area XSEG (XDATA);ser___x

;********************************************************************
; datova oblast site ulan

; jen pro debugging, jinak zakomentovat
.globl ul_hbib,ul_spd,ul_sta
.globl p_ndb,h_bib,h_eib,p_npd,p_aid,p_eid
.globl p_aob,h_bob,h_eob,p_nob,p_aod

ul_hbib:.ds    1  ;    vyssi cast adresy zacatku ib
ul_ib_l:.ds    1  ;    delka ib v 256
ul_ob_l:.ds    1  ;    delka ob v 256
ul_spd: .ds    1  ;    delitel frekvence 57.6 khz
ul_adr: .ds    1  ;    vlastni adresa
ul_cmd: .ds    1  ;    prave prenaseny prikaz (irq)
ul_sa:  .ds    1  ;    adresa vysilace (irq)
ul_da:  .ds    1  ;    adresa prijimace (irq)
ul_st:  .ds    1  ;    status zpravy (irq)
ul_sta: .ds    1  ;    stamp zpravy (irq)
ul_frln:.ds    2  ;    maximalni delka ramce

; prijem dat

p_ndb:  .ds    2  ;r23 ukazatel na pocatek prijimaneho bloku (irq)
h_bib:  .ds    1  ;r4  pocatek vstupniho buferu
h_eib:  .ds    1  ;r5  konec vstupniho buferu
p_npd:  .ds    2  ;r67 ukazatel na pocatek 1. nezprac. bloku (fnc)
p_aid:  .ds    2  ;    ukazatel na zpracovavana data (fnc)

; vysilani dat

p_aob:  .ds    2  ;r23 ukazatel na vysilany blok	(irq)
h_bob:  .ds    1  ;r4  pocatek vystupniho buferu
h_eob:  .ds    1  ;r5  konec vystupniho buferu
p_nob:  .ds    2  ;r67 ukazatel na zapisovany blok (fnc)
p_aod:  .ds    2  ;    ukazatel na zapisovana data (fnc)
.if with_tails
p_aot:	.ds    2  ;    ukazuje na prvni blok tailovane skupiny (fnc)
p_to:	.ds    2  ;    pomocny ukazatel do ob pro taily (irq)
p_ti:	.ds    2  ;    pomocny ukazatel do ib pro taily (irq)
.endif 

p_eid:  .ds    2  ;    zjisteny konec vstupnich dat
ul_isad:.ds    1  ;    promiskuitni sadr zpracovavaneho bloku
ul_idad:.ds    1  ;    da zpracovavaneho bloku
ul_icom:.ds    1  ;    command zpracovavaneho bloku
ul_ist: .ds    1  ;    status zpracovavaneho bloku
ul_ista:.ds    1  ;    stamp - jedinecne cislo zpravy

proc_buf:.ds   17 ; buffer prikazu proceed
proc_bufe:

.area UL_BSEG (ABS,DATA,CON)  ;ser___b
.org 0x20
ul_bseg_s:
.org 0x24

ul_flg: .ds    1
ulf_er0=((ul_flg-ul_bseg_s)*8)+0   ; chyba pri vysilani
ulf_er1=((ul_flg-ul_bseg_s)*8)+1   ; chyba pri vysilani
ulf_err=((ul_flg-ul_bseg_s)*8)+2   ; chyba pri vysilani
ulf_sn=((ul_flg-ul_bseg_s)*8)+3   ; ulan vysila - master mode
ulf_rs=((ul_flg-ul_bseg_s)*8)+4   ; potreba vysilat
ulf_nb=((ul_flg-ul_bseg_s)*8)+5   ; zbernice neobsazena
ulf_na=((ul_flg-ul_bseg_s)*8)+6   ; mazano kazdou akci
ulf_trp=((ul_flg-ul_bseg_s)*8)+7   ; jiny duvod preruseni

ul_flh: .ds    1
ulf_ine=((ul_flh-ul_bseg_s)*8)+0   ; zprava presunuta do vstupni fronty
ulf_kmp=((ul_flh-ul_bseg_s)*8)+1   ; keyboard macro in progress
ulf_rec=((ul_flh-ul_bseg_s)*8)+2   ; zprava urcena pro tuto stanici
ulf_arc=((ul_flh-ul_bseg_s)*8)+3   ; prijimat vsechny zpravy na sbernici
.if with_tails
ulf_owt=((ul_flh-ul_bseg_s)*8)+4   ; pripravuje se vystupni zprava s tailem
ulf_tpf=((ul_flh-ul_bseg_s)*8)+5   ; zpracovava se tailovani pod prerusenim
ulf_tmp=((ul_flh-ul_bseg_s)*8)+6   ; pomocny
.endif 

.area DSEG (DATA) 
ser_stack:
ser_stack_b:
        .ds    8

.area XSEG_A (ABS,XDATA)
.org end_ram-256*leng_ib-256*leng_ob

beg_ib: .ds    0x100*leng_ib
end_ib:
beg_ob: .ds    0x100*leng_ob
end_ob:

; rizeni linky - prikazy
ul_erri =   0x0ff ; ignoruj vse doslo k chybe
ul_err  =   0x07f ; chyba v datech
ul_end  =   0x07c ; konec dat
ul_arq  =   0x07a ; konec dat - vysli ack
ul_prq  =   0x079 ; konec dat - proved prikaz
ul_aap  =   0x076 ; arq + prq
ul_beg  =   0x075 ; zacatek dat

; potvrzovaci zpravy
ul_ack  =   0x019 ; potvrzeni
ul_nak  =   0x07f ; doslo k chybe
ul_wak  =   0x025 ; ted nemohu splnit

; prikazy posilane linkou
; 0x00 .. 0x3f    uloz do bufferu
; 0x40 .. 0x7f    uloz do bufferu bez ack
; 0x80 .. 0x9f    okamzite proved a konci
; a0h .. bfh    proved a prijimej
; c0h .. ffh    proved a vysilej

ul_res  =   0x080 ; znovu inicializuj rs485
ul_sft  =   0x081 ; otestuj velikost volne pameti
ul_sid  =   0x0f0 ; predstav se
ul_sfi  =   0x0f1 ; vysli velikost volne pameti v ib
ul_tf0  =   0x098 ; konec krokovani
ul_tf1  =   0x099 ; pocatek krokovani
ul_stp  =   0x09a ; krok
ul_deb  =   0x09b ; dalsi prikazy pro debug
ul_spc  =   0x0da ; vysle pcl pch psw acc

ul_rdm  =   0x0f8 ; cte pamet typu   t t 0b 0b l l
ul_wrm  =   0x0b8 ; zapise do pameti t t 0b 0b l l

; status zprav
ulbf_nore   = 0x040 ; do not try to repeat if error occurs
ulbf_noreb  = 6
ulbf_tail   = 0x020 ; message has tail frame
ulbf_tailb  = 5
ulbf_rec    = 0x010 ; request receiption of block
ulbf_recb   = 4
ulbf_fail   = 0x008 ; message cannot be send - error
ulbf_proc   = 0x004 ; message succesfull send
ulbf_aap    = 0x003 ; request imediate proccessing of frame by receiver station with acknowledge
ulbf_prq    = 0x002 ; request imediate proccessing of frame by receiver station
ulbf_arq    = 0x001 ; request imediate acknowledge by receiving station
ulbf_end    = 0x000 ; no acknowledge or processing request

.area CSEG (CODE) ;ser___c

; switch to bank 1 
ar0=0+8*1 
ar1=1+8*1 
ar2=2+8*1 
ar3=3+8*1 
ar4=4+8*1 
ar5=5+8*1 
ar6=6+8*1 
ar7=7+8*1 


; cekani na znak r0 - 1 znaku  jinak s_err
wtf_chr:djnz  r0,snd_spc
v2_err: ljmp   s_err

; cekani po dobu 1 znaku
snd_spt:clr   ti_u
        jnb   txd_u,s_ret
        clr   ac
        ret

; cekani po dobu 1 znaku
snd_spc:clr   sm2_u
        setb  ren_u
snd_sp1:
.if neg_dr_eo
        clr   dr_eo           ; *** ceka 1 znak
.else 
        setb   dr_eo          ; *** ceka 1 znak
.endif 
        clr   a
        sjmp  snd_ch2

snd_ctr:setb  c               ; *** send control character
        sjmp  snd_ch1
snd_chc:xrl   ar1,a           ; *** send data + add chk sum
        inc   r1
snd_chr:clr   c               ; *** send data character
snd_ch1:mov   tb8_u,c         ; acc .. character
        clr   ren_u
.if neg_dr_eo
        setb   dr_eo
.else 
        clr   dr_eo
.endif 
snd_ch2:mov   sbuf_u,a
        clr   ti_u
        sjmp  s_ret

rec_ctr:setb  c               ; *** receive control character
        .db    0x74             ; mov a,#d8
rec_chr:clr   c               ; *** receive character
rec_ch1:mov   sm2_u,c         ; acc .. rec. char
.if neg_dr_eo
        clr   dr_eo           ; cy  .. rb8_u
.else 
        setb  dr_eo           ; cy  .. rb8_u
.endif 
        setb  ren_u
        clr   ti_u

rs232:
s_ret:  jb    ulf_trp,s_int_t
s_reti: mov   a,sp
        xch   a,ser_stack
        mov   sp,a
        pop   dph
        pop   dpl
        pop   acc
        pop   psw
        reti

; *************************************
; interupt ti_u nebo ri_u
; *************************************

s_int  :push  psw
        push  acc
        push  dpl
        push  dph
        mov   a,sp
        xch   a,ser_stack
        jz    s_error
        mov   sp,a

s_int_t:mov   psw,#ar0; banka1
;       jb    dr_ei,rs232
        mov   c,rb8_u         ; errata 89c51rd2, point 6 !!!!!!
        clr   rb8_u           ; nutno dodrzet !!!!!!!!!!!!!!!!!
        jbc   ri_u,s_int_2
        jnb   ti_u,s_ret
s_int_1:clr   ulf_na
        ret
s_int_2:jb    ren_u,s_int_3   ; errata 89c51rd2, point 8
        sjmp  s_reti
s_int_3:setb  ac              ; preruseni od ri_u => ac=1
        clr   ulf_nb
        mov   a,sbuf_u
        jnc   s_int_1
        jnb   acc.7,s_int_1
        setb  ulf_nb
        cjne  a,#ul_erri,s_int_1
s_error:mov   psw,#ar0; banka1
        clr   ulf_sn
        ljmp   s_err

; prijem bloku do xdata

rec_bx: clr   sm2_u           ; r23 .. where
        setb  ren_u           ; r45 .. buffer bot/top
.if neg_dr_eo
        clr   dr_eo           ; r67 .. end of buffer
.else 
        setb   dr_eo          ; r67 .. end of buffer
.endif 
        clr   ti_u            ; r1  =  check sum
rec_bx1:lcall  s_ret           ; cy  =  1 when ended by ctr
        jb    ac,rec_bx2
        clr   ti_u
        sjmp  rec_bx1
rec_bx2:jc    rec_bxr
        lcall  s_rctb1
        jnz   rec_bx1
        clr   c
rec_bxr:ret

; prijem bloku do cdata (t89c51rd2)
.if tgt_t89c51rd2
rec_bc: clr   sm2_u           ; r23 .. where
        setb  ren_u           ; r45 .. buffer bot/top
.if neg_dr_eo
        clr   dr_eo           ; r67 .. end of buffer
.else 
        setb   dr_eo          ; r67 .. end of buffer
.endif 
        clr   ti_u            ; r1  =  check sum
rec_bc1:lcall  s_ret           ; cy  =  1 when ended by ctr
        jb    ac,rec_bc2
        clr   ti_u
        sjmp  rec_bc1
rec_bc2:jc    rec_bcr
        mov   fcon,#0x08       ; zapis do programove pameti
        lcall  s_rctb1
        mov   fcon,#0x00       ; vypni zapis do programove pameti
        jnz   rec_bc1
        clr   c
rec_bcr:ret
.else 
rec_bc=rec_bx ;code 
.endif 

; vyslani bloku z xdata

snd_bx: lcall  snd_spc         ; r23 .. where
snd_bx0:clr   ren_u           ; r45 .. buffer bot/top
.if neg_dr_eo
        setb   dr_eo          ; r67 .. end of buffer
.else 
        clr   dr_eo           ; r67 .. end of buffer
.endif 
        clr   ri_u            ; r1  =  check sum
        clr   tb8_u
snd_bx1:lcall  s_snfb
        clr   ti_u
        jz    snd_bxr
snd_bx2:lcall  s_ret
        jb    ac,snd_bx2
        sjmp  snd_bx1
snd_bxr:lcall  s_ret
        clr   ti_u
        ret

.if cx_merged_fl
snd_bc=snd_bx ;code 
.else 
; vyslani bloku z code
snd_bc: lcall  snd_spc         ; r23 .. where
snd_bc0:clr   ren_u           ; r45 .. buffer bot/top
.if neg_dr_eo       ; r67 .. end of buffer
        setb   dr_eo          ; r1  =  check sum
.else 
        clr   dr_eo
.endif 
        clr   ri_u
        clr   tb8_u
snd_bc1:lcall  s_snfc
        clr   ti_u
        jz    snd_bcr
snd_bc2:lcall  s_ret
        jb    ac,snd_bc2
        sjmp  snd_bc1
snd_bcr:lcall  s_ret
        clr   ti_u
        ret
.endif 

; prijem bloku do idata

rec_bi: clr   sm2_u           ; r2  .. where
        setb  ren_u           ; x r45 .. buffer bot/top
.if neg_dr_eo
        clr   dr_eo           ; r6  .. end of buffer
.else 
        setb  dr_eo           ; r6  .. end of buffer
.endif 
        clr   ti_u            ; r1  =  check sum
rec_bi1:lcall  s_ret           ; cy  =  1 when ended by ctr
        jb    ac,rec_bi2
        clr   ti_u
        sjmp  rec_bi1
rec_bi2:jc    rec_bir
        push  ar1
        mov   r1,ar2
        mov   @r1,a
        pop   ar1
        xrl   ar1,a
        inc   r1
        inc   r2
        mov   a,r2
        xrl   a,r6
        jnz   rec_bi1
        clr   c
rec_bir:ret

; vyslani bloku z idata

snd_bi: lcall  snd_spc         ; r2  .. where
snd_bi0:clr   ren_u           ; x r45 .. buffer bot/top
.if neg_dr_eo
        setb  dr_eo           ; r6  .. end of buffer
.else 
        clr   dr_eo           ; r6  .. end of buffer
.endif 
        clr   ri_u            ; r1  =  check sum
        clr   tb8_u
snd_bi1:push  ar1
        mov   r1,ar2
        mov   a,@r1
        mov   sbuf_u,a
        pop   ar1
        xrl   ar1,a
        inc   r1
        inc   r2
        mov   a,r2
        xrl   a,r6
        clr   ti_u
        jz    snd_bir
snd_bi2:lcall  s_ret
        jb    ac,snd_bi2
        sjmp  snd_bi1
snd_bir:lcall  s_ret
        clr   ti_u
        ret

; vyslani bloku z sdata

snd_bs: lcall  snd_spc         ; r2  .. where
snd_bs0:clr   ren_u           ; x r45 .. buffer bot/top
.if neg_dr_eo
        setb  dr_eo           ; r6  .. end of buffer
.else 
        clr   dr_eo           ; r6  .. end of buffer
.endif 
        clr   ri_u            ; r1  =  check sum
        clr   tb8_u
snd_bs1:mov   dptr,#proc_buf
        mov   a,#0x0e5         ; mov a,dir
        movx  @dptr,a
        inc   dptr
        mov   a,r2            ; adresa
        movx  @dptr,a
        inc   dptr
        mov   a,#0x022         ; ret
        movx  @dptr,a
        inc   dptr
        .db    0x12             ; call proc_buf
        .dw    proc_buf
        mov   sbuf_u,a
        xrl   ar1,a
        inc   r1
        inc   r2
        mov   a,r2
        xrl   a,r6
        clr   ti_u
        jz    snd_bsr
snd_bs2:lcall  s_ret
        jb    ac,snd_bs2
        sjmp  snd_bs1
snd_bsr:lcall  s_ret
        clr   ti_u
        ret

; prijem bloku do sdata

rec_bs: clr   sm2_u           ; r2  .. where
        setb  ren_u           ; x r45 .. buffer bot/top
.if neg_dr_eo
        clr   dr_eo           ; r6  .. end of buffer
.else 
        setb  dr_eo           ; r6  .. end of buffer
.endif 
        clr   ti_u            ; r1  =  check sum
rec_bs1:lcall  s_ret           ; cy  =  1 when ended by ctr
        jb    ac,rec_bs2
        clr   ti_u
        sjmp  rec_bs1
rec_bs2:jc    rec_bsr
        xrl   ar1,a
        inc   r1
        mov   r3,a
        mov   dptr,#proc_buf
        mov   a,#0x08b         ; mov dir,r3
        movx  @dptr,a
        inc   dptr
        mov   a,r2            ; adresa
        movx  @dptr,a
        inc   dptr
        mov   a,#0x022         ; ret
        movx  @dptr,a
        inc   dptr
        .db    0x12             ; call proc_buf
        .dw    proc_buf
        inc   r2
        mov   a,r2
        xrl   a,r6
        jnz   rec_bs1
        clr   c
rec_bsr:ret

; vyslani konce ramce
; se zakoncenim pro odpoved ul_end

snd_end:mov  a,#ul_end
	sjmp  snd_en1

; vyslani konce ramce
; podle cmd vysle ul_arq, ul_aap nebo ul_end

snd_endst:
	mov   dptr,#ul_st     ; status zpravy
        movx  a,@dptr
        ;jnb   acc.ulbf_noreb,snd_en0
	jnb   acc.6,snd_en0   ; !!!! asx8051 is broken
        orl   ul_flg,#0x003    ; uz neopakovat vysilani
snd_en0:movx  a,@dptr
        anl   a,#3
	add   a,#snd_ent-snd_en1
	movc  a,@a+pc	      ; ul_end,ul_arq,ul_prq,ul_aap
snd_en1:mov   r0,a
        xrl   ar1,a
        inc   r1
        lcall  snd_ctr         ; vyslani zakonceni
        mov   a,r1
        lcall  snd_chr         ; vyslani checksum
        cjne  r0,#ul_end,snd_en3
        ret
snd_en3:cjne  r0,#ul_prq,snd_en4
        ret
snd_en4:mov   r0,#5
snd_en5:lcall  wtf_chr
        jnb   ac,snd_en5      ; prijeti ul_ack
        jc    snd_en6
        cjne  a,#ul_ack,snd_en6
        ret
snd_en6:ljmp   v3_err

snd_ent:.db    ul_end,ul_arq,ul_prq,ul_aap

; vyslani ul_nak pri chybe v nektere z rutin zpracovani prikazu
rec_err:lcall  rec_ctr
        jnb   ac,rec_err
rec_er2:lcall  cmp_end         ; vstup s prijatym znakem v acc a cy
        jc    v3_err
        mov   r1,ar0
        mov   r0,#5
rec_er3:lcall  wtf_chr         ; prijem checksum
        jnb   ac,rec_en3
        mov   r0,ar1
        mov   r1,#ul_nak
        sjmp  rec_en4

; potvrzeni prikazu
ack_cmd:mov   r1,#ul_ack
        sjmp  rec_en4

; nacteni konce ramce
; ret: acc = cmp_end

rec_end:mov   r0,#5
rec_en1:lcall  wtf_chr         ; prijem zakonceni ramce
        jnb   ac,rec_en1
rec_en2:xrl   ar1,a           ; vstup s prijatym znakem v acc a cy
        inc   r1
        lcall  cmp_end         ; nastaveni r0 dle zpravy (0-3)
        jc    v3_err
        mov   dptr,#ul_st     ; nastav status zpravy
        mov   a,r0
        movx  @dptr,a
        lcall  snd_spc         ; prijem checksum
        jb    ac,rec_en3
        lcall  snd_spc
        jb    ac,rec_en3
        lcall  snd_spc
        jb    ac,rec_en3
        lcall  snd_spc
        jb    ac,rec_en3
rec_e3n:lcall  snd_spc
        jb    ac,rec_en3
        sjmp  v3_err
rec_en3:jc    v3_err
        xrl   a,r1
        jb    ulf_rec,rec_en7 ; zprava urcena pro tuto stanici
        mov   r0,#0           ; nastav ukonceni na end (neodpovidat)
rec_en7:mov   r1,#ul_nak
        jnz   rec_en4         ; pri chybe a arq nebo aap vysle nak
        mov   r1,#ul_ack      ; pri   ok  a arq vysle ack
        mov   a,r0            ; pri   ok  a aap je r1=ack ale vysle
        jb    acc.1,rec_en5   ;  az prooceed rutina
rec_en4:mov   a,r0
        jnb   acc.0,rec_en5
        clr   acc.0
        mov   r0,a
        setb  ren_u
        lcall  snd_spt
        lcall  snd_spc
        mov   a,r1
        lcall  snd_chr
rec_en5:cjne  r1,#ul_nak,rec_en6
v3_err: ljmp   s_err
rec_en6:mov   a,r0
        ret

; pocatek ramce bez urceni destignation address

snd_beb:mov   dptr,#ul_cmd    ; vysilany prikaz
        movx  a,@dptr
        anl   a,#0x7f
        mov   r0,a
        mov   a,#ul_beg

; vyslani zacatku ramce
; call: acc  destignation address
;       r0   command

snd_beg:anl   a,#0x07f
        mov   r1,a
        lcall  snd_spt
        lcall  snd_spc
        mov   a,r1
        inc   r1
        lcall  snd_ctr         ; destignation address nebo ul_beg
        mov   dptr,#ul_sa     ; sa adresa
        movx  a,@dptr
        xrl   ar1,a
        inc   r1
        lcall  snd_chr
        mov   a,r0
        xrl   ar1,a
        inc   r1
        ljmp   snd_chr

; nacteni zacatku ramce
; ret:  acc  source address
;       r0   command

rec_beg:mov   r1,#ul_beg      ; cekani ze swait
        mov   r0,#9
rec_be1:lcall  wtf_chr
        jnb   ac,rec_be1
rec_be2:mov   r0,a
        mov   dptr,#ul_sta    ; nuluj stamp
        clr   a
        movx  @dptr,a
rec_bet:jnc   v4_err
        jb    ulf_nb,v3_err   ; cekani z  pwait s r1=0
        mov   a,r0
        mov   dptr,#ul_da     ; uloz da
        movx  @dptr,a
        mov   dptr,#ul_adr    ; porovnej s vlastni adresou
        movx  a,@dptr
        xrl   a,r0
        setb  ulf_rec         ; zprava urcena pro tuto stanici
        jz    rec_be3
        clr   ulf_rec         ; zprava neni urcena pro tuto stanici nebo ul_beg
        jb    ulf_arc,rec_be3 ; prijimat vsechny zpravy
        mov   a,r0
        jz    rec_be3         ; vseobecna adresa
        setb  ulf_rec         ; zprava urcena pro tuto stanici
        cjne  r1,#ul_beg,s_err
        cjne  a,#ul_beg,s_ewait    ; s_beg a swait
rec_be3:mov   ar1,r0          ; **********************
        inc   r1              ; v r1 se bude pocitat chksum
        mov   r0,#5
rec_be4:lcall  wtf_chr         ; prijem sa
        jnb   ac,rec_be4
        jc    s_err
        xrl   ar1,a
        inc   r1
        mov   r0,a            ; r0=sa
rec_be5:lcall  rec_chr
        jnb   ac,rec_be5      ; cekani na  cmd - prikaz
        jc    s_err
        xrl   ar1,a
        inc   r1
        mov   dptr,#ul_cmd
        movx  @dptr,a         ; zapis cmd
        mov   dptr,#ul_sa
        xch   a,r0
        movx  @dptr,a         ; zapis sa
        ljmp   s_popdp

; cekani na konec bloku pri rec_beg a swait
s_ewait:lcall  rec_ctr
        jnb   ac,s_ewait
        lcall  cmp_end
        jc    s_err
        mov   a,r0
        jnb   acc.0,rec_beg
        mov   r0,#5
s_ewai1:lcall  wtf_chr
        jnb   ac,s_ewai1
        jnc   rec_beg
v4_err: ljmp   s_err

; vrati v r0  0..s_end,1..s_arq,2..s_prq,3..s_aap,jinak cy
cmp_end:jnc   cmp_en4
cmp_en0:mov   r0,#0
        cjne  a,#ul_end,cmp_en1
        ret
cmp_en1:inc   r0
        cjne  a,#ul_arq,cmp_en2
        ret
cmp_en2:inc   r0
        cjne  a,#ul_prq,cmp_en3
        ret
cmp_en3:inc   r0
        cjne  a,#ul_aap,cmp_en4
        ret
cmp_en4:setb  c
        ret

; doslo k chybe pri prijmu nebo vysilani

s_err:  mov   sp,#ser_stack
        jnb   ulf_sn,s_waitd
        mov   r0,#3           ; pocet vyslani erri pri normalni chybe
        jnb   f0,s_err_1
        mov   r0,#10          ; pocet vyslani erri pri zavazne chybe
s_err_1:mov   a,#ul_erri
        lcall  snd_ctr
        djnz  r0,s_err_1
s_err_2:inc   ul_flg
        mov   a,ul_flg
        anl   a,#3
        jnz   s_end
        dec   ul_flg
s_err_3:setb  ulf_err
        lcall  get_oba         ; otevreni vystupni zpravy a naplneni promenych
        lcall  get_nxt         ; ul_sa,ul_da, ...
        mov   dptr,#ul_st     ; priznak neplatneho doruceni zpravy
        movx  a,@dptr
        orl   a,#ulbf_fail
        movx  @dptr,a
        lcall  get_iba         ; otevreni vstupni zpravy
        lcall  ib_begr
	jz    s_err_4	      ; ibfull
        lcall  ib_endr         ; zapis zpravy o nulove delce s prametry ul_sa, ...
s_err_4:setb  ulf_ine         ; zprava ve vstupni fronte
	mov   r0,#0x0ff
        sjmp  s_endte

; rutina maze posledni zpravu

s_endt: mov   r0,#0
s_endte:mov   sp,#ser_stack
        anl   ul_flg,#0x0fc
.if neg_dr_eo
        clr   dr_eo
.else 
        setb  dr_eo
.endif 
        lcall  get_oba
s_endt1:lcall  get_nxt
        jnb   acc.7,s_endt3   ; neexistuje
        lcall  set_nxt
.if with_tails
	jc    s_endt1	      ; preskocit na konec tailovane skupiny
.endif 
        lcall  get_nxt
	jb    acc.7,s_end     ; je dalsi zprava
s_endt3:clr   ulf_rs	      ; neni dalsi zprava

; konec vysilani nebo prijmu

s_end:  mov   sp,#ser_stack
        jnb   ulf_sn,s_waitd
        jb    ulf_nb,s_waitd
        lcall  snd_spt
        jb    ac,s_waitd
        lcall  snd_spc
        jb    ac,s_waitd
        lcall  g_madr
        orl   a,#0x80
        lcall  snd_ctr
        setb  ulf_nb

; cekani na komunikaci

s_waitd:mov   sp,#ser_stack
        clr   ti_u
        setb  f0
        setb  ren_u
        setb  txd_u

; cekani na vlastni adresu nebo pozadavek k vysilani

s_wait: clr   ulf_sn
        jnb   ulf_nb,s_wait1  ; nelze vysilat
        jnb   ulf_rs,s_wait1  ; neni potreba vysilat

        setb  ulf_sn
        lcall  g_madr
        mov   r0,a            ; vlastni adresa
        setb  c
        subb  a,r1            ;  - posledni vysilajici
        anl   a,#0x00f         ; token pasing delay
        add   a,#4            ; min 4
        jnb   f0,s_conn1
        add   a,#0x010         ; prodlouzit - byla chyba
s_conn1:mov   r1,a
s_conn2:lcall  snd_spc         ; cekani token pasing delay
        jb    ac,s_wait2
        djnz  r1,s_conn2
        lcall  s_arb           ; arbitrace pristupu
        jb    f0,s_wait1
snd_ob: lcall  get_oba         ; r23 r4 r5 do ob
.if with_tails
	clr   ulf_tpf
.endif 
snd_ob0:lcall  get_nxt         ; r67 pristi zprava
        jb    acc.7,snd_obv   ; neexistuje
        ljmp   s_endt
snd_obv:ljmp   snd_ob1         ; vysli blok

s_wait1:mov   c,ulf_nb
        cpl   c
        lcall  rec_ch1         ; ceka se na znak
s_wait2:clr   ulf_sn
        jnb   ac,s_waitd      ; vybuzovaci ti_u
        mov   r1,a
        jnc   s_waitd         ; neni ridici
        jb    acc.7,s_wait    ; ukonceni vysilani
        mov   r1,#0
        lcall  rec_be2         ; cekani na prikaz v r0
        sjmp  rec_cm1

rec_cmd:lcall  rec_beg         ; cekani na prikaz v r0
rec_cm1:mov   a,r0
        jb    acc.7,rec_cm2
        lcall  get_iba         ; prijem do ib
        lcall  ib_begr
        jz    rec_cmev
        lcall  rec_bx
        jnc   rec_cmev
        lcall  rec_en2
        mov   r1,a
        lcall  ib_endr           ; prepisuje vse krome r1
        mov   a,r1              ; v r0 vraci cmd
        setb  ulf_ine
        jb    acc.1,rec_cm3     ; prq nebo aap
.if vector_fl
        mov   a,#8              ; priznak ib do r0.3
rec_cm3:xch   a,r0
        ljmp   rec_cm8           ; pro skok na uzivatelsky vektor
.else 
        xch   a,r0
;       call  ul_r_bu           ; pro zpracovani zprav aplikaci
.endif 
v1_wait:ljmp   s_waitd

.if 1^vector_fl
rec_cm3:xch   a,r0
        ljmp   rec_cm8
.endif 

rec_cmev:ljmp  rec_cme

rec_cm2:lcall  get_prb         ; prijem do proc_buf
        jz    rec_cmev
        lcall  rec_bx
        jnc   rec_cmev
        lcall  rec_en2
.if 1^vector_fl
        jnb   acc.1,v1_wait
.endif 
        mov   a,r2
        mov   r6,a
        mov   a,r3
        mov   r7,a
        lcall  get_pr1
rec_cm7:lcall  g_cmd
rec_cm8:                      ; prikaz v acc, zakonceni v r0
.if vector_fl
        ljmp   v_ul_add        ; moznost rozsirit funkce uzivatelem

ul_eadd:xch   a,r0
        jb    acc.3,v1_wait
        jnb   acc.1,v1_wait
        xch   a,r0
.else                         ; pro zpracovani prikazu aplikaci
;       call  ul_r_co
.endif                            ; pocatek provadeni internich prikazu
        cjne  a,#ul_sid,pr_cm20

; vysle svoji identifikaci
        lcall  ack_cmd
        lcall  snd_beb
        mov   r2,#< ul_idb; ; vysle svoji identifikaci
        mov   r3,#> ul_idb
        mov   r6,#< ul_ide
        mov   r7,#> ul_ide
pr_cm12:mov   r5,ar4
        lcall  snd_bc          ; snd_bx
pr_cm19:lcall  snd_end	      ; vysli zakonceni ul_end
        sjmp  v1_wait

; test volne pameti
pr_cm20:cjne  a,#ul_sft,pr_cm30
        mov   a,r0
        jnb   acc.0,v1_wait
        lcall  get_iba
        lcall  len_dat
        mov   ar4,r0
        lcall  get_prb
        lcall  s_r0fb
        mov   a,r4
        clr   c
        subb  a,#6
        jnc   pr_cm22
        inc   r1
pr_cm22:clr   c
        subb  a,r0
        push  psw
        lcall  snd_spc
        lcall  s_r0fb
        pop   psw
        mov   a,r1
        subb  a,r0
        mov   a,#ul_nak
        jc    pr_cm24
        mov   a,#ul_ack
pr_cm24:lcall  snd_chr
        ljmp   s_waitd

; krokovani pocatek
pr_cm30:cjne  a,#ul_tf0,pr_cm32
        clr   ulf_trp      ; ukonci krokovani
pr_cm31:lcall  ack_cmd
        ljmp   s_waitd
pr_cm32:cjne  a,#ul_tf1,pr_cm33
        setb  ulf_trp       ; zacne krokovani
        sjmp  pr_cm31
pr_cm33:cjne  a,#ul_stp,pr_cm36
        setb  ti_u          ; provede jednu instrukci
        lcall  s_reti
        clr   ti_u
        clr   a
        sjmp  pr_cm31
pr_cm36:cjne  a,#ul_spc,pr_cm40
        lcall  ack_cmd
        lcall  snd_beb
        mov   a,ser_stack   ; vysle pcl pch psw acc
        inc   a
        mov   r6,a
        add   a,#-6
        mov   r2,a
        lcall  snd_bi
        ljmp   pr_cm19

pr_cm40:cjne  a,#ul_rdm,pr_cm50
; vyslani pameti
        lcall  ack_cmd
        lcall  snd_beb
        lcall  s_prpmm
        cjne  r4,#1,s_rdm1
        lcall  snd_bi
s_rdm1: cjne  r4,#2,s_rdm2
        lcall  snd_bx
s_rdm2: cjne  r4,#4,s_rdm3
        lcall  snd_bs
s_rdm3: cjne  r4,#5,s_rdm4
        lcall  snd_bc
s_rdm4: ljmp   pr_cm19

pr_cm50:cjne  a,#ul_wrm,pr_cm60
; prijem do pameti
        lcall  ack_cmd
        lcall  rec_beg
        cjne  r0,#ul_wrm & 0x7f,rec_cme
        lcall  s_prpmm
        mov   a,#1
        cjne  r4,#1,s_wrm1
        lcall  rec_bi
s_wrm1: cjne  r4,#2,s_wrm2
        lcall  rec_bx
s_wrm2: cjne  r4,#4,s_wrm3
        lcall  rec_bs
s_wrm3: cjne  r4,#6,s_wrm4    ; zapis do code 89c51rd2?
        lcall  rec_bc
        jnz   rec_cme
        lcall  rec_end
.if tgt_t89c51rd2
s_wrm31:
	mov   a,fcon          ; cekej na dozapsani
        anl   a,#0x01
        jnz   s_wrm31
        mov   fcon,#0x050      ; zapisovaci sequence
        mov   fcon,#0x0a0      ; zapisovaci sequence
.endif 	
        ljmp   s_waitd
s_wrm4: jnz   rec_cme
        lcall  rec_end
        ljmp   s_waitd

pr_cm60:cjne  a,#ul_deb,nak_cmd
; ladici prikazy
s_deb:  lcall  s_eqp
        jz    nak_cmd
        lcall  ack_cmd
        mov   dptr,#proc_buf
        movx  a,@dptr
        inc   dptr
        cjne  a,#0x10,s_deb98
        inc   dptr          ; prikaz go xxxx
        inc   dptr
        mov   a,ser_stack   ; zapise  pcl pch
        add   a,#-5
        mov   r1,a
        movx  a,@dptr
        mov   @r1,a
        inc   dptr
        inc   r1
        movx  a,@dptr
        mov   @r1,a
.if tgt_t89c51rd2
        mov   dptr,#ul_adr    ; zalohuj s vlastni adresou pro presmerovani
        movx  a,@dptr         ; na bootloader
        mov   saddr,a
.endif 
s_deb98:ljmp   s_waitd


nak_cmd:mov   r1,#ul_nak
        ljmp   rec_en4

wak_cmd:mov   r1,#ul_wak
        ljmp   rec_en4

rec_cme:lcall  rec_err
        ljmp   rec_cmd

snd_ob1:cjne  r1,#0,snd_ob2
        lcall  snd_beg
        sjmp  snd_ob3
snd_ob2:lcall  snd_beg
        lcall  snd_bx
snd_ob3:lcall  snd_endst	      ; vysli zakonceni podle ul_st
        mov   dptr,#ul_da     ; oprav adresu da
        movx  a,@dptr
        anl   a,#0x07f
        movx  @dptr,a
        mov   dptr,#ul_st     ; status zpravy z predchoziho get_nxt
        movx  a,@dptr
        orl   a,#ulbf_proc    ; nastavit priznak doruceni zpravy
        movx  @dptr,a
.if with_tails
    	;mov   c,acc.ulbf_tailb
    	mov   c,acc.5	      ; !!!! asx8051 is broken
	mov   ulf_tmp,c
        jb    ulf_tpf,snd_ob5
        jc    snd_ob4
.endif 
        lcall  get_iba         ; otevreni vstupni zpravy
	lcall  ib_begr
	jz    snd_obe	      ; ibfull
        lcall  ib_endr         ; zapis zpravy o nulove delce s vysilanymi parametry
snd_obe:setb  ulf_ine         ; zprava ve vstupni fronte
        ljmp   s_endt

.if with_tails
snd_ob4:mov   dptr,#p_ndb
	movx  a,@dptr
	mov   r0,a
	inc   dptr
	movx  a,@dptr
	xch   a,r0
        mov   dptr,#p_ti
	movx  @dptr,a
	inc   dptr
	mov   a,r0
	movx  @dptr,a
snd_ob5:setb  ren_u
.if neg_dr_eo
        clr   dr_eo           ; prepnout na prijem
.else 
        setb   dr_eo          ; prepnout na prijem
.endif 
        mov   dptr,#p_to
	mov   a,r2
	movx  @dptr,a	      ; zapamatovat si pozici v ob
	inc   dptr
	mov   a,r3
	movx  @dptr,a
        lcall  get_iba         ; otevreni vstupni zpravy
        mov   dptr,#p_ti
	movx  a,@dptr
	mov   r2,a
	inc   dptr
	movx  a,@dptr
	mov   r3,a
	lcall  ib_begr
	jz    snd_obf
	mov   c,ulf_tpf
        lcall  ib_endrt        ; zapis zpravy o nulove delce s vysilanymi parametry
snd_obf:setb  ulf_tpf
snd_ob6:jb    ulf_tmp,snd_ob7

	mov   dptr,#p_ti
	movx  a,@dptr
	mov   r2,a
	inc   dptr
	movx  a,@dptr
	mov   r3,a
        mov   dptr,#p_ndb
	movx  a,@dptr
	xch   a,r2
	movx  @dptr,a
	inc   dptr
	movx  a,@dptr
	xch   a,r3
	movx  @dptr,a
	mov   dpl,r2
	mov   dph,r3
	movx  a,@dptr
	orl   a,#0x80	      ; onacit platnost vstupnich dat
	movx  @dptr,a
        setb  ulf_ine         ; zprava ve vstupni fronte
        ljmp   s_endt
	
snd_ob7:
        mov   dptr,#p_to
	movx  a,@dptr
	mov   r2,a
	inc   dptr
	movx  a,@dptr
	mov   r3,a
        mov   dptr,#h_bob
	movx  a,@dptr
	mov   r4,a
	inc   dptr
	movx  a,@dptr
	mov   r5,a
	
	lcall  get_nxt	      ; dalsi ramec, ac=ulbf_recb
	jnb   acc.7,v5_err
	jb    ac,rec_ta1
	ljmp   snd_ob1
	
rec_ta1:mov   ulf_tmp,c	      ; pokracuje dalsim ramcem 
	mov   dptr,#p_to
	mov   a,r6
	movx  @dptr,a	      ; zapamatovat si pozici v ob
	inc   dptr
	mov   a,r7
	movx  @dptr,a

	mov   r1,#ul_beg      ; cekani ze swait
        mov   r0,#9
rec_ta3:lcall  wtf_chr
        jnb   ac,rec_ta3
	mov   r0,a
        lcall  rec_bet         ; prijem tailu
        lcall  get_iba         ; prijem do ib
        mov   dptr,#p_ti
	movx  a,@dptr
	mov   r2,a
	inc   dptr
	movx  a,@dptr
	mov   r3,a
        lcall  ib_begr
        jz    rec_taer
        lcall  rec_bx
        jnc   rec_taer
        lcall  rec_en2
        mov   dptr,#ul_st     ; priznak doruceni zpravy
        mov   a,#(ulbf_rec | ulbf_proc)
	mov   c,ulf_tmp
	;mov   acc.ulbf_tailb,c
	mov   acc.5,c	      ; !!!! asx8051 is broken
        movx  @dptr,a
	setb  c
        lcall  ib_endrt        ; prepisuje vse krome r1
        ljmp   snd_ob6

rec_taer:lcall rec_err
v5_err:	ljmp   s_err
.endif 


; priprava pro cteni a zapis pameti

s_prpmm:mov   dptr,#proc_buf
        mov   r0,#ar2
s_prpm1:movx  a,@dptr
        mov   @r0,a
        inc   r0
        inc   dptr
        cjne  r0,#ar7+1,s_prpm1
        mov   a,r4
        xch   a,r2
        mov   r4,a
        mov   a,r5
        xch   a,r3
        mov   r0,a
        cjne  r4,#9,s_prpm2
        mov   a,#ser_stack
        sjmp  s_prpm4
s_prpm2:cjne  r4,#8,s_prpm3
        mov   a,ser_stack
        inc   a
        sjmp  s_prpm4
s_prpm3:cjne  r4,#7,s_prpm5
        mov   r0,ser_stack
        dec   r0
        mov   a,@r0
        anl   a,#0x18
s_prpm4:add   a,r2
        mov   r2,a
        mov   r4,#1
s_prpm5:mov   ar5,r4

; pricte k r67 r23

ar67r23:mov   a,r6
        add   a,r2
        mov   r6,a
        mov   a,r7
        addc  a,r3
        mov   r7,a
        clr   c
        subb  a,r5
        jc    ar67r6r
        add   a,r4
        mov   r7,a
ar67r6r:ret

; priprav ib pro prijem dat

ib_begr:mov   r0,#7
ib_beg1:clr   a
        lcall  s_acctb
        jz    ib_begg
        djnz  r0,ib_beg1
ib_begg:ret

; zakonceni prijmu do ib
; call: [r23] konec dat
;       ul_sa adresa vysilace
;       ul_da adresa prijemce
; nastavi p_ndb a ukazatel na dalsi blok
; ret:  [r23] zacatek dat
;       [r67] za konec dat
;       r0    prikaz cmd
;       nesmi mazat r1

ib_endr:setb  c			; zprava ma byt oznacena za platnou
	mov   dpl,r2            ; oznacit za prazdny
        mov   dph,r3            ; blok za prijatou zpravou
        clr   a
        movx  @dptr,a
        mov   dptr,#p_ndb
ib_end1:movx  a,@dptr           ; posunout ukazatel na pristi
        xch   a,r2              ; prijimany blok za zpravu
        mov   r6,a
        movx  @dptr,a
        inc   dptr
        movx  a,@dptr
        xch   a,r3              ; r23 na zacatek bloku
        mov   r7,a              ; r67 za konec bloku
        movx  @dptr,a
        mov   dptr,#ul_sa
        movx  a,@dptr
        mov   acc.7,c           ; ulozit sa or 0x80 a tim
        lcall  s_acctb           ; oznacit zpravu za platnou
        mov   dptr,#ul_da
        movx  a,@dptr
        lcall  s_acctb           ; ulozit da
        mov   a,r6
        lcall  s_acctb           ; ulozit ukazatel za zpravu
        mov   a,r7              ; na pristi blok
        lcall  s_acctb
        mov   dptr,#ul_cmd
        movx  a,@dptr
        lcall  s_acctb           ; ulozit cmd zpravy
        mov   dptr,#ul_st
        movx  a,@dptr
        lcall  s_acctb           ; ulozit status zpravy
        mov   dptr,#ul_sta
        movx  a,@dptr
        lcall  s_acctb           ; ulozit stamp zpravy
        ret
.if with_tails
; verze ib_endr pro tailovani pracuje proti p_ti
; vstup cy informuje o stavu oznaceni zpravy za dokoncenou
ib_endrt:
	mov   dpl,r2            ; oznacit za prazdny
        mov   dph,r3            ; blok za prijatou zpravou
        clr   a
        movx  @dptr,a
        mov   dptr,#p_ti
	sjmp  ib_end1
.endif 

; naplni registry pointry do proc_buf
get_prb:mov   r6,#<  proc_bufe ; prijem do proc_buf
        mov   r7,#> proc_bufe
get_pr1:mov   r2,#<  proc_buf
        mov   r3,#> proc_buf
        mov   ar5,r4
        ret

; r67 naplni ukazately na pristi blok
; ul_da,ul_sa,ul_st,ul_cmd
; ret: acc .. da
;      r0  .. command
;      r1  .. 0 pri nulove delce dat
;      cy=1 .. zprava s tailem
;      ac=1 .. zprava typu rec

get_nxt:lcall  s_r0fb	      ; dadr
        mov   a,r0
	clr   c
	jnb   acc.7,get_nx4
        mov   dptr,#ul_da
        movx  @dptr,a
        lcall  s_r0fb	      ; sadr
        mov   a,r0
        mov   dptr,#ul_sa
        movx  @dptr,a
        lcall  s_r0fb	      ; nacist ukazatel za zpravu
        mov   a,r0
        mov   r6,a
        lcall  s_r0fb 
        mov   a,r0
        mov   r7,a
        lcall  s_r0fb	      ; cmd
        mov   a,r0
	mov   r1,a
        mov   dptr,#ul_cmd 
        movx  @dptr,a
        lcall  s_r0fb	      ; st
        mov   a,r0
        mov   dptr,#ul_st
        movx  @dptr,a
	;mov   c,acc.ulbf_recb
	mov   c,acc.4         ; !!!! asx8051 is broken
	mov   ac,c
	;mov   c,acc.ulbf_tailb
	mov   c,acc.5         ; !!!! asx8051 is broken
	push  psw ; nemelo by byt potreba
        lcall  s_r0fb          ; acc - 0 pri nulove delce dat
        xch   a,r1            ; presun do r1, cmd do a
        xch   a,r0	      ; cmd konecne do r0
        mov   dptr,#ul_sta    ; stamp
        movx  @dptr,a
        mov   dptr,#ul_da     ; dadr
        movx  a,@dptr
	pop   psw ; nemelo by byt potreba
get_nx4:ret

; posune ukazatel za aktualni vystupni zpravu
; nesmi menit r0, r1, cy
set_nxt:mov   dptr,#p_aob
        mov   a,r6
        mov   r2,a
        movx  @dptr,a
        inc   dptr
        mov   a,r7
        mov   r3,a
        movx  @dptr,a
        sjmp  s_popdp

; naplni registry pointry do ob

get_oba:mov   dptr,#p_aob
        ljmp   get_ba1

; naplni registry pointry do ib

get_iba:mov   dptr,#p_ndb
get_ba1:mov   r0,#ar2
get_ba2:movx  a,@dptr
        inc   dptr
        mov   @r0,a
        inc   r0
        cjne  r0,#ar7+1,get_ba2
        sjmp  s_popdp

; nacte prikaz cmd
g_cmd:  mov   dptr,#ul_cmd    ; prikaz
        sjmp  g_madr2

; nacte sa
g_sa:   mov   dptr,#ul_sa     ; vlastni sa
        sjmp  g_madr2

; nacte vlastni adresu
g_madr: mov   dptr,#ul_adr    ; vlastni adresa
g_madr2:movx  a,@dptr
s_popdp:ret

; pripojeni mastera ke sbernici
; vstup vlastni adresy v r0

s_arb:  mov   r1,#3+1         ; 3x2 bitu arbitrace
        clr   ulf_nb
s_arb1:
.if neg_dr_eo
        clr   dr_eo
.else 
        setb  dr_eo
.endif 
        jnb   rxd_u,s_arbe
        jb    ac,s_arbe
        clr   txd_u
        lcall  snd_ch1
        setb  txd_u
.if neg_dr_eo
        clr   dr_eo
.else 
        setb  dr_eo
.endif 
s_arb2: lcall  snd_ch2
        jb    ac,s_arb2
        mov   a,r0          ; rotace adresy
        rr    a
        rr    a
        xch   a,r0
        anl   a,#3          ; nejnizsi 2 bity
        inc   a
        mov   r2,a          ; pocet cekani
        djnz  r1,s_arb4
        ret
s_arb3: jb    ac,s_arbe
        lcall  snd_spc
s_arb4: djnz  r2,s_arb3
        sjmp  s_arb1

s_arbe: setb  f0
        ret


len_dat:clr   c             ; delka dat = r67-r23
        mov   a,r6          ; kdyz je vetsi nez 127 bytu
        subb  a,r2          ;       a = 0x80+delka/256
        mov   r0,a          ; jinak a = delka
        mov   a,r7
        subb  a,r3
        jnc   len_da1
        clr   c
        subb  a,r4
        add   a,r5
len_da1:mov   r1,a
        jnz   len_da2
        mov   a,r0
        jnb   acc.7,len_da3
        clr   a
len_da2:orl   a,#0x080
len_da3:ret

s_chksa:xch   a,r1          ; check sum
        xrl   a,r1
        inc   a
        xch   a,r1
        ret

.if 1^cx_merged_fl
s_snfc: mov   dph,r3          ; vyslani znaku z code
        mov   dpl,r2
        clr   a
        movc  a,@a+dptr
        sjmp  s_snfb1
.endif 

s_snfb: mov   dph,r3          ; vyslani znaku z bufferu
        mov   dpl,r2
        movx  a,@dptr
s_snfb1:mov   sbuf_u,a
        xch   a,r1          ; check sum
        xrl   a,r1
        inc   a
        xch   a,r1
        sjmp  s_acct1

s_r0fb: mov   dph,r3          ; naplneni r0 z bufferu
        mov   dpl,r2
        movx  a,@dptr
        mov   r0,a
        sjmp  s_acct1

s_rctb: mov   a,sbuf_u      ; prijem znaku do bufferu
s_rctb1:xch   a,r1          ; check sum
        xrl   a,r1
        inc   a
        xch   a,r1
s_acctb:mov   dph,r3
        mov   dpl,r2
        movx  @dptr,a
s_acct1:
s_incp: inc   r2            ; pripraveni nasledujici adresy
	mov   a,r2
        jnz   s_incp1
        inc   r3
s_incp1:mov   a,r5
        xrl   a,r3
        jnz   s_eqp
        mov   a,r4
        mov   r3,a
s_eqp:  mov   a,r2          ; kontrola prostoru pro data
        xrl   a,r6          ; acc=0 => konec dat nebo preteceni ib
        jnz   s_eqp1
        mov   a,r3
        xrl   a,r7
s_eqp1: ret


vs_fnc1:ljmp   s_fnc1
; *************************************
; inicializace a sluzby rs-485
; *************************************
_ul_fncc:xch   a,r7         ; v r7 je cislo funkce
        mov    r0,a
        xch   a,r7          ; acc nazpet
        sjmp  ul_fnc
ul_init:mov   r0,#0
        mov   r2,#0
ul_fnc: cjne  r0,#0,vs_fnc1
s_fn0:  clr   es_u          ; po nastaveni ryclosti a adresy se
        clr   et1           ; touto funkci spusti rs485
.if neg_dr_eo
        clr   dr_eo
.else 
        setb  dr_eo
.endif 
;       clr   dr_ei
        clr   pt1
        anl   tmod,#0x00f
        orl   tmod,#0x020
        setb  tr1
        mov   dptr,#ul_spd
        movx  a,@dptr
        jnz   s_fn0_0
        lcall  s_fn1_0
s_fn0_0:mov   th1,a
s_fn6_0:mov   scon_u,#0b11010000
        orl   pcon_u,#0b10000000 ; bd = osc/12/16/(256-th1)
        setb  ps
        clr   a             ; vstupni bod pro uzivatelsky baud gen
        mov   dptr,#ul_cmd
        mov   r1,#proc_bufe-ul_cmd
s_fn0_1:movx  @dptr,a
        inc   dptr
        djnz  r1,s_fn0_1
        mov   a,r2          ; zazalohovani autoinicializace
	push  acc
        mov   dptr,#ul_hbib
        mov   r1,#3
        lcall  ldrfdp
        pop   acc
        jnz   s_fn0_2	    ; autoinicializace ? 
        lcall  s_fn3_0
	mov   a,r2
s_fn0_2:mov   dptr,#p_ndb+1
        movx  @dptr,a       ; p_ndb=beg_ib
        inc   dptr
        movx  @dptr,a       ; h_bib=beg_ib
        mov   dptr,#p_npd+1
        movx  @dptr,a       ; p_npd=beg_ib
        inc   dptr
        inc   dptr
        movx  @dptr,a       ; p_aid=beg_ib
        mov   dph,a
        clr   a
        mov   dpl,a
        movx  @dptr,a
        mov   dptr,#h_eib
        mov   a,r2
        add   a,r3
        movx  @dptr,a       ; h_eib=end_ib
        mov   dptr,#p_aob+1
        movx  @dptr,a       ; p_aob=beg_ob
        inc   dptr
        movx  @dptr,a       ; h_bob=beg_ob
        mov   dptr,#p_nob+1
        movx  @dptr,a       ; p_nob=beg_ob
        inc   dptr
        inc   dptr
        movx  @dptr,a       ; p_aod=beg_ob
        mov   dph,a
        clr   a
        mov   dpl,a
        movx  @dptr,a
        mov   a,r2
        add   a,r3
        add   a,r4
        mov   dptr,#h_eob   ; h_eob=end_ob
        movx  @dptr,a
        mov   a,r2
        mov   dptr,#p_eid+1
        movx  @dptr,a       ; p_eid=beg_ib
        mov   dptr,#ul_frln
        mov   a,#<  s_frln
        movx  @dptr,a
        inc   dptr
        mov   a,#> s_frln
        movx  @dptr,a
.if with_tails
        clr   ulf_owt
.endif 
s_fn0_6:clr   a               ; spusteni ulan communication
        mov   ser_stack,a
        mov   ul_flg,a
        mov   ul_flh,a
.if vector_fl
        mov   r4,#sint_u
        mov   dptr,#s_int
        lcall  vec_set
        mov   r4,#v_ul_fnc
        mov   dptr,#ul_fnc
        lcall  vec_set
        mov   r4,#v_ul_add
        mov   dptr,#ul_eadd
        lcall  vec_set
.else 
.if 0            ; test systemu bez slouceneho
v_ul_add =   0x1e      ; code a xdata na systemu se sloucenim
v_ul_fnc =   0x26
.globl vec_set ; code
        mov   r4,#sint_u
        mov   dptr,#s_int
        lcall  vec_set
        mov   r4,#v_ul_fnc
        mov   dptr,#ul_fnc
        lcall  vec_set
.endif 
.endif 
        setb  es_u
        ret

s_fnc1: cjne  r0,#1,s_fnc2  ; nastavi ryclost 57.6 kbd/acc
s_fn1_0:mov   dptr,#ul_spd
        jnz   s_fn1_1
        mov   a,#s_speed
s_fn1_1:cpl   a
        inc   a
        movx  @dptr,a
        mov   th1,a
        ret

s_fnc2: cjne  r0,#2,s_fnc3  ; nastavuje adresu podle acc
        mov   dptr,#ul_adr
        movx  @dptr,a
        ret

s_fnc3: cjne  r0,#3,s_fnc4  ; nastavi pocatek ib na r2
        clr   es_u
        clr   a
        mov   ser_stack,a
        cjne  r2,#0,s_fn3_1 ; delku ib na r3 a delku ob na r4
s_fn3_0:mov   r2,#> beg_ib ; kdyz je r2=0 provede se autoinicializace
        mov   r3,#leng_ib
        mov   r4,#leng_ob
s_fn3_1:mov   dptr,#ul_hbib
        mov   r1,#3
        ljmp   svrbdp

s_fnc4:

s_fnc5:

s_fnc6: cjne  r0,#6,s_fnc7    ; spusti komunikaci s baud generatorem
        clr   es_u            ; pripravenym uzivatelem
.if neg_dr_eo
        clr   dr_eo
.else 
        setb  dr_eo
.endif 
        ljmp   s_fn6_0

s_fnc7: cjne  r0,#7,s_fnc8    ; zapnuti/vypnuti prijimani vsech zprav na sbernici
        clr   ulf_arc
        jz    s_fnc71
        setb  ulf_arc
s_fnc71:ret

s_fnc8:

s_fnc10:cjne  r0,#0x10,s_fnc11
ul_o_op:mov   a,r5	      ; otevre vystupni zpravu pro dadr r4 s com r5
	mov   r6,#ulbf_aap
	jb    acc.7,ul_o_op1
	mov   r6,#ulbf_arq
	jb    acc.6,ul_o_op2
ul_o_op1:cjne r4,#0,ul_o_op3
ul_o_op2:dec  r6	      ; no acknowledge ulbf_ack
ul_o_op3:mov  r7,#0
ul_o_new:		      ; dalsi parametry status r6, stamp r7
	push  dpl	      ; pri promiskuitnim rezimu ulf_arc
        push  dph	      ; je sadr nastaveno na ul_isad
	mov   dptr,#p_nob
        lcall  s_ger23
        lcall  s_pur23
        mov   a,r4            ; dadr
        anl   a,#0x7f
        lcall  ul_wrb0
        mov   dptr,#ul_adr    ; vlastni adresa do sadr
	jnb   ulf_arc,ul_o_op6
        mov   dptr,#ul_isad   ; promiskuitni adresa do sadr
ul_o_op6:movx a,@dptr
        lcall  ul_wrb0
        lcall  ul_wrb0         ; end pointer low
        lcall  ul_wrb0         ; end pointer high
        mov   a,r5            ; command
        lcall  ul_wrb0
        mov   a,r6            ; status
        lcall  ul_wrb0
        mov   a,r7            ; stamp
        lcall  ul_wrb0
	jnb   f0,ul_o_op8
ul_o_abrt1:
ul_o_op7:mov  dptr,#p_nob     ; blokovani ul_o_cl v pripade
        lcall  s_ger23	      ; nespravne otevrene zpravy
        lcall  s_pur23
ul_o_op8:pop  dph
        pop   dpl
        ret

s_ger23:movx  a,@dptr
        mov   r2,a
        inc   dptr
        movx  a,@dptr
        mov   r3,a
        inc   dptr
        ret

s_pur23:mov   a,r2
        movx  @dptr,a
        inc   dptr
        mov   a,r3
        movx  @dptr,a
        inc   dptr
        ret

s_fnc11:cjne  r0,#0x11,s_fnc12
ul_wr:	jb    f0,ul_wrr       ; zapise r45 bytu z @dp - rusi r012345
ul_wr1: mov   a,r4
        orl   a,r5
        jz    ul_wrr
        lcall  ul_wrb
        jb    f0,ul_wrr
        dec   r4
        cjne  r4,#-1,ul_wr1
        dec   r5
        sjmp  ul_wr1
ul_wrr: ret

ul_i23o:inc   r2	      ; posouva ukazatel r23 ve vystupnim
        cjne  r2,#0,ul_i23r   ; cyklickem bufferu a prepisuje ho
        inc   r3	      ; do dp
        mov   dptr,#h_eob
        movx  a,@dptr
        xrl   a,r3
        jnz   ul_i23r
        mov   dptr,#h_bob
        movx  a,@dptr
        mov   r3,a
ul_i23r:mov   dpl,r2
        mov   dph,r3
ul_o_ce:ret

s_fnc12:cjne  r0,#0x12,s_fnc13
ul_o_cl:jb    f0,ul_o_ce      ; uzavre vystupni zpravu
        mov   r6,dpl
        mov   r7,dph
        mov   dptr,#p_aod
        movx  a,@dptr	      ; r45 ukazatel za konec dat
        mov   r4,a
        inc   dptr
        movx  a,@dptr
        mov   r5,a
	mov   dpl,r4
	mov   dph,a
        clr   a
        movx  @dptr,a	      ; nastavi dadr pristiho bloku na 0
        mov   dptr,#p_nob     ; pristi blok bude zacinat na r45
        movx  a,@dptr
        mov   r2,a
	mov   r0,a
        mov   a,r4
        movx  @dptr,a
        inc   dptr
        movx  a,@dptr
        mov   r3,a
	mov   r1,a	      ; zacatek aktualniho bloku do r01 a r23
        mov   a,r5
        movx  @dptr,a
        xrl   a,r3
        jnz   ul_o_c1
        mov   a,r4	      ; vylouceni snahy o ul_o_cl
        xrl   a,r2	      ; bez predchoziho ul_o_op
        jz    ul_o_c6
ul_o_c1:lcall  ul_i23o	      ;preskocit dadr
        lcall  ul_i23o	      ;preskocit sadr
        mov   a,r4
        movx  @dptr,a	      ; ulozi ukazatel na pristi blok
        lcall  ul_i23o
        mov   a,r5
        movx  @dptr,a
.if with_tails
        lcall  ul_i23o
        lcall  ul_i23o	      ; preskocit command
	movx  a,@dptr	      ; status
        ;mov   c,acc.ulbf_tailb
        mov   c,acc.5         ; !!!! asx8051 is broken
        jnc   ul_o_c5
	jb    ulf_owt,ul_o_c5
	mov   dptr,#p_aot     ; ulozit zacatek useku zprav
	mov   a,r0	      ; spojenych tailovanim
	movx  @dptr,a
	inc   dptr
	mov   a,r1
	movx  @dptr,a
	setb  ulf_owt
ul_o_c4:mov   dpl,r6
        mov   dph,r7
	ret
.endif 
ul_o_c5:mov   dpl,r0	      ; pocatek bloku dp=r01
        mov   dph,r1
	movx  a,@dptr
        mov   r4,a
        orl   a,#0x80          ; nastavenim dadr.8 oznaci
        movx  @dptr,a	      ; blok za pripraveny k odeslani
.if with_tails
        jc    ul_o_c4
	jnb   ulf_owt,ul_o_c6
	clr   ulf_owt
        mov   dptr,#p_aot
	movx  a,@dptr
	mov   r0,a
	inc   dptr
	movx  a,@dptr
	mov   dpl,r0
	mov   dph,a
	movx  a,@dptr
        orl   a,#0x80          ; nastavi dadr.8 prvniho bloku
        movx  @dptr,a	      ; tailovaneho useku a tim zpusti
.endif 			      ; jeho zpracovani
ul_o_c6:mov   dpl,r6
        mov   dph,r7
	sjmp  ul_o_st

s_fnc13:cjne  r0,#0x13,s_fnc14
ul_wrb: movx  a,@dptr         ; zapise byte z @dp - rusi r0123
ul_wrb0:mov   r3,a
        mov   r0,dpl
        mov   r1,dph
        jb    f0,ul_wrbe
        mov   dptr,#p_aod
        movx  a,@dptr
        mov   r2,a
        inc   dptr
        movx  a,@dptr
        mov   dph,a
        xch   a,r3
        mov   dpl,r2
        movx  @dptr,a
        inc   r2
        cjne  r2,#0,ul_wrb1
        inc   r3
        mov   dptr,#h_eob
        movx  a,@dptr
        xrl   a,r3
        jnz   ul_wrb1
        mov   dptr,#h_bob
        movx  a,@dptr
        mov   r3,a
ul_wrb1:mov   dptr,#p_aob+1
        movx  a,@dptr
        xrl   a,r3
        jnz   ul_wrb3
        mov   dptr,#p_aob
        movx  a,@dptr
        xrl   a,r2
        jnz   ul_wrb3
        inc   dptr
        movx  a,@dptr
        xrl   a,r3
        jnz   ul_wrb3
ul_wrbe:setb  f0
        mov   dpl,r0
        mov   dph,r1
        ret

ul_wrb3:mov   dptr,#p_aod
ul_wrb4:mov   a,r2
        movx  @dptr,a
        inc   dptr
        mov   a,r3
        movx  @dptr,a
        mov   dpl,r0
        mov   dph,r1
        movx  a,@dptr
        inc   dptr
        ret

ul_o_st:setb  ulf_rs
        clr   es_u
        jnb   ulf_nb,ul_o_su
        jb    ulf_sn,ul_o_su
        setb  ti_u
ul_o_su:setb  es_u
	ret

s_fnc14:cjne  r0,#0x14,s_fnc15
ul_o_ln:mov   r0,dpl
        mov   r1,dph
        mov   dptr,#p_aod
        movx  a,@dptr
        mov   r2,a
        inc   dptr
        movx  a,@dptr
        mov   r3,a
        mov   dptr,#p_aob
        setb  c
        movx  a,@dptr
        subb  a,r2
        mov   r2,a
        inc   dptr
        movx  a,@dptr
        inc   dptr
        ljmp   ul_o_l1

s_fnc15:cjne  r0,#0x15,s_fnc16
.if vector_fl
ul_wrc: ljmp   ul_wr
.else 
ul_wrc: jb    f0,ul_wrcr      ; zapise r45 bytu z code @dp
ul_wrc1:mov   a,r4            ; rusi r012345
        orl   a,r5
        jz    ul_wrcr
        clr   a
        movc  a,@a+dptr
        lcall  ul_wrb0
	jb    f0,ul_wrcr
        dec   r4
        cjne  r4,#-1,ul_wrc1
        dec   r5
        sjmp  ul_wrc1
ul_wrcr:ret
.endif 

s_fnc16:cjne  r0,#0x16,s_fnc17 ; abors msg
ul_o_abrt:push dpl
        push  dph
	ljmp   ul_o_abrt1

s_fnc17:

s_fnc23:cjne  r0,#0x23,s_fnc20
ul_rdb: mov   r0,dpl          ; nacte byte na @dp - rusi r0123
        mov   r1,dph
        jb    f0,ul_wrbe
        mov   dptr,#p_aid
        movx  a,@dptr
        mov   r2,a
        inc   dptr
        movx  a,@dptr
        mov   r3,a
        mov   dptr,#p_eid
        movx  a,@dptr
        xrl   a,r2
        jnz   ul_rdb1
        inc   dptr
        movx  a,@dptr
        xrl   a,r3
        jz    ul_wrbe
ul_rdb1:mov   dpl,r2
        mov   dph,r3
        movx  a,@dptr
        mov   dpl,r0
        mov   dph,r1
        movx  @dptr,a
        inc   r2
        cjne  r2,#0,ul_rdb2
        inc   r3
        mov   dptr,#h_eib
        movx  a,@dptr
        xrl   a,r3
        jnz   ul_rdb2
        mov   dptr,#h_bib
        movx  a,@dptr
        mov   r3,a
ul_rdb2:mov   dptr,#p_aid
        ljmp   ul_wrb4

s_fnc20:cjne  r0,#0x20,s_fnc21
ul_i_op:push  dpl             ; otevre vstupni zpravu : vyplni
        push  dph             ; ul_isad,ul_idad,ul_icom,ul_ist,ul_ista
        jb    f0,ul_i_or
        mov   dptr,#p_npd
        lcall  s_ger23
        lcall  s_pur23
	inc   r3
	mov   dptr,#p_eid
	lcall  s_pur23
        mov   dptr,#ul_isad
        lcall  ul_rdb          ; sadr
	mov   r4,a
        jnb   acc.7,ul_i_oe
        mov   dptr,#ul_isad
        anl   a,#0x7f	      ; odmaskuj nejvyssi bit
	mov   r4,a	      ; drive bylo se 7 bitem
        movx  @dptr,a
        mov   dptr,#ul_idad   ; dadr
        lcall  ul_rdb
        anl   a,#0x7f	      ; zrusit nejvyssi bit
	mov   r7,a
        mov   dptr,#p_eid     ; konec dat
        lcall  ul_rdb
        lcall  ul_rdb
        mov   dptr,#ul_icom   ; command
        lcall  ul_rdb
	mov   r5,a
        mov   dptr,#ul_ist    ; status
        lcall  ul_rdb
	mov   r6,a
        mov   dptr,#ul_ista   ; stamp
        lcall  ul_rdb
	xch   a,r7
	mov   r3,a
ul_i_or:pop   dph
        pop   dpl
        ret

ul_i_oe:setb  f0
        mov   dptr,#p_npd
        lcall  s_ger23
        mov   dptr,#p_eid
        lcall  s_pur23
        sjmp  ul_i_or

s_fnc21:cjne  r0,#0x21,s_fnc22
ul_rd:	jb    f0,ul_rdr       ; nacte r45 bytu na @dp - rusi r012345
ul_rd1: mov   a,r4
        orl   a,r5
        jz    ul_rdr
        lcall  ul_rdb
        jb    f0,ul_rdr
        dec   r4
        cjne  r4,#-1,ul_rd1
        dec   r5
        sjmp  ul_rd1
ul_rdr: ret

s_fnc22:cjne  r0,#0x22,s_fnc24
ul_i_cl:mov   r6,dpl          ; uzavre vstupni zpravu
        mov   r7,dph
        jb    f0,ul_i_cr
        mov   dptr,#p_eid
        lcall  s_ger23
        mov   dptr,#p_npd
        lcall  s_pur23
ul_i_cr:mov   dpl,r6
        mov   dph,r7
        ret

s_fnc24:cjne  r0,#0x24,s_fnc25
ul_i_ln:mov   r0,dpl
        mov   r1,dph
        mov   dptr,#p_aid
        movx  a,@dptr
        mov   r2,a
        inc   dptr
        movx  a,@dptr
        mov   r3,a
        mov   dptr,#p_eid
        clr   c
        movx  a,@dptr
        subb  a,r2
        mov   r2,a
        inc   dptr
        movx  a,@dptr
        mov   dptr,#h_bib
ul_o_l1:subb  a,r3
        mov   r3,a
        jnc   ul_i_l1
        movx  a,@dptr
        xch   a,r3
        clr   c
        subb  a,r4
        mov   r3,a
        inc   dptr
        movx  a,@dptr
        add   a,r3
        mov   r3,a
ul_i_l1:clr   c
        mov   a,r2
        subb  a,r4
        mov   a,r3
        subb  a,r5
        jnc   ul_i_l3
ul_i_l2:setb  f0
        mov   a,r2
        mov   r4,a
        mov   a,r3
        mov   r5,a
ul_i_l3:clr   c
        mov   dptr,#ul_frln
        movx  a,@dptr
        mov   r2,a
        subb  a,r4
        inc   dptr
        movx  a,@dptr
        mov   r3,a
        subb  a,r5
        jc    ul_i_l2
        mov   dpl,r0
        mov   dph,r1
        ret

s_fnc25:cjne  r0,#0x25,s_fnc26   ; pocet byte do konce zpravy (delka zpravy)
ul_i_l: mov   r0,dpl
        mov   r1,dph
        mov   dptr,#p_aid
        movx  a,@dptr
        mov   r4,a
        inc   dptr
        movx  a,@dptr
        mov   r5,a
        mov   dptr,#p_eid
        clr   c
        movx  a,@dptr
        subb  a,r4
        mov   r4,a
        inc   dptr
        movx  a,@dptr
        subb  a,r5
        mov   r5,a
        jnc   ul_i_ll
        mov   dptr,#h_bib     ; nutna korekce cyklicnosti 
	movx  a,@dptr	      ; r5=r5+(h_eib-h_bib)
	subb  a,r5	      ; r5=h_eib-(h_bib-r5-1)-1
	mov   r5,a
	inc   dptr
	movx  a,@dptr ; h_eib
	setb  c
	subb  a,r5
	mov   r5,a
ul_i_ll:mov   dpl,r0
        mov   dph,r1
	ret

s_fnc26:cjne  r0,#0x26,s_fnc27 ; je na vstupu zprava
ul_i_ne:jbc   ulf_ine,ul_i_n2
ul_i_n1:clr   cy
	ret
ul_i_n2:mov   dptr,#p_npd
	movx  a,@dptr
	mov   r0,a
	inc   dptr
	movx  a,@dptr
	mov   dph,a
	mov   dpl,r0
	movx  a,@dptr
	jz    ul_i_n1
	setb  ulf_ine
	setb  cy
	ret

s_fnc27:
	ret

; nacte od r2 pres dp r1 registru
ldrfdp :mov   a,psw
        anl   a,#0x018
        mov   r0,a
        inc   r0
ldrfdp1:inc   r0
        movx  a,@dptr
        mov   @r0,a
        inc   dptr
        djnz  r1,ldrfdp1
        ret

; ulozi od r2 pres dp r1 registru
svrbdp :mov   a,psw
        anl   a,#0x018
        mov   r0,a
        inc   r0
s_fn3_2:inc   r0
        mov   a,@r0
        movx  @dptr,a
        inc   dptr
        djnz  r1,s_fn3_2
        ret

ul_str: jbc   es_u,ul_strg
        ret
ul_strg:jnb   ulf_na,ul_stre
        jnb   ulf_nb,ul_strf
        jnb   ulf_rs,ul_strf
        setb  ti_u
ul_strf:setb  ulf_nb
ul_stre:setb  ulf_na
        setb  es_u
        ret

        
