#define _GNU_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include "rs232_fn.h"

#define DEBUG 0
#define HAS_GETOPT_LONG 1

char *msc_sdev="/dev/ttyS0";
int msc_baud=19200;
int msc_waitrep=1000000;
int msc_fd;
int msc_xhz=18432000;

int go_flg=0;
int reset_flg=0;
int break_flg=0;
int upload_flg=0;
int regerase_flg=0;
int dump_flg=0;
int masserase_flg=0;
unsigned long masserase_end=0;

int command=0;
int blockmode=0;
int erase_block=-1;
int mem_type='C';
unsigned long mem_start=0;
unsigned long mem_length=0;
unsigned long go_addr=0;

#define MEM_BUF_LEN 0x10000

unsigned char mem_buf[MEM_BUF_LEN];

int get_hex(char **p, unsigned *v, int chars)
{
  unsigned u=0;
  unsigned char c;
  *v=0;
  while(**p==' ') (*p)++;
  do
  { c=**p;
    if((c>='0')&&(c<='9')) c-='0';
    else if((c>='A')&&(c<='F')) c-=('A'-10);
    else {
      if(chars>=0) return -1;
      *v=u;
      return -chars;
    }
    u<<=4; u+=c;
    (*p)++;
  } while(--chars);
  *v=u;
  return 0;
};

int msc_setup(int fd,int msc_xhz)
{

  int r_usec;
  int r_msec;
  int r_ftcon;

  r_usec=msc_xhz/1000000-1;
  if(r_usec<0) r_usec=0;

  r_msec=msc_xhz/1000-1;
  if(r_msec<0) r_msec=0;

  /* FER msec, FWR usec */
  r_ftcon=(10<<4)+5;

  /* USEC */
  if(rs232_sendf(fd, RS232_SF_ECHO, "RWFB%02X",r_usec&0xff)<0) return -1;
  if(rs232_wait_delimit(msc_fd,">",1000)<0) return -1;

  /* MSEC */
  if(rs232_sendf(fd, RS232_SF_ECHO, "RWFC%02X",r_msec&0xff)<0) return -1;
  if(rs232_wait_delimit(msc_fd,">",1000)<0) return -1;
  if(rs232_sendf(fd, RS232_SF_ECHO, "RWFD%02X",(r_msec>>8)&0xff)<0) return -1;
  if(rs232_wait_delimit(msc_fd,">",1000)<0) return -1;

  /* FTCON */
  if(rs232_sendf(fd, RS232_SF_ECHO, "RWEF%02X",r_usec&0xff)<0) return -1;
  if(rs232_wait_delimit(msc_fd,">",1000)<0) return -1;

  return 0;
}


int analyze_ihex_line(char *line, unsigned long *paddr, int *rec_type, unsigned char **data)
{
  unsigned char *p=NULL;
  unsigned addr, len, csum, v;
  int ret;
  while(*line==' ') line++;
  if(data) {
    *data=NULL;
  }
  if(*(line++)!=':') return -1;
  if(get_hex(&line, &len, 2)<0) return -2; csum=ret=len;
  if(get_hex(&line, &addr, 4)<0) return -3; csum+=(addr&0xff)+((addr>>8)&0xff);
  if(paddr) *paddr=addr;
  if(get_hex(&line, &v, 2)<0) return -4; csum+=v;
  if(rec_type) *rec_type=v;
  if(data) {
    if(!(p=malloc(len))) return -5;
    *data=p;
  }
  while(len--){
    if(get_hex(&line, &v, 2)<0) { ret=-6; goto err_ret;} csum+=v;
    if(p) *(p++)=v;
  }

  if(get_hex(&line, &v, 2)<0) ret=-7;
  else {
    csum+=v;
    if(!(csum&0xff)) return ret;
    ret=-8;
  }
  
  err_ret:
    if(data) {
      if(*data) free(*data);
      *data=NULL;
    }
    return ret;
}

int msc_program_ihex(int fd, FILE *F)
{
  size_t size=0;
  int res;
  int len;
  int ind_fl=0;
  int rec_type;
  unsigned long addr;
  
  char *line=NULL;

  if(rs232_sendf(fd, RS232_SF_ECHO, "L")<0){
    fprintf(stderr,"L command failed\n");
    return -1;
  }

  while((len=getline(&line, &size, F))>0){
    while(len && strchr("\n\r\t ",line[len-1])) len--;
    line[len]=0;
    if(!len) continue;
    
    res=analyze_ihex_line(line, &addr, &rec_type, NULL);
    if(res<0){
      if(ind_fl) fprintf(stderr,"\n");
      fprintf(stderr,"analyze_ihex_line returned %d\n",res);
      if(line) free(line);
      return -1;
    }
    if((rec_type!=0)&&(rec_type!=1)){
      if(ind_fl) fprintf(stderr,"\n");
      fprintf(stderr,"Unknown record \"%s\"\n",line);
      continue;
    }
    
    res=rs232_writem(fd, RS232_SF_ECHO, line, len);
    if(res<0){
      if(ind_fl) fprintf(stderr,"\n");
      fprintf(stderr,"rs232_writem returned %d\n",res);
      if(line) free(line);
      return -1;
    }
      
    res=rs232_wait_delimit(fd,".XTE",1000);
    if(res<0){
      if(ind_fl) fprintf(stderr,"\n");
      fprintf(stderr,"rs232_wait_delimit returned %d\n",res);
      if(line) free(line);
      return -1;
    }
    if(res!='.'){
      if(res=='T') break;
      if(ind_fl) fprintf(stderr,"\n");
      fprintf(stderr,"MSC ERROR: Line %s returned %c\n",line,res);
      ind_fl=0;
      return -2;
    } else {
      fprintf(stderr,".");
      ind_fl=1;
    }
    /*res=rs232_wait_delimit(fd,"",1000);*/
  }

  fprintf(stderr,"\n");
  if(line) free(line);

  if(rs232_wait_delimit(fd,">",1000)<0){
    fprintf(stderr,"msc_program_ihex: final delimiter > not found\n");
    return -1;
  }
  return 0;
}

int msc_masserase(int fd,unsigned long erase_end)
{
  int res;

  if(rs232_sendf(fd, RS232_SF_ECHO, "M%04lX", erase_end)<0){
    fprintf(stderr,"M command failed\n");
    return -1;
  }
  while((res=rs232_recch_wait(fd,2000))==' ');
  if((res!='o')||(rs232_recch_wait(fd,1000)!='k')) {
    fprintf(stderr,"msc_masserase: bad reply\n");
    return -2;
  }

  if(rs232_wait_delimit(fd,">",1000)<0){
    fprintf(stderr,"msc_masserase: final delimiter > not found\n");
    return -1;
  }
  
  return 0;
}

int msc_regerase(int fd, unsigned long mem_start, unsigned long mem_length)
{
  int res;
  unsigned long addr;
  unsigned long mem_end=mem_start+mem_length;
  addr=mem_start&~0x7f;
  while(addr<mem_end){
    if(rs232_sendf(fd, RS232_SF_ECHO, "CP%04lX", addr)<0){
      fprintf(stderr,"CP command failed\n");
      return -1;
    }
    
    while(((res=rs232_recch_wait(fd,2000))=='x')||(res==' '));
    if((res!='o')||(rs232_recch_wait(fd,1000)!='k')) {
      fprintf(stderr,"msc_regerase: bad reply\n");
      return -2;
    }
    
    if(rs232_wait_delimit(fd,">",1000)<0){
      fprintf(stderr,"msc_regerase: final delimiter > not found\n");
      return -1;
    }
  
    addr+=0x80;
  }
  return 0;  
}

int msc_mem_read(int fd, int mem_type, unsigned long start, unsigned long len, char *buff)
{
  int res;
  char rcmd[10];
  char s[16*3+5+1];
  char *p;
  int blen=16;
  int i;
  unsigned val;
  
  strcpy(rcmd,"?R%04lX");
  rcmd[0]=mem_type;
  if((mem_type=='I')||(mem_type=='R'))
    rcmd[4]='2';
  if(mem_type=='R') blen=1;
  if(mem_type=='F') strcpy(rcmd,"F%02lX");
  
  while(len){
    if(rs232_sendf(fd, RS232_SF_ECHO, rcmd, start)<0){
      fprintf(stderr,"%s command failed\n",rcmd);
      return -1;
    }
    i=blen*3+5;
    s[i]=0;
    res=rs232_readm(fd, 1000, s, i);
    if(res!=i){
      fprintf(stderr,"msc_mem_read: rs232_readm returned %d\n", res);
      return -1;
    }
    p=s;
    if((p[0]!=0xd)||(p[1]!=0xa)){
      fprintf(stderr,"msc_mem_read: bad reply start %s\n", s);
      return -1;
    }
    p+=2;
    for(i=0; (i<blen) && len; i++){
      res=get_hex(&p,&val,2);
      if(res<0) {
        fprintf(stderr,"msc_mem_read: can not parse reply %s\n", s);
        return -1;
      }
      *(buff++)=val;
      p++;
      len--;
    }
  }
  return 0;
}


int msc_mem_dump(int fd, int mem_type, unsigned long start, unsigned long len)
{
  int res;
  unsigned long addr=start;
  unsigned char buff[16];
  unsigned char *p;
  int blen=16;
  int cnt;
  
  while(len){
    cnt=(len>blen)?blen:len;
    res=msc_mem_read(fd, mem_type, addr, cnt, buff);
    if(res<0){
      fprintf(stderr,"msc_mem_dump: msc_mem_read returned %d\n", res);
      return -1;
    }
    printf("%c.%04lX:",mem_type,addr);
    addr+=cnt;
    len-=cnt;
    p=buff;
    while(cnt--){
      printf("%02X%c",*(p++),cnt?' ':'\n');
    }
  }
  return 0;  
}



static void
usage(void)
{
  printf("usage: flashmsc <parameters> <send_file>\n");
  printf("  -d, --sdev <name>        name of RS232 device [%s]\n",msc_sdev);
  printf("  -B, --baud <num>         RS232 baudrate [%d]\n",msc_baud);
  printf("  -X, --xtal-hz <num>      target crystal frequency\n");
  printf("  -E, --mass-erase <upto>  erase 0-0x7fff or 0x8000-0x807f\n");
  printf("  -e, --erase		     erase region defined by -s -l\n");
  printf("  -m, --mem-dump           dump memory range defined by -s -l -t\n");
  printf("  -s, --start  <addr>      start address of transfer\n");
  printf("  -l, --length <num>       length of upload block\n");
  printf("  -t, --mem-type <chr>     select memory type X C R I F\n");
  printf("  -g, --go     <addr>      start program from address\n");
  printf("  -V, --version            show version\n");
  printf("  -h, --help               this usage screen\n");

/*
  printf("  -c, --command <num>      numeric command value\n");
  printf("  -b, --blockmode <num>    block size\n");
  printf("  -w, --wait-reply <num>   time to wait for reply in ms\n");
  printf("  -r, --reset              reset before download\n");
  printf("  -b, --break              send communication break character\n");
  printf("  -u, --upload             upload memory block [download]\n");
  printf("  -f, --format <format>    format of data file [intelhex]\n");
*/
}

int main(int argc, char **argv) 
{
  int res;
  FILE *F;
  char ch;
  
  static struct option long_opts[] = {
    { "sdev",  1, 0, 'd' },
    { "baud",  1, 0, 'B' },
    { "command",1,0, 'c' },
    { "blockmode",1,0,'b' },
    { "wait-reply",1,0,'w' },
    { "mass-erase", 1, 0, 'E' },
    { "erase", 0, 0, 'e' },
    { "mem-dump",0, 0, 'm' },
    { "mem-type",1, 0, 't' },
    { "start", 1, 0, 's' },
    { "length",1, 0, 'l' },
    { "go",    1, 0, 'g' },
    { "reset", 0, 0, 'r' },
    { "break", 0, 0, 'k' },
    { "upload",0, 0, 'u' },
    { "format",1, 0, 'f' },
    { "version",0,0, 'V' },
    { "help",  0, 0, 'h' },
    { "xtal-hz",1, 0, 'X' },
    { 0, 0, 0, 0}
  };
  int opt;

 #ifndef HAS_GETOPT_LONG
  while ((opt = getopt(argc, argv, "d:B:c:b:w:E:emt:s:l:g:rkuf:VhD:X:")) != EOF)
 #else
  while ((opt = getopt_long(argc, argv, "d:B:c:b:w:E:emt:s:l:g:rkuf:VhD:X:",
			    &long_opts[0], NULL)) != EOF)
 #endif
  switch (opt) {
    case 'd':
      msc_sdev=optarg;
      break;
    case 'B':
      msc_baud = strtol(optarg,NULL,0);
      break;
    case 'X':
      msc_xhz = strtol(optarg,NULL,0);
      break;
    case 'c':
      if(optarg[0]=='B'){
        command=0;
	break;
      }
      command = strtol(optarg,NULL,0);
      break;
    case 'b':
      blockmode = strtol(optarg,NULL,0);
      break;
    case 'w':
      msc_waitrep = strtol(optarg,NULL,0)*1000;
      break;
    case 'E':
      masserase_end = strtol(optarg,NULL,0);
      masserase_flg=1;
      break;
    case 'e':
      regerase_flg=1;
      break;
    case 'm':
      dump_flg=1;
      break;
    case 't':
      mem_type=toupper(optarg[0]);
      break;
    case 's':
      mem_start = strtol(optarg,NULL,0);
      break;
    case 'l':
      mem_length = strtol(optarg,NULL,0);
      break;
    case 'g':
      go_addr = strtol(optarg,NULL,0);
      go_flg = 1;
      break;
    case 'r':
      reset_flg = 1;
      break;
    case 'k':
      break_flg = 1;
      break;
    case 'u':
      upload_flg = 1;
      break;
    case 'f':
      break;
    case 'V':
      fputs("tohit pre alpha\n", stdout);
      exit(0);
    case 'h':
    default:
      usage();
      exit(opt == 'h' ? 0 : 1);
  }

  /* Open RS232 device */
  if ((msc_fd = open(msc_sdev, O_RDWR | O_NONBLOCK)) == -1) {
    fprintf(stderr,"Cannot open serial device %s\n",msc_sdev);
    return -1;
  }

  /* Set RS232 device mode and speed */
  if(rs232_setmode(msc_fd,msc_baud,0,0)<0){
    printf("Error in rs232_setmode\n");
    close(msc_fd);
    return -1;
  }

  sleep(1);
  tcflush(msc_fd, TCIOFLUSH);

  ch=0xd;
  write(msc_fd,&ch,1);
  ch=0xa;
  write(msc_fd,&ch,1);
  
  if(rs232_wait_delimit(msc_fd,">",1000)<0){
    fprintf(stderr,"Initial delimiter not found\n");
    close(msc_fd);
    return -1;
  }

  sleep(1);
  tcflush(msc_fd, TCIOFLUSH);
  
  if(msc_setup(msc_fd,msc_xhz)<0){
    fprintf(stderr,"Target setup failed\n");
    close(msc_fd);
    return -1;
  }

  if(reset_flg){
  }

  if(masserase_flg){
    if(msc_masserase(msc_fd,masserase_end)<0){
      fprintf(stderr,"Mass Erase failed\n");
      close(msc_fd);
      return -1;
    }
  }

  if(regerase_flg){
    res=msc_regerase(msc_fd, mem_start, mem_length);
    if(res<0){
      fprintf(stderr,"msc_regerase: returned %d\n",res);
      exit(1);
    }
  }

  if(!upload_flg&&(optind<argc)){
    F=fopen(argv[optind++],"r");
    if(F==NULL){
      fprintf(stderr,"Error to open file for reading\n");
      exit(1); 
    }
    if(msc_program_ihex(msc_fd,F)<0){
      fprintf(stderr,"Error to program file\n");
      exit(1); 
    }

    fclose(F);
  
  }else 
  if(upload_flg&&(optind<argc)){
  }

  if(dump_flg){
    res=msc_mem_dump(msc_fd, mem_type, mem_start, mem_length);
    if(res<0){
      fprintf(stderr,"msc_mem_dump: returned %d\n",res);
      exit(1);
    }
  }

  if(go_flg){
  }
  
  return 0;
}
